import axios from 'axios'

class Uploader {
  constructor(editor, uploadUrl, uploadHandler) {
    this.editor = editor
    this.uploadUrl = uploadUrl
    this.uploadHandler = uploadHandler
    this.thumbnailUrl = '//t1.daumcdn.net/thumb/R1280x0/?fname='

    this.execUploadForOption = this.execUploadForOption.bind(this)
  }

  initDom() {
    const $ = this.editor.$
    this.$progressbarNode = $('<span class="bar_progress"></span>')
    this.$fragmentElement = $('<div>')
  }

  getImageItemNode(options, uploadData) {
    const imageNode = this.editor.dom.create('img', { 'src': uploadData.url});
    const figureNode = this.editor.dom.create('figure', { 'data-ke-type': 'image' });
    this.editor.dom.add(figureNode, imageNode)
    if (options.caption) {
      const captionNode = this.editor.dom.create('figcaption')
      captionNode.innerHTML = options.caption
      this.editor.dom.add(figureNode, captionNode)
    }
    return figureNode
  }

  getFileForBlob(blob) {
    const file = new File([blob], id, {type: blob.type});
  }

  execUploadForOption(options) {
    if (options.url) {
      this.uploadForUrl(options)
    } else if (options.file) {
      this.uploadForFile(options)
    }
  }

  uploadForUrl(options) {
    const url = options.url, targetId = options.targetId, callback = options.callback
    axios.get(url, {responseType:"blob"})
      .then(result => {
        if (result.data && result.data.size > 0 && result.data.type) {
          const file = options.shouldUploadAsBlob ? result.data : new File([result.data], 'urlUploadImage', {type: result.data.type});
          options.file = file
          this.uploadForFile(options)
        }
      })
      .catch(res => {
        console.error('image upload error', res)
        if (res.toString().split('Network Error').length > 1) {
          const thumbnailUrl = this.thumbnailUrl + encodeURIComponent(url)
          options.url = thumbnailUrl
          this.uploadForUrl(options, this)
        }
        else {
          callback({options});
        }
      })
  }

  addProgressBar(options) {
    const $localImage = this.editor.dom.$('#' + options.targetId)
    $localImage.addClass('uploading')
    $localImage.before(this.$progressbarNode)
  }

  removeProgressBar() {
    this.$fragmentElement.append(this.$progressbarNode)
    this.$progressbarNode.css('width', 0)
  }

  resetUpload(options) {
    alert('업로드에 실패 하였습니다.')
    this.$fragmentElement.append(this.$progressbarNode)
    this.editor.dom.$('#' + options.targetId).closest('[data-ke-type]').remove()
  }

  uploadForFile(options) {
    const file = options.file
    const targetId = options.targetId
    const callback = options.callback
    options.that = this;

    const uploadCallback = (result) => {
      const resultObj = {
        options: options,
        data: result
      }
      if (options.requireDom) {
        resultObj.dom = this.getImageItemNode(options, result)
      }
      callback(resultObj)
    }

    const progressHandler = (loaded, total) => {
      const percent = Math.round(loaded / total * 100)
      const _this = this
      this.$progressbarNode.css('width', percent + '%')
    }

    if(!options.isInvisible) {
      this.addProgressBar(options)
    }

    if (typeof this.uploadHandler === 'function') {
      this.uploadHandler(file, uploadCallback, progressHandler)
    } else {
      this.uploadFile(file, uploadCallback, progressHandler, options)
    }
  }

  uploadFile(file, uploadCallback, progressHandler, options) {
    const formData = new FormData();
    const _this = this
    formData.append("file", file);

    if (!this.uploadUrl) {
      callback({});
      return;
    }

    const config = {
      onUploadProgress: e => {
        if (progressHandler && typeof progressHandler === 'function') {
          progressHandler(e.loaded, e.total)
        }
      }
    }

    axios.post(this.uploadUrl, formData, config)
      .then(res => res.data)
      .then(result => {
        uploadCallback(result)
      })
      .catch(res => {
        _this.removeProgressBar(options)
        console.error('image upload error', res)
        uploadCallback({});
      })
  }
}

export default Uploader
