import CodeEditorDialog from './components/CodeEditorDialog'
import highlightjs from 'highlightjs'

import './styles/codeblock.css'
import 'codemirror/lib/codemirror.css'
import 'codemirror/addon/dialog/dialog.css'
import 'codemirror/addon/search/matchesonscrollbar.css'

const isCodeBlock = (node) => node && node.nodeName == 'PRE'

export default (editor, pluginUrl) => {

  let dialog = new CodeEditorDialog(editor)
  
  const handleButtonClick = () => {
    dialog.open()
  }

  editor.addCommand('codeblock', handleButtonClick)
  editor.addButton('codeblock', {
    cmd: 'codeblock',
    icon: 'codesample',
    tooltip: '코드블럭',
    stateSelector: 'pre'
  })

  editor.on('PreProcess', (e) => {
    const $ = editor.dom.$
    $('pre[contenteditable=false]', e.node).each((idx, elm) => {
      if (!isCodeBlock(elm)) {
        return
      }
      let $elm = $(elm), code = elm.textContent

      $elm.removeAttr('contentEditable')

      $elm.empty().removeAttr('class').append($('<code></code>').each((idx, elm) => {
        elm.textContent = code
      }))
    })
  })

  editor.on('SetContent', () => {
    const $ = editor.dom.$
    let unprocessedCodeSamples = $('pre')
      .filter((idx, elm) => isCodeBlock(elm))
      .filter((idx, elm) => elm.contentEditable !== "false")

    if (unprocessedCodeSamples.length) {
      editor.undoManager.transact(() => {
        unprocessedCodeSamples.each((idx, elm) => {
          $(elm).find('br').each((idx, elm) => {
            elm.parentNode.replaceChild(editor.getDoc().createTextNode('\n'), elm)
          })

          elm.contentEditable = false
          elm.innerHTML = editor.dom.encode(elm.textContent)
          if (highlightjs && highlightjs.highlightBlock) {
            highlightjs.highlightBlock(elm)
          }
        })
      })
    }
  })

  editor.on('init', () => {
    if (editor.settings.codeblock && editor.settings.codeblock.highlightStyle) {
      let linkElm = editor.dom.create('link', {
        rel: 'stylesheet',
        href: editor.settings.codeblock.highlightStyle
      })

      editor.getDoc().getElementsByTagName('head')[0].appendChild(linkElm)
    }
    
  })
}
