import openFile from '../../../lib/openFile';
import attachFile from './components/attach';
import uploadFile from './components/upload';
import deleteUploadedFile from './components/deleteUploadedFile';
import deleteAttachedFile from './components/deleteAttachedFile';

const plugin = editor => {
  const settings = editor.settings.fileUpload;

  editor.addButton('fileUpload', {
    icon: 'image',
    tooltip: 'upload file',
    onclick() {
      openFile({multiple: true}).then(files => {
        foreachAttachFiles(files);
      })
    },
    stateSelector: 'a:not([data-mce-object],[data-mce-placeholder],figure[data-ke-type="file"])'
  });

  editor.on('KeyDown', e => {
    if ((e.keyCode == 8 || e.keyCode == 46) && editor.selection) {
      const selectedNode = editor.selection.getNode();
      if (selectedNode && selectedNode.nodeName == 'A') {
        const parentFigureElement = editor.dom.getParent(selectedNode, 'figure[data-ke-type="file"]');
        if (parentFigureElement) {
          e.preventDefault();
          deleteUploadedFile(selectedNode, settings, editor);
          deleteAttachedFile(editor, parentFigureElement);
        }
      }
    }
  });

  editor.on('drop', e => {
    if (e && e.dataTransfer && e.dataTransfer.files.length > 0 && e.dataTransfer.types.length == 1 && e.dataTransfer.types[0] == 'Files') {

      const files = e.dataTransfer.files;
      foreachAttachFiles(files);
    }
  });

  const attachedFilePredicate = (element) => {
    return editor.dom.is(element, 'a[data-ke-type="a-file"]');
  }

  editor.addButton('deleteFile', {
    title: 'title test',
    icon: 'image',
    tooltip: 'test',
    cmd: 'deleteFile'
  });

  editor.addCommand('deleteFile', () => {
    const selectedNode = editor.selection.getNode();
    const parentFigureElement = editor.dom.getParent(selectedNode, 'figure[data-ke-type="file"]');

    deleteUploadedFile(selectedNode, settings, editor);
    deleteAttachedFile(editor, parentFigureElement);
  });

  editor.addContextToolbar(attachedFilePredicate, 'deleteFile');

  const foreachAttachFiles = (files) => {
    for(let file of files) {
      if (file.type.indexOf('image/') < 0) {
        const fileId = attachFile(editor, file);
        uploadFile(file, settings, fileUploadCallback, fileId);
      }
    }
  }

  const fileUploadCallback = (fileId, url) => {
    const fileNode = editor.$('#' + fileId);
    const fileUrl = url;

    if(fileNode) {
      fileNode.attr('id', null);
      fileNode.attr('href', url);
      fileNode.attr('data-mce-href', url);
    }
  }
};

export default plugin;
