class Grid {
  constructor(editor) {
    this.editor = editor
    this.imageSelector = 'figure[data-ke-type="image"]'
    this.imageGridSelector = 'figure[data-ke-type="imageGrid"]'
    this.allowClosestItemSelector = 'figure[data-ke-type="imageGrid"], figure[data-ke-type="image"]'
    this.$dragNode = null
    this.dragoverNode = null
    this.guideLinePosition = null
    this.$guideLineNode = editor.$('<span class="line_drop_guide"></span>')
    this.$fragmentElement = editor.$('<div>')
    this.initEvent(editor)
  }

  initEvent(editor) {
    
    editor.on('dragstart', e => {
      const $target = editor.$(e.target)
      if ($target.closest(this.allowClosestItemSelector).length > 0) {
        this.$dragNode = $target
      }
    })

    editor.on('drop', e => {
      e.preventDefault()
      e.stopPropagation()
      if (this.$dragNode && this.$dragNode[0]) {
        this.dropImage(e)
      }
    })

    editor.on('drag', e => {
      e.preventDefault()
      e.stopPropagation()
    })

    editor.on('dragend', e => {
      this.resetSetting()
    })

    editor.on('dragover', e => {
      e.preventDefault()
      e.stopPropagation()
      if (this.$dragNode && this.$dragNode[0]) {
        this.viewGuidLine(e)
      }
    })
  }

  resetSetting() {
    this.$dragNode = null
    this.dragoverNode = null
    this.resetGuideLine()
  }

  resetGuideLine() {
    this.$fragmentElement.append(this.$guideLineNode)
    this.guideLinePosition = null
  }

  calculateImageWidthRatio() {
    const $ = this.editor.dom.$
    const $imageFigure = $(this.imageGridSelector)
    let $images, sumWidthRatio = 0, widthRatioPercent, imageRatio, imageRatioPercent, figureWidth = 1000,
      imagePadding = 20, gapWdith
    for (let i = 0, l = $imageFigure.length; i < l; i++) {
      $images = $('img', $imageFigure[i])
      figureWidth = $imageFigure[i].offsetWidth
      for (let r = 0, rl = $images.length; r < rl; r++) {
        imageRatio = $images[r].naturalWidth / $images[r].naturalHeight
        $images[r].kEditorImageGridRatio = imageRatio
        sumWidthRatio += imageRatio
      }
      widthRatioPercent = 100 / sumWidthRatio
      gapWdith = ($images.length - 1) * imagePadding

      $($imageFigure[i]).removeClass('none_duration')
      
      for (let r = 0, rl = $images.length; r < rl; r++) {
        imageRatioPercent = widthRatioPercent * $images[r].kEditorImageGridRatio
        const imageWidthPixel = imageRatioPercent / 100 * (figureWidth - gapWdith)
        const imageWidthPercent = imageWidthPixel / figureWidth * 100 + '%'
        $images[r].style.width = imageWidthPercent
        $($images[r]).attr('data-mce-style', 'width:' + imageWidthPercent)
      }
      sumWidthRatio = 0
    }
  }

  getGuideNode(position) {
    let type = 'vertical'
    if (position == 'top' || position == 'bottom') {
      type = 'horizontal'
    }
    const removeClassName = 'vertical horizontal'
    this.$guideLineNode.removeClass(removeClassName)
    this.$guideLineNode.addClass(type)
    return this.$guideLineNode
  }

  viewGuidLine(dragoverEvent) {
    const target = dragoverEvent.target
    if (target && target.tagName == 'IMG') {
      const linePosition = this.getGuideLinePosition(dragoverEvent)
      if (this.guideLinePosition != linePosition) {
        const $ = this.editor.dom.$
        this.dragoverNode = target
        this.guideLinePosition = linePosition
        const $guideLineNode = this.getGuideNode(linePosition)
        if (linePosition == 'top') {
          $(target.parentNode).prepend($guideLineNode)
        } else if (linePosition == 'bottom') {
          $(target.parentNode).append($guideLineNode)
        } else if (linePosition == 'left' && this.$dragNode[0] != target) {
          $(target).before($guideLineNode)
        } else if (linePosition == 'right' && this.$dragNode[0] != target) {
          $(target).after($guideLineNode)
        } else {
          this.resetGuideLine()
        }
      }
    } else {
      this.resetGuideLine()
    }
  }

  getGuideLinePosition(dragoverEvent) {
    const target = dragoverEvent.target
    const halfWidth = target.clientWidth / 2
    const guideHeightTop = target.clientHeight / 6
    const guideHeightBottom = target.clientHeight - guideHeightTop
    const offsetX = dragoverEvent.offsetX, offsetY = dragoverEvent.offsetY
    if (offsetY <= guideHeightTop) {
      return 'top'
    } else if (offsetY >= guideHeightBottom) {
      return 'bottom'
    } else if (offsetX <= halfWidth && offsetY >= guideHeightTop) {
      return 'left'
    } else if (offsetX > halfWidth && offsetY < guideHeightBottom) {
      return 'right'
    }
  }

  dropImage(dropEvent) {
    const $ = this.editor.dom.$
    const imageMargin = 20
    const $target = $(dropEvent.target)
    const $dropClosestItemNode = $target.closest(this.allowClosestItemSelector)
    const $dragClosestItemNode = this.$dragNode.closest(this.allowClosestItemSelector)

    if (this.$dragNode[0] != dropEvent.target && (this.guideLinePosition == 'left' || this.guideLinePosition == 'right')) {
      this.$dragNode[0].style.width = '10%'

      if ($dropClosestItemNode[0] != $dragClosestItemNode[0]) {
        $dropClosestItemNode.addClass('none_duration')
        const $img = $('img', $dropClosestItemNode[0])
        $img.each((index, elem) => {
          elem.style.width = elem.clientWidth - (this.$dragNode[0].clientWidth + (imageMargin * $img.length)) / $img.length + 'px'
        })
      }

      if (this.guideLinePosition == 'left') {
        $target.before(this.$dragNode)
      } else if (this.guideLinePosition == 'right') {
        $target.after(this.$dragNode)
      }
    } else if (this.guideLinePosition == 'top' || this.guideLinePosition == 'bottom') {
      this.appendImageItem(this.$dragNode, $dropClosestItemNode, this.guideLinePosition)
    }
    
    this.updateImageFigure()
    this.calculateImageWidthRatio()
  }

  // TODO image plugin에 file과 image url로 이미지 아이템을 template를 만들어 넣는 인터페이스를 만들어서 대치하기
  appendImageItem($imageNode, $dropTargetItem, topBottomWay) {
    const $ = this.editor.dom.$
    const $figureNode = $('<figure data-ke-type="image">')
    $imageNode[0].style.width = 'auto'
    $figureNode.append($imageNode)
    if (this.editor.settings.kImage.is_caption) {
      $figureNode.append($('<figcaption>'))
    }
    if (topBottomWay == 'top') {
      $dropTargetItem.before($figureNode)
    } else if (topBottomWay == 'bottom') {
      $dropTargetItem.after($figureNode)
    }
  }

  updateImageFigure() {
    const $ = this.editor.dom.$;
    $(this.allowClosestItemSelector).each((index, elem) => {
      const $elem = $(elem)
      const $img = $('img', elem)
      if ($img.length == 0) {
        $elem.remove()
      } else if ($img.length == 1 && $elem.attr('data-ke-type') == 'imageGrid') {
        $elem.attr('data-ke-type', 'image')
        $img[0].style.width = 'auto'
      } else if ($img.length > 1) {
        $elem.attr('data-ke-type', 'imageGrid')
        $elem.removeAttr('style')
        $elem.removeAttr('data-mce-style')
      }
    })
  }
}

export default Grid