const register = editor => {
  const imageItemSelector = 'figure[data-ke-type="image"]'
  editor.on('KeyDown', e => {
    const selectedNode = editor.selection.getNode()
    const $selectedNode = editor.$(selectedNode)
    let $imageFigure, $figcaption
    if (!selectedNode) {
      return false
    }
    if (e.keyCode == 8 || e.keyCode == 46) {
      if (selectedNode.nodeName == 'IMG') {
        $imageFigure = $selectedNode.closest(imageItemSelector);
        if ($imageFigure[0]) {
          e.preventDefault()
          e.stopPropagation()
          const imageFigureNextSibling = $imageFigure[0].nextSibling
          $imageFigure.remove()
          if (imageFigureNextSibling) {
            setCursorLocationFirst(imageFigureNextSibling)
          }
          editor.undoManager.add()
        }
      } else if (selectedNode.nodeName == 'FIGCAPTION' && editor.selection.getSel().focusOffset == 0 && editor.selection.getSel().anchorOffset == 0) {
        e.preventDefault()
        e.stopPropagation()
      }
    } else if (e.keyCode == 13) {
      if (selectedNode.nodeName == 'IMG') {
        e.preventDefault()
        e.stopPropagation()
        $imageFigure = $selectedNode.closest(imageItemSelector)
        $figcaption = editor.$('figcaption', $imageFigure[0])
        if($figcaption[0]) {
          const cursorOffset = $figcaption.text().length > 0 ? 1 : 0
          setCursorLocationFirst(selectedNode.nextSibling)
        } else {
          $imageFigure = $selectedNode.closest(imageItemSelector)
          appendAfterNewParagraph($imageFigure)
        }
      } else if (selectedNode.nodeName == 'FIGCAPTION') {
        e.preventDefault()
        e.stopPropagation()
        $imageFigure = $selectedNode.closest(imageItemSelector)
        if ($imageFigure[0]) {
          appendAfterNewParagraph($imageFigure)
        }
      }
    } else if (e.key != 'Meta' && e.key != 'Alt' && e.key != 'Control') {
      if (!(e.key == 'c' && e.metaKey) && !(e.key == 'v' && e.metaKey) && !(e.key == 'x' && e.metaKey) && !(e.key == 'z' && e.metaKey)
        && e.keyCode != 37 && e.keyCode != 38 && e.key != 39 && e.key != 40) {
        if ($selectedNode.attr('data-ke-type') == 'image') {
          $figcaption = editor.$('figcaption', selectedNode)
          if ($figcaption[0]) {
            setCursorLocationFirst($figcaption[0])
          } else {
            appendAfterNewParagraph($selectedNode)
          }
        } else if (selectedNode.tagName == 'IMG') {
          $imageFigure = $selectedNode.closest(imageItemSelector);
          if ($imageFigure[0]) {
            $figcaption = editor.$('figcaption', $imageFigure[0])
            if ($figcaption[0]) {
              setCursorLocationFirst($figcaption[0])
            } else {
              appendAfterNewParagraph($imageFigure)
            }
          }
        }
      }
    }
  })

  const appendAfterNewParagraph = targetElement => {
    const $targetElement = editor.$(targetElement)
    if (!$targetElement.next()[0] || $targetElement.next()[0].nodeName != 'P') {
      $targetElement.after('<p><br></p>')
    }
    setCursorLocationFirst($targetElement.next()[0])
  }

  const setCursorLocationFirst = targetElement => {
    const range = editor.selection.getRng()
    range.setStart(targetElement, 0)
    range.setEnd(targetElement, 0)
    editor.selection.setCursorLocation(targetElement)
    editor.selection.getSel().addRange(range);
  }

  const appendBeforeNewParagraph = targetElement => {
    const $targetElement = editor.$(targetElement)
    $targetElement.before('<p><br></p>')
    setCursorLocationFirst($targetElement.prev()[0])
  }
}

export default {
  register
}
