import attachLocalImage from './components/attachLocalImage'
import openFile from '../../../lib/openFile'
import initPhotos from './components/initPhotos'
import caption from './components/caption'
import ContextToolbar from './components/ui/ContextToolbar'
import FloatAction from './components/FloatAction'
import KeyEvent from './components/KeyEvent'
import Uploader from '../../../lib/Uploader'
import Grid from './components/Grid'

const Delay = tinymce.util.Delay

const plugin = editor => {
  const _editingImageId = 'kEditorPhotosEditingImage'
  const _kImageSettings = editor.settings.kImage
  const contextToolbar = new ContextToolbar(editor)
  const uploader = new Uploader(editor, _kImageSettings.upload_url, _kImageSettings.upload_handler)

  editor.addCommand('KImageUploadUrl', uploader.execUploadForOption)
  editor.addCommand('KImageUploadFile', uploader.execUploadForOption)

  const imageUploadCallback = (resultObj) => {
    if (resultObj && resultObj.options && resultObj.data) {
      const $imageNode = editor.dom.$('#' + resultObj.options.targetId);
      const uploadData = resultObj.data;
      if ($imageNode[0] && uploadData) {
        $imageNode.attr('id', null);
        $imageNode.attr('data-filename', uploadData.name);
        $imageNode.removeClass('uploading')
        $imageNode.attr('src', uploadData.url);
        uploader.removeProgressBar()
        editor.undoManager.add()
      }
    }
    editor.undoManager.ignore(() => {
      editor.execCommand('editImageForPhotosEnd')
    })
  };

  const photoEditCancelCallback = () => {
    editor.undoManager.ignore(() => {
      const imageNode = editor.dom.$('#' + _editingImageId);
      if (imageNode) {
        imageNode.attr('id', null)
      }
      editor.undoManager.ignore(() => {
        editor.execCommand('editImageForPhotosEnd')
      })
    })
  }

  const photosAppSuccessCallback = (resultArr, file) => {
    uploader.uploadForFile({
      file: file, 
      targetId: _editingImageId,
      callback: imageUploadCallback
    })
  }

  const attachImage = (file, settings) => {
    const imageId = attachLocalImage(editor, file, settings);
    uploader.uploadForFile({
      file: file,
      targetId: imageId,
      callback: imageUploadCallback
    })
  }

  const foreachAttachImages = files => {
    for (let i = 0, l = files.length; i < l; i++) {
      if(files[i].type.indexOf('image/') < 0) {
        continue;
      }
      attachImage(files[i], _kImageSettings);
    }
  };

  editor.on('dragover', e => {
    e.preventDefault()
    e.stopPropagation()
  })

  editor.on('drop', e => {
    if (e.target.nodeName == 'TD') {
      e.preventDefault()
      e.stopPropagation()
      return false
    }
    if (e && e.dataTransfer && e.dataTransfer.files.length > 0 && e.dataTransfer.types.length == 1 && e.dataTransfer.types[0] == 'Files') {
      const files = e.dataTransfer.files;
      foreachAttachImages(files);
      e.preventDefault()
      e.stopPropagation()
    } else {
      const $selectedNode = editor.$(editor.selection.getNode())
      if (e.target && $selectedNode[0].nodeName == 'IMG') {
        e.preventDefault()
        e.stopPropagation()
      }
    }
  });

  editor.addButton('kImage', {
    icon: 'image',
    tooltip: '이미지',
    onclick() {
      if (editor.selection.getNode().nodeName != 'TD') {
        openFile({multiple: true, accept: 'image/*'}).then(files => {
          foreachAttachImages(files);
        });
      }
    }
  });

  KeyEvent.register(editor)

  editor.on('init', () => {
    uploader.initDom()

    if(_kImageSettings.is_caption) {
      new caption(editor)
    }
    if (_kImageSettings.is_grid_layout) {
      new Grid(editor)
    }
    if (_kImageSettings.is_image_align) {
      FloatAction.register(editor, contextToolbar)
    }
    if (_kImageSettings.is_image_edit) {
      initPhotos(editor, photosAppSuccessCallback, _editingImageId, photoEditCancelCallback, contextToolbar);
    }
    contextToolbar.register()
  })
};

export default plugin;
