import rm from './safeRemove';

const copyCutHandler = (editor, e, isCopy) => {

  const boundCloneElement = cloneElement.bind({isCopy});

  const finishHandling = (pastebin, tag) => {
    if(tag) {
      pastebin.setAttribute('data-ke-special-inner', tag);
    }
    if(e.clipboardData) {
      e.clipboardData.setData('text/html', pastebin.outerHTML);
      e.clipboardData.setData('text/plain', plainTextGenerator(pastebin));
    }
    else if(window.clipboardData) {
      window.clipboardData.setData('Text', pastebin.outerHTML);
    }
    rm(pastebin);
    if(!isCopy) {
      clonedRng.deleteContents();
    }

    if(nodeToSelect) {
      editor.selection.select(nodeToSelect);
      editor.selection.collapse(true);
    }

    editor.nodeChanged();
  }
  const sel = editor.selection.getSel();
  const rng = sel.getRangeAt(0);
  const pastebin = document.createElement('div');
  pastebin.setAttribute("class", "keditor-inner-content");
  const clonedRng = sel.getRangeAt(0);
  const tdElement = editor.dom.getParent(clonedRng.endContainer, 'td');
  const isCellSelected = tdElement ? tdElement.hasAttribute('data-mce-selected') : null;

  // 지도 등 contentEditable = false인 요소들은 선택이 제대로 안되기 때문에 처리해주는 부분
  let nodeToSelect;
  if(rng.commonAncestorContainer.classList && rng.commonAncestorContainer.classList.contains('mce-offscreen-selection')) {
    const figureDiv = editor.dom.get(rng.commonAncestorContainer.getElementsByTagName('figure')[0].id);
    const clonedFigureDiv = figureDiv.cloneNode(true);
    nodeToSelect = figureDiv.nextSibling;
    clonedFigureDiv.removeAttribute('data-mce-selected');
    pastebin.appendChild(clonedFigureDiv);
    if(!isCopy) {
      rm(figureDiv);
    }
    finishHandling(pastebin);
    return;
  }

  // 표를 처리하는 부분
  if(isCellSelected && clonedRng.startContainer.nodeName === clonedRng.endContainer.nodeName) {
    const tableElement = editor.dom.getParent(clonedRng.endContainer, 'table').childNodes[0];
    const clonedTable = isCopy ? tableElement.cloneNode(true) : tableElement;
    const newParentTable = document.createElement('table');
    const newParentTbody = document.createElement('tbody');
    newParentTable.appendChild(newParentTbody);

    let selectedCellCount = 0, totalCellCount = 0;
    for(let row = 0; row < clonedTable.childNodes.length; row++) {
      let tr = clonedTable.childNodes[row];
      let newTrNode;
      for(let col = 0; col < tr.childNodes.length; col++) {
        totalCellCount++;
        let td = tr.childNodes[col];
        if(td.hasAttribute('data-mce-selected')) {
          selectedCellCount++;
          newTrNode = newTrNode || document.createElement('tr');
          const clonedTd = td.cloneNode(true);
          clonedTd.removeAttribute('data-mce-selected');
          clonedTd.removeAttribute('data-mce-first-selected');
          clonedTd.removeAttribute('data-mce-last-selected');
          newTrNode.appendChild(clonedTd);
          const tdChildNodes = Array.from(td.childNodes);
          tdChildNodes.forEach(child => {
            rm(child);
          });
        }
      }
      if(newTrNode) {
        newParentTbody.appendChild(newTrNode);
      }
    }
    if(selectedCellCount === totalCellCount && !isCopy) {
      nodeToSelect = tableElement.nextSibling;
      rm(tableElement.parentNode)
    }
    newParentTable.style['width'] = "100%";
    newParentTable.style['table-layout'] = "fixed";
    pastebin.appendChild(newParentTable);
    finishHandling(pastebin);
    return;
  }

  // figure'만' 있을 때 처리하는 부분
  if(clonedRng.commonAncestorContainer.nodeName === 'FIGURE') {
    clonedRng.setEnd(clonedRng.endContainer, clonedRng.endContainer.childNodes.length);
    const figureNode = clonedRng.commonAncestorContainer;
    figureNode.removeAttribute('data-mce-selected');
    clonedRng.setStart(figureNode, 0);
    clonedRng.setEnd(figureNode, figureNode.length);
    pastebin.appendChild(boundCloneElement(figureNode));
    finishHandling(pastebin);
    return;
  }

  // 단순 텍스트와 html 처리하는 부분
  if(clonedRng.endContainer.nodeName === 'FIGURE') {
    clonedRng.setEnd(clonedRng.endContainer, clonedRng.endContainer.childNodes.length);
  }
  const htmlFragment = isCopy ? clonedRng.cloneContents() : clonedRng.extractContents();

  if(htmlFragment.childNodes.length === 1 && htmlFragment.childNodes[0].nodeName === '#text') {
    if(e.clipboardData) {
      e.clipboardData.setData('text/plain', htmlFragment.childNodes[0].textContent);
    }
    else {
      window.clipboardData.setData('Text', htmlFragment.childNodes[0].textContent);
    }
    const textNode = htmlFragment.childNodes[0];
    const styleResult = getTextStyle(textNode, editor.dom.getParents(clonedRng.startContainer, 'SPAN, B, S, U'));
    const directParent = styleResult[1], rootParent = styleResult[0];

    if(directParent) {
      directParent.appendChild(textNode); // 예외처리 할 것
      pastebin.appendChild(rootParent);
      finishHandling(pastebin, 'parsable');
    }

    return;
  }
  pastebin.appendChild(boundCloneElement(htmlFragment));
  finishHandling(pastebin);
  return;
}

const plainTextGenerator = node => {
  let plainText = "";
  if(node.nodeType === Node.TEXT_NODE) {
    return node.textContent;
  }
  for(let childNode of node.childNodes) {
    plainText += plainTextGenerator(childNode);
  }
  if(['SPAN', 'B', 'S', 'U'].includes(node.nodeName)) {
    return plainText
  }
  else {
    if(node.className === 'keditor-inner-content') {
      return plainText;
    }
    else {
      return plainText + '\n';
    }
  }
}

const cloneElement = function(element) {
  if(this.isCopy) {
    return element.cloneNode(true);
  }
  return element;
}

const getTextStyle = (textNode, parentNodeList) => {
  const span = document.createElement('span');
  let lastChild;
  parentNodeList.reverse().reduce((res, cur) => {
    const clonedCur = cur.cloneNode(false);
    res.appendChild(clonedCur);
    lastChild = clonedCur;
    return clonedCur;
  }, span);
  return [span.childNodes[0], lastChild];
}

export default copyCutHandler;
