import regex from './regex';
import msWordCleaner from './msWordCleaner';
import htmlParser from './htmlParser';
import bowser from 'bowser';
import rm from './safeRemove';

const htmlHandler = (editor, htmlItem, targetNode, deleteRange) => {

  const del = () => {if(deleteRange) deleteRange();};

  const ua = bowser.name;
  if(regex.isMSWord(htmlItem)) {
    htmlItem = msWordCleaner(htmlItem);
  }

  const originalDiv = document.createElement('div');
  originalDiv.innerHTML = htmlItem;
  for(let i = 0, len = originalDiv.childNodes.length; i < len; i++) {
    if(originalDiv.childNodes[i].nodeName === 'META') {
      originalDiv.removeChild(originalDiv.childNodes[i]);
      break;
    }
  }
  const {cleanedDiv, imgList} = htmlParser(editor, originalDiv, targetNode);
  const children = cleanedDiv.childNodes;
  if(!cleanedDiv) {
    return;
  }

  const sel = editor.selection.getSel();
  const curNode = sel.focusNode;
  const rootParent = editor.dom.getParent(curNode, 'body > *');
  const caretSpan = editor.dom.getParent(sel.focusNode, 'body > * > *');
  if(caretSpan && caretSpan.id === '_mce_caret') {
    rm(caretSpan);
  }

  if(cleanedDiv.attributes && cleanedDiv.attributes['data-ke-special-inner']) {
    const rootStyleParent = editor.dom.getParent(curNode, `${rootParent.nodeName} > *`);

    if(rootStyleParent) {
      const leftSide = document.createRange();
      leftSide.setStart(rootParent, 0);
      leftSide.setEnd(curNode, sel.focusOffset);
      del();
      const split = leftSide.extractContents();
      const x = rootParent.insertBefore(children[0], rootStyleParent);
      rootParent.insertBefore(split, x);

      editor.undoManager.add();
      return;
    }
  }

  const sameStartElement = children[0] && rootParent ? children[0].nodeName === rootParent.nodeName : undefined;
  const sameEndElement = children[children.length-1] && rootParent ? children[children.length-1].nodeName === rootParent.nodeName : undefined;
  const startEndLookup = {sameStartElement, sameEndElement};
  const boundTransact = editor.undoManager.transact.bind(startEndLookup);

  const finishHandling = (uploadedImageList) => boundTransact(() => {
    if(uploadedImageList) {
      uploadedImageList.forEach(img => {
        const imgNode = cleanedDiv.querySelector('#'+img.id);
        if(img.url) {
          imgNode.src = img.url;
          imgNode.removeAttribute("id");
        }
        else {
          rm(imgNode.parentNode);
        }
      })
    }

    if(!cleanedDiv.firstChild) {
      return;
    }
    del();
    let firstElement;
    if(startEndLookup.sameStartElement) {
      firstElement = children[0];
      cleanedDiv.removeChild(children[0]);
      editor.insertContent(firstElement.innerHTML);
    }
    editor.selection.getSel().focusNode;
    editor.insertContent(cleanedDiv.innerHTML);
    editor.nodeChanged();

    const sel = editor.selection.getSel();
    let curNode = sel.focusNode;
    if(curNode.nodeName === '#text') {
      curNode = editor.dom.getParent(curNode, 'body > *');
    }
    if(startEndLookup.sameEndElement && curNode.nextSibling) {
      if(['P', 'H1', 'H2', 'H3'].includes(curNode.nodeName)) {
        while(curNode.nextSibling.nodeName === curNode.nodeName && curNode.nextSibling.childNodes.length) {
          curNode.appendChild(curNode.nextSibling.firstChild);
        }
      }
    }
  });

  if(imgList.length) {
    let len = imgList.length;
    let tailNumber = 0;
    const resultList = [];
    const shouldUploadAsBlob = ['Safari', 'Internet Explorer'].includes(ua);
    const boundImgBulkUploadCallback = imgBulkUploadCallback.bind({imgList, editor, len, finishHandling, resultList});
    imgList.forEach(img => {
      let uploadUrl, targetId;
      uploadUrl = img.src;
      targetId = img.id;
      if(!uploadUrl.startsWith("data:image")) {
        const match = /.*daumcdn.net\/thumb.*?\?fname=(.*)/i.exec(uploadUrl);
        if(match) {
          uploadUrl = decodeURIComponent(match[match.length-1]);
        }
      }
      editor.undoManager.ignore(() => {
        editor.execCommand('KImageUploadUrl', {
          url: uploadUrl,
          targetId,
          isInvisible: true,
          callback: boundImgBulkUploadCallback,
          shouldUploadAsBlob,
        })
      })
    })

    editor.windowManager.open({
      width: 800,
      height: 100,
      classes: 'white-dialog',
      items: [{
        type: 'container',
        html: `<div class='mce-white-body'><p class='mce-white-message'>붙여넣기 및 이미지 업로드 중입니다</p><div class='mce-white-progress-border'><div class='mce-white-progress-percentage'></div></div><p class='mce-white-percentage-number'>0%</p></div>`
      }],
      buttons: [],
    });
  }

  else {
    finishHandling();
  }
  editor.nodeChanged();
}

const imgBulkUploadCallback = function (resultObj) {
  this.resultList.push({
    id: resultObj.options.targetId,
    url: resultObj.data ? resultObj.data.url : undefined,
  });

  this.len--;
  const bar = document.getElementsByClassName('mce-white-progress-percentage')[0];
  const percentText = document.getElementsByClassName('mce-white-percentage-number')[0];
  const percent = (this.imgList.length - this.len) / this.imgList.length * 100;
  bar.style.width = percent + '%';
  percentText.textContent = Math.round(percent) + '%';

  if(this.len === 0) {
    this.finishHandling(this.resultList);
    this.editor.windowManager.close();
  }
}

export default htmlHandler;
