import LinkDialog from "./ui/dialog/LinkDialog";
import generatePanelHtml from "./ui/panel/generatePanelHtml";
import LinkPanel from "./ui/panel/LinkPanel";
import tinymce from 'tinymce'
export default (editor) => {

	const Delay = tinymce.util.Tools.resolve('tinymce.util.Delay');
	const $ = editor.$
	const settings = editor.settings.klink;
	const isPanel = settings && settings.is_panel;

	const onPostRender = (editor) => {
		return (e) => {

			const control = e.control
			const disabledStateSelector = control.settings.disabledStateSelector
			editor.shortcuts.add('meta+k', '', () => {
				if (!control.disabled()) {
					control.showPanel();
				}
			})

			editor.on('nodeChange mouseUp keyUp', Delay.throttle(() => {
				if (!disabledStateSelector) {
					return
				}

				let disabled = false
				editor.selection.getSelectedBlocks().map(node => {
					if (editor.dom.is(node, disabledStateSelector)) {
						disabled = true
					}
				})

				control.disabled(disabled)
			}, 0));
		}
	}

	if (isPanel) {
		const linkPanel = new LinkPanel(editor)

		editor.addButton('klink', {
			icon: 'link',
			type: 'panelbutton',
			tooltip: 'Insert/edit link',
			popoverAlign: ['bc-tl'],
			disabledStateSelector: 'img,figure,iframe,div',
			panel: {
	      role: 'dialog',
				html: generatePanelHtml(),
			},
			onPostRender: onPostRender(editor),
			onshow() {
				linkPanel.open(this.panel)
			},
			onhide() {
				linkPanel.removeEvent()
				editor.focus();
			},
			onclick() {
				window.panel = this.panel;
			},
		})
	}	else {
		const linkDialog = new LinkDialog(editor)

		editor.addCommand('klink', () => {
			linkDialog.open()
		})
		editor.addButton('klink', {
			cmd: 'klink',
			icon: 'link',
			tooltip: 'Insert/edit link',
			stateSelector: 'a'
		})
		editor.shortcuts.add('meta+k', 'open link', 'klink')
	}
}
