import autobind from 'autobind-decorator'
import * as LinkUtils from '../../utils/LinkUtils'

class LinkDialog {
	constructor(editor) {
		this.editor = editor
		this.container = null
	}

	open() {
		this.makeWindow()
		this.setEvent()
		this.setData()
		this.checkStatus()
    this.$url[0].focus()
	}

	makeWindow() {
		let win = this.editor.windowManager.open({
      title: 'Insert/edit link',
      items: [{
        type: 'container',
        html: `<div class="keditor-dialog" style="width:360px; height:350px">
					<form class='keditor-form klink-form'>
						<div class="body">
							<label class="form-field">
								<b class="form-label">URL</b>
								<input ref="url" type="text" class="form-text klink-url" placeholder="http://" />
							</label>

							<label class="form-field">
								<b class="form-label">링크 텍스트</b>
								<input type="text" class="form-text klink-text" placeholder="" />
							</label>

							<label class="form-field">
								<input type="checkbox" class="form-checkbox klink-linknew" checked="checked" />
								<span class="checkbox-text">새창으로 열기</span>
							</label>
						</div>
						<div class="footer">
							<button type="reset" class="btn btn-cancel">취소</button>
							<button type="submit" class="btn btn-default">확인</button>
						</div>
					</form>
        </div>`
      }],
      buttons: []
    })
		win.statusbar.remove()

		this.$form = win.$('.klink-form')
		this.$url = win.$('.klink-url')
		this.$text = win.$('.klink-text')
		this.$linknew = win.$('.klink-linknew')
    this.$btnSubmit = win.$('.btn-default')
    this.$btnCancel = win.$('.btn-cancel')
    this.$window = win.$(window)
    this.$modalBlock = win.$('#mce-modal-block')
		this.win = win
	}

	setEvent() {
		this.$form.on("submit", this.onSubmit)
		this.$form.on("reset", this.onCancel)
		this.$url.on("keydown", this.checkStatus)
    this.$window.on("keydown", this.onWindowKeydown)
    this.$modalBlock.on("click", this.onCancel)
	}

	setData() {
		const linkElm = LinkUtils.getAnchorElement(this.editor)
		const dom = this.editor.dom

		if (linkElm) {
			this.$url[0].value = dom.getAttrib(linkElm, 'href')
			this.$linknew[0].checked = dom.getAttrib(linkElm, 'target') === '_blank'
		}

		this.$text[0].value = LinkUtils.getAnchorText(this.editor.selection, linkElm)
	}

	@autobind
	checkStatus() {
		const value = this.$url[0].value
		if (LinkUtils.isValidHref(value)) {
			this.$btnSubmit.removeAttr("disabled")
		} else {
			this.$btnSubmit.attr("disabled", true)
		}
	}

	@autobind
	onSubmit(e) {
		e.preventDefault()
		const value = this.$url[0].value
		const valid = LinkUtils.isValidHref(value)
		if (!valid) {
			return
		}

		// TODO
		const editor = this.editor
		const href = LinkUtils.makeValidHref(value)
		const linkNew = this.$linknew[0].checked
		const text = this.$text[0].value

		const linkElm = LinkUtils.getAnchorElement(editor)

		let linkAttrs = {
			href: href,
			target: linkNew ? '_blank' : null
		}

		if (linkElm) {
			editor.undoManager.transact(function () {
				editor.focus();

				if (text) {
					if ("innerText" in linkElm) {
						linkElm.innerText = text;
					} else {
						linkElm.textContent = text;
					}
				}

				editor.dom.setAttribs(linkElm, linkAttrs);

				editor.selection.select(linkElm);
				editor.undoManager.add();
			})
		} else {
			if (text) {
				editor.insertContent(editor.dom.createHTML('a', linkAttrs, editor.dom.encode(text)));
			} else {
				editor.execCommand('mceInsertLink', false, linkAttrs);
			}
		}
		this.close()
	}

	@autobind
	onCancel(e) {
		this.close()
	}

	@autobind
	onWindowKeydown(e) {
		let keyCode = e.keyCode
    if (keyCode === 27) {
      this.close()
    }
	}


	close() {
    if (this.win) {
      this.$form.off("submit")
			this.$form.off("reset")
			this.$url.off("change")
			this.$window.off("keydown")
			this.$modalBlock.off("click")
      this.win.close()

      this.$form = null
			this.$url = null
			this.$text = null
			this.$linknew = null
			this.$btnSubmit = null
			this.$btnCancel = null
			this.$window = null
			this.$modalBlock = null
		}
  }
}

export default LinkDialog
