import autobind from 'autobind-decorator'
import * as LinkUtils from '../../utils/LinkUtils';

class LinkPanel {
  constructor(editor, panel) {
    this.editor = editor;
  }

  open(panel) {
    this.rng = this.editor.selection.getRng()

    this.makePanelItem(panel)
    this.setEvent();
    this.setData();
    this.checkStatus();
    this.$url[0].focus();
  }

  makePanelItem(panel) {
    const panelDocument = window.parent.document
    const editor = this.editor

    this.panel = panel;
    this.$btn = editor.$('.klink-linknew', panelDocument);
    this.$url = editor.$('.klink-url', panelDocument);
    this.$submit = editor.$('#klink-submit', panelDocument);
    this.$form = editor.$('.klink-form', panelDocument);
  }

  setEvent() {
    this.$btn.on('click', e => {e.stopPropagation();})
    this.$url.on('keyup', this.checkStatus);
    this.$submit.on('click', this.onSubmit);
    this.panel.on("keydown", this.onWindowKeydown);
		this.$form.on("reset", this.onCancel);
  }

  removeEvent() {
    this.$btn.off('click')
    this.$url.off('keyup')
    this.$submit.off('click')
    this.panel.off("keydown")
    this.$form.off("reset")
  }

  setData() {
    const linkElm = LinkUtils.getAnchorElement(this.editor);
    const dom = this.editor.dom;

    let url = ''
    let checked = true
    if (linkElm) {
      url = dom.getAttrib(linkElm, 'href')
      checked = dom.getAttrib(linkElm, 'target') === '_blank'
    }
    this.$url[0].value = url
    this.$btn[0].checked = checked
  }
  @autobind
  checkStatus() {
		const value = this.$url[0].value
		if (LinkUtils.isValidHref(value)) {
			this.$submit.removeAttr("disabled")
		} else {
			this.$submit.attr("disabled", true)
		}
  }

  @autobind
  onSubmit(e) {
  	e.preventDefault()
  	const value = this.$url[0].value
  	const valid = LinkUtils.isValidHref(value)
  	if (!valid) {
  		return
  	}

  	const editor = this.editor
  	const href = LinkUtils.makeValidHref(value)
  	const linkNew = this.$btn[0].checked

    const linkElm = LinkUtils.getAnchorElement(editor)

    let linkAttrs = {
      href: href,
      target: linkNew ? '_blank' : null
    }

    if (linkElm) {
      editor.undoManager.transact(function () {
        editor.focus();
        editor.dom.setAttribs(linkElm, linkAttrs);
        editor.selection.select(linkElm);
        editor.undoManager.add();
      })
    } else {
      editor.selection.setRng(this.rng)
      editor.execCommand('mceInsertLink', false, linkAttrs);
    }


  	this.panel.hide();
	}

  @autobind
	onCancel(e) {
		this.panel.hide();
	}
}

export default LinkPanel;
