import querystring from 'query-string'

const MAP_TYPE = {
  BASIC: 'basic',
  FULL: 'full'
}

const makeMode = mapType => {
  switch(mapType) {
    case MAP_TYPE.BASIC:
      return 'B'
    case MAP_TYPE.FULL:
    default:
      return 'F'

  }
}

const plugin = (editor) => {
  let targetUrl = editor.settings.maps.attach_map_url || ""
  let mapType = editor.settings.maps.map_type || 'data'

  editor.addButton('maps', {
    icon: 'maps',
    tooltip: '지도',
    onclick: () => {
      window.open('https://attach.map.daum.net/?' + querystring.stringify({
        mode: makeMode(mapType),
        appendType: 'S',
        target: targetUrl
      }), "attach_map", "width=900px,height=600px")
    }
  })

  editor.addCommand('attachMap', (data) => {
    editor.undoManager.transact(() => {
      const win = editor.getWin()
      const id = `maps_${Date.now()}`
      editor.insertContent(editor.dom.createHTML('figure', {
        "id": id,
        'data-ke-type': 'map',
        'data-maps-data': querystring.stringify(data),
        'style': 'width:490px;height:360px;',
        'contentEditable': false
      }))
      editor.nodeChanged()
      win.loadMap(id, JSON.parse(data.mapInfo))
    })
  })

  // 플러스맵 프리뷰를 위해 script를 로드한다.
  editor.on('init', () => {
    const win = editor.getWin()
    const $ = editor.dom.$
    win.onLoadMap = () => {
      $('[data-maps-data]').each((idx, elm) => {
        const $elm = $(elm)
        const data = querystring.parse($elm.attr('data-maps-data'))
        const id = $elm.attr('id')
        win.loadMap(id, JSON.parse(data.mapInfo))
      })
    }

    const $head = $('head')
    const mapScript = editor.dom.create('script', {
      type: 'text/javascript',
      src: 'https://spi.maps.daum.net/imap/map_js_init/v3.js?autoload=false'
    })
    mapScript.onload = () => {
      const mapLoader = editor.dom.create('script', {
        type: 'text/javascript'
      })
      mapLoader.innerHTML = `
      daum.maps.load(onLoadMap)

      function loadMap(id, mapInfo) {
        var mapContainer = document.getElementById(id);
        var markers = []
        if (mapInfo.markInfo) {
          for (var i = 0 ; i < mapInfo.markInfo.length ; i++) {
            var mark = mapInfo.markInfo[i];
            markers.push({
              position: new daum.maps.Coords(mark.x, mark.y),
              text: mark.content
            })
          }
        }
        mapContainer.innerHTML = '';
        var map = new daum.maps.StaticMap(mapContainer, {
          center: new daum.maps.Coords(mapInfo.mapCenterX, mapInfo.mapCenterY),
          level: mapInfo.mapLevel,
          marker: markers
        });
      }
      `
      $head.append(mapLoader)
    }
    $head.append(mapScript)
  })

  editor.on('SetContent', e => {
    const $ = editor.dom.$
    $('[data-maps-data]', e.node).each((idx, elm) => {
      if (elm.contentEditable != 'false') {
        const $elm = $(elm)
        let id = elm.id? elm.id : `maps_${Date.now()}_${idx}`
        const mapdata = $elm.attr('data-maps-data')

        const wrap = editor.dom.create('figure', {
          "id": id,
          "data-ke-type": 'map',
          "data-maps-data": mapdata,
          'contentEditable': false,
          'style': 'width:490px;height:360px;'
        })
        wrap.innerHTML = '<img src="https://i1.daumcdn.net/icon/editor/img_map_490360.gif" width="490" height="360">'
        elm.parentNode.replaceChild(wrap, elm)
      }
    })
  })

  editor.on('PreProcess', e => {
    const $ = editor.dom.$
    $('[data-maps-data]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const id = $elm.attr('id')
      const data = $elm.attr('data-maps-data') 

      const wrap = editor.dom.create('iframe', {
        "id": id,
        "data-ke-type": 'map',
        "mapdata": data,
        "data-maps-data": data,
        "width": "490px",
        "height": "362px",
        "frameborder": '0',
        "scrolling": 'no',
        "src": `http://dmaps.daum.net/map_include/map_editor/v3/iframe.html?id=${id}`
      })
      elm.outerHTML = wrap.outerHTML
    })
  })
}

export default plugin;
