import '../style/unsplash.scss'
import autobind from 'autobind-decorator'
import UnsplashRenderer from './UnsplashRenderer';
import relocateCursor from '../../../kPaste/components/relocateCursor';

class App {
  constructor(editor) {
    this.editor = editor;
    this.win = null;

    this.loadTimeout = undefined;
    this.settings = editor.settings.unsplash;
    this.fetchHandler = editor.settings.unsplash && editor.settings.unsplash.fetch_handler ? editor.settings.unsplash.fetch_handler : () => {}
  }

  open() {
    let win = this.editor.windowManager.open({
      title: 'unsplash 이미지 삽입',
      items: [{
        type: 'container',
        html: '<div class="mce-unsplash" style="width:600px; height:534px">\
          <div class="mce-unsplash-header">\
            <input class="mce-unsplash-input" placeholder="검색어를 입력해주세요.">\
            <button class="mce-unsplash-search"><i class="mce-ico mce-plus">확인</i></button>\
          </div>\
          <div class="mce-unsplash-body"></div>\
          <div class="mce-unsplash-footer">\
            <div class="mce-unsplash-status"></div>\
            <button class="mce-unsplash-cancel">취소</button>\
          </div>\
        </div>'
      }],
      buttons: []
    });
    win.statusbar.remove()

    this.$input = win.$('.mce-unsplash-input')
    this.$btnSearch = win.$('.mce-unsplash-search')
    this.$body = win.$('.mce-unsplash-body')
    this.$btnCancel = win.$('.mce-unsplash-cancel')
    this.$window = win.$(window)
    this.$modalBlock = win.$('#mce-modal-block')
    this.$status = win.$('.mce-unsplash-status');

    this.$input.on("keydown", this.onKeydown)
    this.$btnSearch.on("click", this.onSearch)
    this.$btnCancel.on("click", this.onCancel)
    this.$window.on("keydown", this.onWindowKeydown)
    this.$modalBlock.on("click", this.onCancel)
    this.$input[0].focus()

    this.win = win
  }

  close() {
    if (this.win) {
      this.$input.off("keydown")
      this.$btnSearch.off("click")
      this.$btnCancel.off("click")
      this.$window.off("keydown")
      this.$modalBlock.off("click")

      this.win.close()

      this.$input = null
      this.$body = null
      this.win = null
    }
  }

  onSearch() {
    this.$body[0].textContent = "Searching..."
    this.$status[0].textContent = null;
    if(this.$loadMoreData) {
      this.$loadMoreData = undefined;
    }
    if(this.$wrapper) {
      this.$wrapper.remove();
      this.$wrapper = undefined;
    }
    this.fetchUnsplash();
  }

  loadMore(page) {
    this.$body.off('scroll');
    this.fetchUnsplash(page);
  }

  fetchUnsplash(page) {
    let wrapper = this.$wrapper || document.createElement('div');
    let bodyClientWidth = this.$body[0].clientWidth;
    this.fetchHandler(this.$input[0].value, page, (res) => {
      if(this.$body) {
        this.$body[0].textContent = null;
        if(res.results.length) {
          this.$status[0].textContent = `${res.total_pages} 페이지 중 ${page || 1} 페이지`
        }
        let renderer = new UnsplashRenderer(wrapper, res, page, bodyClientWidth);
        this.$wrapper = renderer.generateListItems();
        if(this.$loadMoreData) {
          this.$loadMoreData.remove();
        }
        this.$body[0].appendChild(this.$wrapper);
        this.$loadMoreData = this.win.$('.unsplash-search-result-load-more').length ? this.win.$('.unsplash-search-result-load-more') : undefined;
        this.$body.on('scroll', this.onScroll);
        let imageElements = document.getElementsByClassName('unsplash-search-result-image');

        Array.from(imageElements).forEach(element => {
          element.addEventListener('click', this.onListItemClicked);
        });
      }
    })
  }

  @autobind
  onKeydown(e) {
    let key = e.keyCode;
    if(key == 9 || key == 13) {
      this.onSearch();
    }
  }

  @autobind
  onCancel(e) {
    this.close();
  }

  @autobind
  onWindowKeydown(e) {
    if(e.keyCode == 27) {
      this.close();
    }
  }

  @autobind
  onListItemClicked(e) {
    const image = e.currentTarget;
    const imageUrl = image.attributes['data-unsplash-image-regular-url'].value;
    const imageId = image.attributes['data-unsplash-image-id'].value;
    const imageCopyright = image.attributes['data-unsplash-image-copyright'].value;

    relocateCursor(this.editor);
    this.editor.execCommand('KImageUploadUrl', {
      url: imageUrl,
      requireDom: true,
      caption: `Photo by ${imageCopyright} on Unsplash`,
      callback: result => {this.editor.insertContent(result.dom.outerHTML)}
    })
    this.close();
  }

  @autobind
  onScroll(e) {
    if(this.$body[0].scrollTop + this.$body[0].offsetHeight === this.$body[0].scrollHeight && this.$loadMoreData) {
      this.clearLoadTimeout();
      this.loadTimeout = setTimeout(() => {
        this.onScrollToEnd()
      }, 50);
    }
  }

  onScrollToEnd() {
    this.$loadMoreData[0].textContent = 'Loading...';
    this.loadMore(Number(this.$loadMoreData[0].attributes['data-unsplash-next-page'].value) + 1);
  }

  clearLoadTimeout() {
    if(this.loadTimeout) {
      clearTimeout(this.loadTimeout);
    }
  }
}

export default App;
