class UnsplashRenderer {

  constructor(wrapper, res, page, bodyClientWidth) {
    this.$wrapper = wrapper;
    this.res = res;
    this.page = page;
    this.bodyClientWidth = bodyClientWidth;
  }

  generateListItems() {
    let results = this.res.results;
    let len = results.length;
    if(!len) {
      let noResultText = document.createElement('p');
      noResultText.setAttribute('class', 'unsplash-no-result');
      noResultText.textContent = "검색 결과가 없습니다."
      this.$wrapper.appendChild(noResultText);
      return this.$wrapper;
    }
    this.$wrapper.setAttribute('class', 'unsplash-search-result-wrapper');

    let lines = len / 3;
    for(let i = 0; i < lines; i++) {
      let total = 0;
      let countItems = 0;
      let imageList = [];
      for(let j = 0; j < 3; j++) {

        let index = i*3 + j;
        if(!results[index]) {
          break;
        }
        countItems++;

        let thumbnailType = 'thumb';
        if(j === 0) {
          thumbnailType = results[index+2] ? 'thumb' : 'small';
        } else if(j === 1) {
          thumbnailType = results[index+1] ? 'thumb' : 'small';
        }

        let img = document.createElement('img');
        img.setAttribute('src', results[index].urls[thumbnailType]);
        img.setAttribute('class', 'unsplash-search-result-image');
        img.setAttribute('data-unsplash-image-copyright', results[index].user.username);
        img.setAttribute('data-unsplash-image-regular-url', results[index].urls.regular);
        img.setAttribute('data-unsplash-image-id', results[index].id);

        let cur = results[index].width / results[index].height;
        total += cur;
        imageList[j] = {
          img,
          cur,
          originalWidth: results[index].width,
          originalHeight: results[index].height
        }
      }

      let maxHeight = 0;
      for(let j = 0; j < 3; j++) {
        if(!imageList[j]) {
          break;
        }
        // let marginConsideredRatio = (this.bodyClientWidth - 12) / this.bodyClientWidth * 100;
        // imageList[j].img.style.width = `${imageList[j].cur / total * marginConsideredRatio}%`;

        let computedWidth = imageList[j].cur / total * (this.bodyClientWidth - 6 * (countItems - 1));
        let ratio = computedWidth / imageList[j].originalWidth;
        let computedHeight = ratio * imageList[j].originalHeight;
        maxHeight = computedHeight > maxHeight ? computedHeight : maxHeight;
        imageList[j].img.style.width = `${computedWidth}px`;
        this.$wrapper.appendChild(imageList[j].img);
      }

      for(let j = 0; j < 3; j++) {
        if(!imageList[j]) {
          break;
        }
        imageList[j].img.style.height = `${maxHeight}px`;
        if(j !== countItems-1) {
          imageList[j].img.style.marginRight = '6px';
        } else {
        }
        this.$wrapper.appendChild(imageList[j].img);
      }
    }

    if(this.res.total_pages !== 1 && this.res.total_pages !== this.page) {
      let loadMoreData = document.createElement('div');
      loadMoreData.setAttribute('class', 'unsplash-search-result-load-more')
      loadMoreData.textContent = "Load More"
      let currentPage = this.page || 1;
      loadMoreData.setAttribute('data-unsplash-next-page', currentPage);
      this.$wrapper.appendChild(loadMoreData);
    }
    return this.$wrapper;
  }
}

export default UnsplashRenderer;
