import openFile from "../../lib/openFile"
import VideoUploader from "./VideoUploader";

const plugin = (editor) => {
  const $ = editor.$

  let uploader;
  try {
    uploader = new VideoUploader(editor)
  } catch (e) {
    console.error(e.message)
    return
  }
  
  editor.addCommand('UploadVideo', uploader.upload)

  editor.addButton('video', {
    name: 'Video',
    tooltip: '동영상',
    onclick: () => {
      openFile({multiple: false, accept: 'video/mp4,video/x-m4v,video/*'}).then(files => {
        editor.undoManager.ignore(() => {
          editor.execCommand('UploadVideo', files[0])
        })
      })
    }
  })

  editor.on("PreProcess", e => {
    $('[data-ke-type="video"]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const url = $elm.attr('data-video-url')
      const thumbnail = $elm.attr('data-video-thumbnail')
      const vid = $elm.attr('data-video-vid')
      const uploaderHost = $elm.attr('data-video-uploader-host')
      
      if (!vid) {
        $elm.remove()
        throw new Error('동영상 업로드가 완료된 후 시도해 주세요.')
      }
      
      if (url) {
        $elm[0].outerHTML = `<iframe data-ke-type="video" src='${url}' data-video-url='${url}' data-video-thumbnail='${thumbnail}' data-video-vid='${vid}' data-video-uploader-host='${uploaderHost}'></iframe>`
      } else {
        $elm[0].outerHTML = `<figure data-ke-type="video" data-video-thumbnail='${thumbnail}' data-video-vid='${vid}' data-video-uploader-host='${uploaderHost}' style='background: url(${thumbnail}) no-repeat'></figure>`
      }
    })
  })

  editor.on("SetContent", e => {
    $('[data-ke-type="video"]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const url = $elm.attr('data-video-url')
      const thumbnail = $elm.attr('data-video-thumbnail')
      const vid = $elm.attr('data-video-vid')
      const uploaderHost = $elm.attr('data-video-uploader-host')
      $elm[0].outerHTML = `<figure data-ke-type="video" data-video-url='${url}' data-video-thumbnail='${thumbnail}' data-video-vid='${vid}' data-video-upload-host='${uploaderHost}' contentEditable='false' style='background: url(${thumbnail}) no-repeat'></figure>`
    })
  })
}

export default plugin
