const path = require('path')
const CopyWebpackPlugin = require('copy-webpack-plugin')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin')
const { CheckerPlugin, TsConfigPathsPlugin } = require('awesome-typescript-loader');

module.exports = (env, argv) => {

  const isProduction = argv.mode === "production"
  
  const plugins = !isProduction? [
    new MiniCssExtractPlugin({
      filename: '../static/[name].min.css'
    })
  ] : [
    new CopyWebpackPlugin([
      {
        from: 'LICENSE',
        to: '[name]'
      },
      {
        from: 'static/style/editor*.css',
        to: 'style/',
        flatten: true
      },
      {
        from: 'static/style/**/content.css',
        to: 'style/[1]/[name].[ext]',
        test: /([^/]+)\/[^/]+\.css/,
        flatten: true
      },
      {
        from: 'static/image/editor*',
        to: 'image/',
        flatten: true
      }
    ]),
    new MiniCssExtractPlugin({
      filename: '[name].min.css'
    })
  ]

  const config = {
    entry: {
      'polyfill': 'babel-polyfill',
      'plugins/codeblock/plugin': __dirname + '/src/plugins/codeblock/src/index.js',
      'plugins/maps/plugin': __dirname + '/src/plugins/maps/src/index.js',
      'plugins/opengraph/plugin': __dirname + '/src/plugins/opengraph/src/index.js',
      'plugins/kImage/plugin': __dirname + '/src/plugins/kImage/src/index.js',
      'plugins/fileUpload/plugin': __dirname + '/src/plugins/fileUpload/src/index.js',
      'plugins/video/plugin': __dirname + '/src/plugins/video/index.js',
      'plugins/location/plugin': __dirname + '/src/plugins/location/src/index.js',
      'plugins/kData/plugin': __dirname + '/src/plugins/kData/src/index.js',
      'plugins/simpleTable/plugin': __dirname + '/src/plugins/simpleTable/src/index.js',
      'plugins/unsplash/plugin': __dirname + '/src/plugins/unsplash/src/index.js',
      'plugins/klink/plugin': __dirname + '/src/plugins/klink/index.js',
      'plugins/autosave/plugin': __dirname + '/src/plugins/autosave/src/index.js',
      'plugins/google-photos/plugin': __dirname + '/src/plugins/google-photos/index.js',
      'plugins/toggle-toolbar/plugin': __dirname + '/src/plugins/toggle-toolbar/index.js',
      'plugins/kPaste/plugin': __dirname + '/src/plugins/kPaste/src/index.js',
      'plugins/colorPalette/plugin': __dirname + '/src/plugins/colorPalette/src/index.js',
  
      // service plugin
      'plugins/tip-quotation/plugin': __dirname + '/src/plugins/tip-quotation/index.js',
  
      // theme
      'themes/classic/theme': __dirname + '/src/themes/classic/src/index.js',
      'themes/balloon/theme': __dirname + '/src/themes/balloon/src/index.js',
      'themes/mw/theme':__dirname + '/src/themes/mw/src/index.js',
  
      // langs
      'langs/ko_KR': __dirname + '/src/langs/ko_KR/index.js',
      
      'skins/classic/skin': [__dirname + '/src/skins/classic/main/less/desktop/Skin.less'],
      'skins/classic/content': [__dirname + '/src/skins/classic/main/less/desktop/Content.less'],
      'skins/classic/content-inline': __dirname + '/src/skins/classic/main/less/desktop/Content.Inline.less',
      'skins/classic/skin.mobile': __dirname + '/src/skins/classic/main/less/mobile/app/mobile-less.less',
      'skins/classic/content.mobile': __dirname + '/src/skins/classic/main/less/mobile/content.less',
    },
    output: {
      path: path.join(__dirname, 'dist'),
      filename: '[name].min.js',
      publicPath: '/'
    },
    optimization: {
      minimizer: isProduction? [
        new UglifyJsPlugin(),
        new OptimizeCssAssetsPlugin()
      ] : []
      // minimize: false
    },
    module: {
      rules: [
        // {
        //   test: /Theme\.js$/,
        //   use: [{
        //     loader: 'webpack-rollup-loader',
        //     options: {
        //       treeshake: true,
        //       output: {
        //         name: 'mwts',
        //         format: 'iife',
        //         banner: '!!(function () {',
        //         footer: '})();',
        //       },
        //       plugins: [
        //         swag.nodeResolve({
        //           basedir: __dirname,
        //           prefixes: gruntUtils.prefixes({
        //             'tinymce/core': 'lib/tinymce/src/globals/tinymce/core',
        //             'tinymce/ui': 'lib/tinymce/src/ui/main/ts'
        //           }, [
        //             [`tinymce/themes/mwts`, `lib/src/themes/mwts/main/ts`]
        //           ])
        //         }),
        //         swag.remapImports()
        //       ],
        //       external: [/* modules that shouldn't be rollup'd */],
        //     }
        //   }]
        // },
        {
          test: /\.js$/,
          exclude: /node_modules/,
          use: [
            // 'source-map-loader',
            'babel-loader'
          ]
        },
        {
          test: /\.css$/,
          use: [
            'style-loader',
            'css-loader'
          ]
        },
        {
          test: /\.scss$/,
          use: [
            'style-loader',
            'css-loader',
            'sass-loader'
          ]
        },
        {
          test: /\.ts$/,
          use: [
            {
              loader: 'awesome-typescript-loader',
              options: {
                transpileOnly: true,
                configFileName: 'tsconfig.json'
              }
            }
          ]
        },
        {
          test: /\.less$/,
          use: [
            MiniCssExtractPlugin.loader,
            'css-loader',
            {
              loader: 'less-loader',
              options: {
                javascriptEnabled: true
              }
            }
          ]
        },
        {
          test: /skins\/classic\/.*\.(gif|png|jpg|jpeg|svg)($|\?)/,
          use: [
            {
              loader: 'file-loader',
              options: {
                name: 'skins/classic/img/[name].[ext]',
                publicPath: '../../'
              }
            }
          ],
        },
        {
          test: /skins\/classic\/.*\.(woff|woff2|eot|ttf)($|\?)/,
          use: [
            {
              loader: 'file-loader',
              options: {
                name: 'skins/classic/fonts/[name].[ext]',
                publicPath: '../..'
              }
            }
          ]
        },
      ]
    },
    resolve: {
      extensions: ['.ts', '.js'],
      plugins: [
        new TsConfigPathsPlugin({
          baseUrl: '.',
          compiler: 'typescript',
          configFileName: 'tsconfig.json'
        })
      ]
    },
    externals: {
      'tinymce': 'tinymce'
    },
    plugins: plugins,
    mode: process.env.NODE_ENV
  }

  if (!isProduction) {
    config.entry['themes/mwts/theme'] = __dirname + '/src/themes/mwts/main/ts/Theme.ts'
  }
  
  return config
}