import { Arr, Fun, Option } from '@ephox/katamari'
import ContextToolbars from './components/ContextToolbars'
import Toolbar from './components/Toolbar'
import NodeChangedSelector from './components/NodeChangedSelector'
import InsertEmptyParagrash from './components/InsertEmptyParagraph'
import KeyEvent from './components/KeyEvent'
import TiaraClick from './components/TiaraClick'

const classic = (editor) => {
  const EditorManager = tinymce.util.Tools.resolve('tinymce.EditorManager');
  const Tools = tinymce.util.Tools.resolve('tinymce.util.Tools');

  const isBrandingEnabled = editor => editor.getParam('branding', true, 'boolean');
  const hasMenubar = editor => getMenubar(editor) !== false;
  const getMenubar = editor => editor.getParam('menubar');
  const hasStatusbar = editor => editor.getParam('statusbar', true, 'boolean');
  const getToolbarSize = editor => editor.getParam('toolbar_items_size');
  const isReadOnly = editor => editor.getParam('readonly', false, 'boolean');
  const getFixedToolbarContainer = editor => editor.getParam('fixed_toolbar_container');
  const getInlineToolbarPositionHandler = editor => editor.getParam('inline_toolbar_position_handler');
  const getMenu = editor => editor.getParam('menu');
  const getRemovedMenuItems = editor => editor.getParam('removed_menuitems', '');
  const getMinWidth = editor => editor.getParam('min_width', 100, 'number');
  const getMinHeight = editor => editor.getParam('min_height', 100, 'number');
  const getMaxWidth = editor => editor.getParam('max_width', 65535, 'number');
  const getMaxHeight = editor => editor.getParam('max_height', 65535, 'number');
  const isSkinDisabled = editor => editor.settings.skin === false;
  const isInline = editor => editor.getParam('inline', false, 'boolean');
  const getResize = editor => {
    const resize = editor.getParam('resize', 'vertical');
    if (resize === false) {
      return 'none';
    } else if (resize === 'both') {
      return 'both';
    } else {
      return 'vertical';
    }
  };
  const getSkinUrl = editor => {
    const settings = editor.settings;
    const skin = settings.skin;
    let skinUrl = settings.skin_url;
    if (skin !== false) {
      const skinName = skin ? skin : 'lightgray';
      if (skinUrl) {
        skinUrl = editor.documentBaseURI.toAbsolute(skinUrl);
      } else {
        skinUrl = EditorManager.baseURL + '/skins/' + skinName;
      }
    }
    return skinUrl;
  };
  /* import Toolbar
  const getIndexedToolbars = (settings, defaultToolbar) => {
    const toolbars = [];
    for (let i = 1; i < 10; i++) {
      const toolbar_1 = settings['toolbar' + i];
      if (!toolbar_1) {
        break;
      }
      toolbars.push(toolbar_1);
    }
    const mainToolbar = settings.toolbar ? [settings.toolbar] : [defaultToolbar];
    return toolbars.length > 0 ? toolbars : mainToolbar;
  };
  const getToolbars = editor => {
    const toolbar = editor.getParam('toolbar');
    const defaultToolbar = 'undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image';
    if (toolbar === false) {
      return [];
    } else if (Tools.isArray(toolbar)) {
      return Tools.grep(toolbar, toolbar => toolbar.length > 0);
    } else {
      return getIndexedToolbars(editor.settings, defaultToolbar);
    }
  };
  */

  const DOMUtils = tinymce.util.Tools.resolve('tinymce.dom.DOMUtils');
  const DOM = DOMUtils.DOM;
  const Factory = tinymce.util.Tools.resolve('tinymce.ui.Factory');
  const I18n = tinymce.util.Tools.resolve('tinymce.util.I18n');
  const Rect = tinymce.util.Tools.resolve('tinymce.geom.Rect');
  const Delay = tinymce.util.Tools.resolve('tinymce.util.Delay');

  const SkinLoadedEvents = {
    fireSkinLoaded: editor => editor.fire('SkinLoaded'),
    fireResizeEditor: editor => editor.fire('ResizeEditor'),
    fireBeforeRenderUI: editor => editor.fire('BeforeRenderUI')
  };

  const focus = (panel, type) => () => {
    const item = panel.find(type)[0];
    if (item) {
      item.focus(true);
    }
  };

  const Shortcuts = {
    addKeys: (editor, panel) => {
      editor.shortcuts.add('Alt+F9', '', focus(panel, 'menubar'));
      editor.shortcuts.add('Alt+F10,F10', '', focus(panel, 'toolbar'));
      editor.shortcuts.add('Alt+F11', '', focus(panel, 'elementpath'));
      panel.on('cancel', () => {
        editor.focus();
      });
    }
  }

/* import Fun from '@ephox/katamari'
  const noop = function(...args) {
    const x = [];
    for (let _i = 0; _i < args.length; _i++) {
      x[_i] = args[_i];
    }
  };
  const noarg = f => function(...args) {
    const x = [];
    for (let _i = 0; _i < args.length; _i++) {
      x[_i] = args[_i];
    }
    return f();
  };
  const compose = (fa, fb) => function(...args) {
    const x = [];
    for (let _i = 0; _i < args.length; _i++) {
      x[_i] = args[_i];
    }
    return fa(fb(...args));
  };
  const constant = value => () => value;
  const identity = x => x;
  const tripleEquals = (a, b) => a === b;
  const curry = function (f) {
    const x = [];
    for (let _i = 1; _i < arguments.length; _i++) {
      x[_i - 1] = arguments[_i];
    }
    const args = new Array(arguments.length - 1);
    for (let i = 1; i < arguments.length; i++)
      args[i - 1] = arguments[i];
    return function () {
      const x = [];
      for (let _i = 0; _i < arguments.length; _i++) {
        x[_i] = arguments[_i];
      }
      const newArgs = new Array(arguments.length);
      for (let j = 0; j < newArgs.length; j++)
        newArgs[j] = arguments[j];
      const all = args.concat(newArgs);
      return f(...all);
    };
  };
  const not = f => function(...args) {
    const x = [];
    for (let _i = 0; _i < args.length; _i++) {
      x[_i] = args[_i];
    }
    return !f(...args);
  };
  const die = msg => () => {
    throw new Error(msg);
  };
  const apply = f => f();
  const call = f => {
    f();
  };
  const never = constant(false);
  const always = constant(true);
  const $_5axftft1jfjm4nbg = {
    noop,
    noarg,
    compose,
    constant,
    identity,
    tripleEquals,
    curry,
    not,
    die,
    apply,
    call,
    never,
    always
  };
*/

/* import Option from '@ephox/katamari'
  const never$1 = Fun.never;
  const always$1 = Fun.always;
  const none = () => NONE;
  var NONE = (() => {
    const eq = o => o.isNone();
    const call = thunk => thunk();
    const id = n => n;
    const noop = () => {
    };
    const me = {
      fold(n, s) {
        return n();
      },
      is: never$1,
      isSome: never$1,
      isNone: always$1,
      getOr: id,
      getOrThunk: call,
      getOrDie(msg) {
        throw new Error(msg || 'error: getOrDie called on none.');
      },
      or: id,
      orThunk: call,
      map: none,
      ap: none,
      each: noop,
      bind: none,
      flatten: none,
      exists: never$1,
      forall: always$1,
      filter: none,
      equals: eq,
      equals_: eq,
      toArray() {
        return [];
      },
      toString: Fun.constant('none()')
    };
    if (Object.freeze)
      Object.freeze(me);
    return me;
  })();
  const some = a => {
    const constant_a = () => a;
    const self = () => me;
    const map = f => some(f(a));
    const bind = f => f(a);
    var me = {
      fold(n, s) {
        return s(a);
      },
      is(v) {
        return a === v;
      },
      isSome: always$1,
      isNone: never$1,
      getOr: constant_a,
      getOrThunk: constant_a,
      getOrDie: constant_a,
      or: self,
      orThunk: self,
      map,
      ap(optfab) {
        return optfab.fold(none, fab => some(fab(a)));
      },
      each(f) {
        f(a);
      },
      bind,
      flatten: constant_a,
      exists: bind,
      forall: bind,
      filter(f) {
        return f(a) ? me : NONE;
      },
      equals(o) {
        return o.is(a);
      },
      equals_(o, elementEq) {
        return o.fold(never$1, b => elementEq(a, b));
      },
      toArray() {
        return [a];
      },
      toString() {
        return 'some(' + a + ')';
      }
    };
    return me;
  };
  const from = value => value === null || value === undefined ? NONE : some(value);
  const Option = {
    some,
    none,
    from
  };

*/

  const getUiContainerDelta = ctrl => {
    const uiContainer = getUiContainer(ctrl);
    if (uiContainer && DOMUtils.DOM.getStyle(uiContainer, 'position', true) !== 'static') {
      const containerPos = DOMUtils.DOM.getPos(uiContainer);
      const dx = uiContainer.scrollLeft - containerPos.x;
      const dy = uiContainer.scrollTop - containerPos.y;
      return Fun.some({
        x: dx,
        y: dy
      });
    } else {
      return Fun.none();
    }
  };
  const setUiContainer = (editor, ctrl) => {
    const uiContainer = DOMUtils.DOM.select(editor.settings.ui_container)[0];
    ctrl.getRoot().uiContainer = uiContainer;
  };
  const getUiContainer = ctrl => ctrl ? ctrl.getRoot().uiContainer : null;
  const inheritUiContainer = (fromCtrl, toCtrl) => toCtrl.uiContainer = getUiContainer(fromCtrl);
  const $_esvtriszjfjm4nba = {
    getUiContainerDelta,
    setUiContainer,
    getUiContainer,
    inheritUiContainer
  };

  /* import Toolbar use 2018/04/13
  const createToolbar = (editor, items, size) => {
    const toolbarItems = [];
    let buttonGroup;
    if (!items) {
      return;
    }
    Tools.each(items.split(/[ ,]/), item => {
      let itemName;
      const bindSelectorChanged = () => {
        const selection = editor.selection;
        if (item.settings.stateSelector) {
          selection.selectorChanged(item.settings.stateSelector, state => {
            item.active(state);
          }, true);
        }
        if (item.settings.disabledStateSelector) {
          selection.selectorChanged(item.settings.disabledStateSelector, state => {
            item.disabled(state);
          });
        }
      };
      if (item === '|') {
        buttonGroup = null;
      } else {
        if (!buttonGroup) {
          buttonGroup = {
            type: 'buttongroup',
            items: []
          };
          toolbarItems.push(buttonGroup);
        }
        if (editor.buttons[item]) {
          itemName = item;
          item = editor.buttons[itemName];
          if (typeof item === 'function') {
            item = item();
          }
          item.type = item.type || 'button';
          item.size = size;
          item = Factory.create(item);
          buttonGroup.items.push(item);
          if (editor.initialized) {
            bindSelectorChanged();
          } else {
            editor.on('init', bindSelectorChanged);
          }
        }
      }
    });
    return {
      type: 'toolbar',
      layout: 'flow',
      items: toolbarItems
    };
  };
  const createToolbars = (editor, size) => {
    const toolbars = [];
    const addToolbar = items => {
      if (items) {
        toolbars.push(createToolbar(editor, items, size));
      }
    };
    Tools.each(getToolbars(editor), toolbar => {
      addToolbar(toolbar);
    });
    if (toolbars.length) {
      return {
        type: 'panel',
        layout: 'stack',
        classes: 'toolbar-grp',
        ariaRoot: true,
        ariaRemember: true,
        items: toolbars
      };
    }
  };
  const Toolbar = {
    createToolbar,
    createToolbars
  };
  */


/* ContextToolbars component use 2018/04/10
  const toClientRect = geomRect => ({
    left: geomRect.x,
    top: geomRect.y,
    width: geomRect.w,
    height: geomRect.h,
    right: geomRect.x + geomRect.w,
    bottom: geomRect.y + geomRect.h
  });

  const hideAllFloatingPanels = editor => {
    Tools.each(editor.contextToolbars, toolbar => {
      if (toolbar.panel) {
        toolbar.panel.hide();
      }
    });
  };

  const movePanelTo = (panel, pos) => {
    panel.moveTo(pos.left, pos.top);
  };

  const togglePositionClass = (panel, relPos, predicate) => {
    relPos = relPos ? relPos.substr(0, 2) : '';
    Tools.each({
      t: 'down',
      b: 'up'
    }, (cls, pos) => {
      panel.classes.toggle('arrow-' + cls, predicate(pos, relPos.substr(0, 1)));
    });
    Tools.each({
      l: 'left',
      r: 'right'
    }, (cls, pos) => {
      panel.classes.toggle('arrow-' + cls, predicate(pos, relPos.substr(1, 1)));
    });
  };

  const userConstrain = (handler, x, y, elementRect, contentAreaRect, panelRect) => {
    panelRect = toClientRect({
      x,
      y,
      w: panelRect.w,
      h: panelRect.h
    });
    if (handler) {
      panelRect = handler({
        elementRect: toClientRect(elementRect),
        contentAreaRect: toClientRect(contentAreaRect),
        panelRect
      });
    }
    return panelRect;
  };


  const addContextualToolbars = editor => {
    let scrollContainer;
    const getContextToolbars = () => editor.contextToolbars || [];
    const getElementRect = elm => {
      let pos, targetRect, root;
      pos = DOM.getPos(editor.getContentAreaContainer());
      targetRect = editor.dom.getRect(elm);
      root = editor.dom.getRoot();
      if (root.nodeName === 'BODY') {
        targetRect.x -= root.ownerDocument.documentElement.scrollLeft || root.scrollLeft;
        targetRect.y -= root.ownerDocument.documentElement.scrollTop || root.scrollTop;
      }
      targetRect.x += pos.x;
      targetRect.y += pos.y;
      return targetRect;
    };
    const reposition = (match, shouldShow) => {
      let relPos, panelRect, elementRect, contentAreaRect, panel, relRect, testPositions, smallElementWidthThreshold;
      const handler = getInlineToolbarPositionHandler(editor);
      if (editor.removed) {
        return;
      }
      if (!match || !match.toolbar.panel) {
        hideAllFloatingPanels(editor);
        return;
      }
      testPositions = [
        'bc-tc',
        'tc-bc',
        'tl-bl',
        'bl-tl',
        'tr-br',
        'br-tr'
      ];
      panel = match.toolbar.panel;
      if (shouldShow) {
        panel.show();
      }
      elementRect = getElementRect(match.element);
      panelRect = DOM.getRect(panel.getEl());
      contentAreaRect = DOM.getRect(editor.getContentAreaContainer() || editor.getBody());
      const delta = $_esvtriszjfjm4nba.getUiContainerDelta(panel).getOr({
        x: 0,
        y: 0
      });
      elementRect.x += delta.x;
      elementRect.y += delta.y;
      panelRect.x += delta.x;
      panelRect.y += delta.y;
      contentAreaRect.x += delta.x;
      contentAreaRect.y += delta.y;
      smallElementWidthThreshold = 25;
      if (DOM.getStyle(match.element, 'display', true) !== 'inline') {
        const clientRect = match.element.getBoundingClientRect();
        elementRect.w = clientRect.width;
        elementRect.h = clientRect.height;
      }
      if (!editor.inline) {
        contentAreaRect.w = editor.getDoc().documentElement.offsetWidth;
      }
      if (editor.selection.controlSelection.isResizable(match.element) && elementRect.w < smallElementWidthThreshold) {
        elementRect = Rect.inflate(elementRect, 0, 8);
      }
      relPos = Rect.findBestRelativePosition(panelRect, elementRect, contentAreaRect, testPositions);
      elementRect = Rect.clamp(elementRect, contentAreaRect);
      if (relPos) {
        relRect = Rect.relativePosition(panelRect, elementRect, relPos);
        movePanelTo(panel, userConstrain(handler, relRect.x, relRect.y, elementRect, contentAreaRect, panelRect));
      } else {
        contentAreaRect.h += panelRect.h;
        elementRect = Rect.intersect(contentAreaRect, elementRect);
        if (elementRect) {
          relPos = Rect.findBestRelativePosition(panelRect, elementRect, contentAreaRect, [
            'bc-tc',
            'bl-tl',
            'br-tr'
          ]);
          if (relPos) {
            relRect = Rect.relativePosition(panelRect, elementRect, relPos);
            movePanelTo(panel, userConstrain(handler, relRect.x, relRect.y, elementRect, contentAreaRect, panelRect));
          } else {
            movePanelTo(panel, userConstrain(handler, elementRect.x, elementRect.y, elementRect, contentAreaRect, panelRect));
          }
        } else {
          panel.hide();
        }
      }
      togglePositionClass(panel, relPos, (pos1, pos2) => pos1 === pos2);
    };
    const repositionHandler = show => () => {
      const execute = () => {
        if (editor.selection) {
          reposition(findFrontMostMatch(editor.selection.getNode()), show);
        }
      };
      Delay.requestAnimationFrame(execute);
    };
    const bindScrollEvent = panel => {
      if (!scrollContainer) {
        const reposition_1 = repositionHandler(true);
        const uiContainer_1 = $_esvtriszjfjm4nba.getUiContainer(panel);
        scrollContainer = editor.selection.getScrollContainer() || editor.getWin();
        DOM.bind(scrollContainer, 'scroll', reposition_1);
        DOM.bind(uiContainer_1, 'scroll', reposition_1);
        editor.on('remove', () => {
          DOM.unbind(scrollContainer, 'scroll', reposition_1);
          DOM.unbind(uiContainer_1, 'scroll', reposition_1);
        });
      }
    };
    const showContextToolbar = match => {
      let panel;
      if (match.toolbar.panel) {
        match.toolbar.panel.show();
        reposition(match);
        return;
      }
      panel = Factory.create({
        type: 'floatpanel',
        role: 'dialog',
        classes: 'tinymce tinymce-inline arrow',
        ariaLabel: 'Inline toolbar',
        layout: 'flex',
        direction: 'column',
        align: 'stretch',
        autohide: false,
        autofix: true,
        fixed: true,
        border: 1,
        items: Toolbar.createToolbar(editor, match.toolbar.items),
        oncancel() {
          editor.focus();
        }
      });
      $_esvtriszjfjm4nba.setUiContainer(editor, panel);
      bindScrollEvent(panel);
      match.toolbar.panel = panel;
      panel.renderTo().reflow();
      reposition(match);
    };
    const hideAllContextToolbars = () => {
      Tools.each(getContextToolbars(), toolbar => {
        if (toolbar.panel) {
          toolbar.panel.hide();
        }
      });
    };
    var findFrontMostMatch = targetElm => {
      let i, y, parentsAndSelf;
      const toolbars = getContextToolbars();
      parentsAndSelf = editor.$(targetElm).parents().add(targetElm);
      for (i = parentsAndSelf.length - 1; i >= 0; i--) {
        for (y = toolbars.length - 1; y >= 0; y--) {
          if (toolbars[y].predicate(parentsAndSelf[i])) {
            return {
              toolbar: toolbars[y],
              element: parentsAndSelf[i]
            };
          }
        }
      }
      return null;
    };
    editor.on('click keyup setContent ObjectResized', e => {
      if (e.type === 'setcontent' && !e.selection) {
        return;
      }
      Delay.setEditorTimeout(editor, () => {
        let match;
        match = findFrontMostMatch(editor.selection.getNode());
        if (match) {
          hideAllContextToolbars();
          showContextToolbar(match);
        } else {
          hideAllContextToolbars();
        }
      });
    });
    editor.on('blur hide contextmenu', hideAllContextToolbars);
    editor.on('ObjectResizeStart', () => {
      const match = findFrontMostMatch(editor.selection.getNode());
      if (match && match.toolbar.panel) {
        match.toolbar.panel.hide();
      }
    });
    editor.on('ResizeEditor ResizeWindow', repositionHandler(true));
    editor.on('nodeChange', repositionHandler(false));
    editor.on('remove', () => {
      Tools.each(getContextToolbars(), toolbar => {
        if (toolbar.panel) {
          toolbar.panel.remove();
        }
      });
      editor.contextToolbars = {};
    });
    editor.shortcuts.add('ctrl+shift+e > ctrl+shift+p', '', () => {
      const match = findFrontMostMatch(editor.selection.getNode());
      if (match && match.toolbar.panel) {
        match.toolbar.panel.items()[0].focus();
      }
    });
  };

  const $_60fbpnswjfjm4nav = { addContextualToolbars };

*/

/* import Arr from '@ephox/katamari'
  const rawIndexOf = (() => {
    const pIndexOf = Array.prototype.indexOf;
    const fastIndex = (xs, x) => pIndexOf.call(xs, x);
    const slowIndex = (xs, x) => slowIndexOf(xs, x);
    return pIndexOf === undefined ? slowIndex : fastIndex;
  })();
  const indexOf = (xs, x) => {
    const r = rawIndexOf(xs, x);
    return r === -1 ? Option.none() : Option.some(r);
  };
  const contains = (xs, x) => rawIndexOf(xs, x) > -1;
  const exists = (xs, pred) => findIndex(xs, pred).isSome();
  const range = (num, f) => {
    const r = [];
    for (let i = 0; i < num; i++) {
      r.push(f(i));
    }
    return r;
  };
  const chunk = (array, size) => {
    const r = [];
    for (let i = 0; i < array.length; i += size) {
      const s = array.slice(i, i + size);
      r.push(s);
    }
    return r;
  };
  const map = (xs, f) => {
    const len = xs.length;
    const r = new Array(len);
    for (let i = 0; i < len; i++) {
      const x = xs[i];
      r[i] = f(x, i, xs);
    }
    return r;
  };
  const each = (xs, f) => {
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      f(x, i, xs);
    }
  };
  const eachr = (xs, f) => {
    for (let i = xs.length - 1; i >= 0; i--) {
      const x = xs[i];
      f(x, i, xs);
    }
  };
  const partition = (xs, pred) => {
    const pass = [];
    const fail = [];
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      const arr = pred(x, i, xs) ? pass : fail;
      arr.push(x);
    }
    return {
      pass,
      fail
    };
  };
  const filter = (xs, pred) => {
    const r = [];
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      if (pred(x, i, xs)) {
        r.push(x);
      }
    }
    return r;
  };
  const groupBy = (xs, f) => {
    if (xs.length === 0) {
      return [];
    } else {
      let wasType = f(xs[0]);
      const r = [];
      let group = [];
      for (let i = 0, len = xs.length; i < len; i++) {
        const x = xs[i];
        const type = f(x);
        if (type !== wasType) {
          r.push(group);
          group = [];
        }
        wasType = type;
        group.push(x);
      }
      if (group.length !== 0) {
        r.push(group);
      }
      return r;
    }
  };
  const foldr = (xs, f, acc) => {
    eachr(xs, x => {
      acc = f(acc, x);
    });
    return acc;
  };
  const foldl = (xs, f, acc) => {
    each(xs, x => {
      acc = f(acc, x);
    });
    return acc;
  };
  const find = (xs, pred) => {
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      if (pred(x, i, xs)) {
        return Option.some(x);
      }
    }
    return Option.none();
  };
  var findIndex = (xs, pred) => {
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      if (pred(x, i, xs)) {
        return Option.some(i);
      }
    }
    return Option.none();
  };
  var slowIndexOf = (xs, x) => {
    for (let i = 0, len = xs.length; i < len; ++i) {
      if (xs[i] === x) {
        return i;
      }
    }
    return -1;
  };
  const push = Array.prototype.push;
  const flatten = xs => {
    const r = [];
    for (let i = 0, len = xs.length; i < len; ++i) {
      if (!Array.prototype.isPrototypeOf(xs[i]))
        throw new Error('Arr.flatten item ' + i + ' was not an array, input: ' + xs);
      push.apply(r, xs[i]);
    }
    return r;
  };
  const bind = (xs, f) => {
    const output = map(xs, f);
    return flatten(output);
  };
  const forall = (xs, pred) => {
    for (let i = 0, len = xs.length; i < len; ++i) {
      const x = xs[i];
      if (pred(x, i, xs) !== true) {
        return false;
      }
    }
    return true;
  };
  const equal = (a1, a2) => a1.length === a2.length && forall(a1, (x, i) => x === a2[i]);
  const slice = Array.prototype.slice;
  const reverse = xs => {
    const r = slice.call(xs, 0);
    r.reverse();
    return r;
  };
  const difference = (a1, a2) => filter(a1, x => !contains(a2, x));
  const mapToObject = (xs, f) => {
    const r = {};
    for (let i = 0, len = xs.length; i < len; i++) {
      const x = xs[i];
      r[String(x)] = f(x, i);
    }
    return r;
  };
  const pure = x => [x];
  const sort = (xs, comparator) => {
    const copy = slice.call(xs, 0);
    copy.sort(comparator);
    return copy;
  };
  const head = xs => xs.length === 0 ? Option.none() : Option.some(xs[0]);
  const last = xs => xs.length === 0 ? Option.none() : Option.some(xs[xs.length - 1]);
  const $_ermrc2t4jfjm4nbr = {
    map,
    each,
    eachr,
    partition,
    filter,
    groupBy,
    indexOf,
    foldr,
    foldl,
    find,
    findIndex,
    flatten,
    bind,
    forall,
    exists,
    contains,
    equal,
    reverse,
    chunk,
    difference,
    mapToObject,
    pure,
    sort,
    range,
    head,
    last
  };

*/

  const defaultMenus = {
    file: {
      title: 'File',
      items: 'newdocument restoredraft | preview | print'
    },
    edit: {
      title: 'Edit',
      items: 'undo redo | cut copy paste pastetext | selectall'
    },
    view: {
      title: 'View',
      items: 'code | visualaid visualchars visualblocks | spellchecker | preview fullscreen'
    },
    insert: {
      title: 'Insert',
      items: 'image link media template codesample inserttable | charmap hr | pagebreak nonbreaking anchor toc | insertdatetime'
    },
    format: {
      title: 'Format',
      items: 'bold italic underline strikethrough superscript subscript codeformat | blockformats align | removeformat'
    },
    tools: {
      title: 'Tools',
      items: 'spellchecker spellcheckerlanguage | a11ycheck'
    },
    table: { title: 'Table' },
    help: { title: 'Help' }
  };
  const delimiterMenuNamePair = () => ({
    name: '|',
    item: { text: '|' }
  });
  const createMenuNameItemPair = (name, item) => {
    const menuItem = item ? {
      name,
      item
    } : null;
    return name === '|' ? delimiterMenuNamePair() : menuItem;
  };
  const hasItemName = (namedMenuItems, name) => Arr.findIndex(namedMenuItems, namedMenuItem => namedMenuItem.name === name).isSome();
  const isSeparator = namedMenuItem => namedMenuItem && namedMenuItem.item.text === '|';
  const cleanupMenu = (namedMenuItems, removedMenuItems) => {
    const menuItemsPass1 = Arr.filter(namedMenuItems, namedMenuItem => removedMenuItems.hasOwnProperty(namedMenuItem.name) === false);
    const menuItemsPass2 = Arr.filter(menuItemsPass1, (namedMenuItem, i, namedMenuItems) => !isSeparator(namedMenuItem) || !isSeparator(namedMenuItems[i - 1]));
    return Arr.filter(menuItemsPass2, (namedMenuItem, i, namedMenuItems) => !isSeparator(namedMenuItem) || i > 0 && i < namedMenuItems.length - 1);
  };
  const createMenu = (editorMenuItems, menus, removedMenuItems, context) => {
    let menuButton, menu, namedMenuItems, isUserDefined;
    if (menus) {
      menu = menus[context];
      isUserDefined = true;
    } else {
      menu = defaultMenus[context];
    }
    if (menu) {
      menuButton = { text: menu.title };
      namedMenuItems = [];
      Tools.each((menu.items || '').split(/[ ,]/), name => {
        const namedMenuItem = createMenuNameItemPair(name, editorMenuItems[name]);
        if (namedMenuItem) {
          namedMenuItems.push(namedMenuItem);
        }
      });
      if (!isUserDefined) {
        Tools.each(editorMenuItems, (item, name) => {
          if (item.context === context && !hasItemName(namedMenuItems, name)) {
            if (item.separator === 'before') {
              namedMenuItems.push(delimiterMenuNamePair());
            }
            if (item.prependToContext) {
              namedMenuItems.unshift(createMenuNameItemPair(name, item));
            } else {
              namedMenuItems.push(createMenuNameItemPair(name, item));
            }
            if (item.separator === 'after') {
              namedMenuItems.push(delimiterMenuNamePair());
            }
          }
        });
      }
      menuButton.menu = Arr.map(cleanupMenu(namedMenuItems, removedMenuItems), menuItem => menuItem.item);
      if (!menuButton.menu.length) {
        return null;
      }
    }
    return menuButton;
  };
  const getDefaultMenubar = editor => {
    let name;
    const defaultMenuBar = [];
    const menu = getMenu(editor);
    if (menu) {
      for (name in menu) {
        defaultMenuBar.push(name);
      }
    } else {
      for (name in defaultMenus) {
        defaultMenuBar.push(name);
      }
    }
    return defaultMenuBar;
  };
  const createMenuButtons = editor => {
    const menuButtons = [];
    const defaultMenuBar = getDefaultMenubar(editor);
    const removedMenuItems = Tools.makeMap(getRemovedMenuItems(editor).split(/[ ,]/));
    const menubar = getMenubar(editor);
    const enabledMenuNames = typeof menubar === 'string' ? menubar.split(/[ ,]/) : defaultMenuBar;
    for (let i = 0; i < enabledMenuNames.length; i++) {
      const menuItems = enabledMenuNames[i];
      const menu = createMenu(editor.menuItems, getMenu(editor), removedMenuItems, menuItems);
      if (menu) {
        menuButtons.push(menu);
      }
    }
    return menuButtons;
  };
  const $_zri7tt3jfjm4nbl = { createMenuButtons };

  const DOM$1 = DOMUtils.DOM;
  const getSize = elm => ({
    width: elm.clientWidth,
    height: elm.clientHeight
  });
  const resizeTo = (editor, width, height) => {
    let containerElm, iframeElm, containerSize, iframeSize;
    containerElm = editor.getContainer();
    iframeElm = editor.getContentAreaContainer().firstChild;
    containerSize = getSize(containerElm);
    iframeSize = getSize(iframeElm);
    if (width !== null) {
      width = Math.max(getMinWidth(editor), width);
      width = Math.min(getMaxWidth(editor), width);
      DOM$1.setStyle(containerElm, 'width', width + (containerSize.width - iframeSize.width));
      DOM$1.setStyle(iframeElm, 'width', width);
    }
    height = Math.max(getMinHeight(editor), height);
    height = Math.min(getMaxHeight(editor), height);
    DOM$1.setStyle(iframeElm, 'height', height);
    SkinLoadedEvents.fireResizeEditor(editor);
  };
  const resizeBy = (editor, dw, dh) => {
    const elm = editor.getContentAreaContainer();
    resizeTo(editor, elm.clientWidth + dw, elm.clientHeight + dh);
  };
  const $_5ku2twt5jfjm4nbw = {
    resizeTo,
    resizeBy
  };

  const global$8 = tinymce.util.Tools.resolve('tinymce.Env');

  const api = elm => ({
    element() {
      return elm;
    }
  });
  const trigger = (sidebar, panel, callbackName) => {
    const callback = sidebar.settings[callbackName];
    if (callback) {
      callback(api(panel.getEl('body')));
    }
  };
  const hidePanels = (name, container, sidebars) => {
    Tools.each(sidebars, sidebar => {
      const panel = container.items().filter('#' + sidebar.name)[0];
      if (panel && panel.visible() && sidebar.name !== name) {
        trigger(sidebar, panel, 'onhide');
        panel.visible(false);
      }
    });
  };
  const deactivateButtons = toolbar => {
    toolbar.items().each(ctrl => {
      ctrl.active(false);
    });
  };
  const findSidebar = (sidebars, name) => Tools.grep(sidebars, sidebar => sidebar.name === name)[0];
  const showPanel = (editor, name, sidebars) => e => {
    const btnCtrl = e.control;
    const container = btnCtrl.parents().filter('panel')[0];
    let panel = container.find('#' + name)[0];
    const sidebar = findSidebar(sidebars, name);
    hidePanels(name, container, sidebars);
    deactivateButtons(btnCtrl.parent());
    if (panel && panel.visible()) {
      trigger(sidebar, panel, 'onhide');
      panel.hide();
      btnCtrl.active(false);
    } else {
      if (panel) {
        panel.show();
        trigger(sidebar, panel, 'onshow');
      } else {
        panel = Factory.create({
          type: 'container',
          name,
          layout: 'stack',
          classes: 'sidebar-panel',
          html: ''
        });
        container.prepend(panel);
        trigger(sidebar, panel, 'onrender');
        trigger(sidebar, panel, 'onshow');
      }
      btnCtrl.active(true);
    }
    SkinLoadedEvents.fireResizeEditor(editor);
  };
  const isModernBrowser = () => !global$8.ie || global$8.ie >= 11;
  const hasSidebar = editor => isModernBrowser() && editor.sidebars ? editor.sidebars.length > 0 : false;
  const createSidebar = editor => {
    const buttons = Tools.map(editor.sidebars, sidebar => {
      const settings = sidebar.settings;
      return {
        type: 'button',
        icon: settings.icon,
        image: settings.image,
        tooltip: settings.tooltip,
        onclick: showPanel(editor, sidebar.name, editor.sidebars)
      };
    });
    return {
      type: 'panel',
      name: 'sidebar',
      layout: 'stack',
      classes: 'sidebar',
      items: [{
          type: 'toolbar',
          layout: 'stack',
          classes: 'sidebar-toolbar',
          items: buttons
        }]
    };
  };
  const $_eea5cut6jfjm4nby = {
    hasSidebar,
    createSidebar
  };

  const fireSkinLoaded$1 = editor => {
    const done = () => {
      editor._skinLoaded = true;
      SkinLoadedEvents.fireSkinLoaded(editor);
    };
    return () => {
      if (editor.initialized) {
        done();
      } else {
        editor.on('init', done);
      }
    };
  };
  const $_9j2wbtt8jfjm4nc1 = { fireSkinLoaded: fireSkinLoaded$1 };

  const switchMode = panel => e => {
    panel.find('*').disabled(e.mode === 'readonly');
  };
  const editArea = border => ({
    type: 'panel',
    name: 'iframe',
    layout: 'stack',
    classes: 'edit-area',
    border,
    html: ''
  });
  const editAreaContainer = editor => ({
    type: 'panel',
    layout: 'stack',
    classes: 'edit-aria-container',
    border: '1 0 0 0',

    items: [
      editArea('0'),
      $_eea5cut6jfjm4nby.createSidebar(editor)
    ]
  });
  const render = (editor, theme, args) => {
    let panel, resizeHandleCtrl, startSize;
    if (isSkinDisabled(editor) === false && args.skinUiCss) {
      DOM.styleSheetLoader.load(args.skinUiCss, $_9j2wbtt8jfjm4nc1.fireSkinLoaded(editor));
    } else {
      $_9j2wbtt8jfjm4nc1.fireSkinLoaded(editor)();
    }
    panel = theme.panel = Factory.create({
      type: 'panel',
      role: 'application',
      classes: 'tinymce',
      style: 'visibility: hidden',
      layout: 'stack',
      border: 1,
      items: [
        {
          type: 'container',
          classes: 'top-part',
          items: [
            hasMenubar(editor) === false ? null : {
              type: 'menubar',
              border: '0 0 1 0',
              items: $_zri7tt3jfjm4nbl.createMenuButtons(editor)
            },
            Toolbar.createToolbars(editor, getToolbarSize(editor))
          ]
        },
        $_eea5cut6jfjm4nby.hasSidebar(editor) ? editAreaContainer(editor) : editArea('1 0 0 0')
      ]
    });
    $_esvtriszjfjm4nba.setUiContainer(editor, panel);
    if (getResize(editor) !== 'none') {
      resizeHandleCtrl = {
        type: 'resizehandle',
        direction: getResize(editor),
        onResizeStart() {
          const elm = editor.getContentAreaContainer().firstChild;
          startSize = {
            width: elm.clientWidth,
            height: elm.clientHeight
          };
        },
        onResize(e) {
          if (getResize(editor) === 'both') {
            $_5ku2twt5jfjm4nbw.resizeTo(editor, startSize.width + e.deltaX, startSize.height + e.deltaY);
          } else {
            $_5ku2twt5jfjm4nbw.resizeTo(editor, null, startSize.height + e.deltaY);
          }
        }
      };
    }
    if (hasStatusbar(editor)) {
      const linkHtml = '<a href="https://www.tinymce.com/?utm_campaign=editor_referral&utm_medium=poweredby&utm_source=tinymce" rel="noopener" target="_blank" role="presentation" tabindex="-1">tinymce</a>';
      const html = I18n.translate([
        'Powered by {0}',
        linkHtml
      ]);
      const brandingLabel = isBrandingEnabled(editor) ? {
        type: 'label',
        classes: 'branding',
        html: ' ' + html
      } : null;
      panel.add({
        type: 'panel',
        name: 'statusbar',
        classes: 'statusbar',
        layout: 'flow',
        border: '1 0 0 0',
        ariaRoot: true,
        items: [
          {
            type: 'elementpath',
            editor
          },
          resizeHandleCtrl,
          brandingLabel
        ]
      });
    }
    SkinLoadedEvents.fireBeforeRenderUI(editor);
    editor.on('SwitchMode', switchMode(panel));
    panel.renderBefore(args.targetNode).reflow();
    if (isReadOnly(editor)) {
      editor.setMode('readonly');
    }
    if (args.width) {
      DOM.setStyle(panel.getEl(), 'width', args.width);
    }
    editor.on('remove', () => {
      panel.remove();
      panel = null;
    });
    Shortcuts.addKeys(editor, panel);
    ContextToolbars.addContextualToolbars(editor);
    return {
      iframeContainer: panel.find('#iframe')[0].getEl(),
      editorContainer: panel.getEl()
    };
  };
  const $_5f2n0osqjfjm4nao = { render };

  const global$9 = tinymce.util.Tools.resolve('tinymce.dom.DomQuery');

  let count = 0;
  const funcs = {
    id() {
      return 'mceu_' + count++;
    },
    create(name, attrs, children) {
      const elm = document.createElement(name);
      DOMUtils.DOM.setAttribs(elm, attrs);
      if (typeof children === 'string') {
        elm.innerHTML = children;
      } else {
        Tools.each(children, child => {
          if (child.nodeType) {
            elm.appendChild(child);
          }
        });
      }
      return elm;
    },
    createFragment(html) {
      return DOMUtils.DOM.createFragment(html);
    },
    getWindowSize() {
      return DOMUtils.DOM.getViewPort();
    },
    getSize(elm) {
      let width, height;
      if (elm.getBoundingClientRect) {
        const rect = elm.getBoundingClientRect();
        width = Math.max(rect.width || rect.right - rect.left, elm.offsetWidth);
        height = Math.max(rect.height || rect.bottom - rect.bottom, elm.offsetHeight);
      } else {
        width = elm.offsetWidth;
        height = elm.offsetHeight;
      }
      return {
        width,
        height
      };
    },
    getPos(elm, root) {
      return DOMUtils.DOM.getPos(elm, root || funcs.getContainer());
    },
    getContainer() {
      return global$8.container ? global$8.container : document.body;
    },
    getViewPort(win) {
      return DOMUtils.DOM.getViewPort(win);
    },
    get(id) {
      return document.getElementById(id);
    },
    addClass(elm, cls) {
      return DOMUtils.DOM.addClass(elm, cls);
    },
    removeClass(elm, cls) {
      return DOMUtils.DOM.removeClass(elm, cls);
    },
    hasClass(elm, cls) {
      return DOMUtils.DOM.hasClass(elm, cls);
    },
    toggleClass(elm, cls, state) {
      return DOMUtils.DOM.toggleClass(elm, cls, state);
    },
    css(elm, name, value) {
      return DOMUtils.DOM.setStyle(elm, name, value);
    },
    getRuntimeStyle(elm, name) {
      return DOMUtils.DOM.getStyle(elm, name, true);
    },
    on(target, name, callback, scope) {
      return DOMUtils.DOM.bind(target, name, callback, scope);
    },
    off(target, name, callback) {
      return DOMUtils.DOM.unbind(target, name, callback);
    },
    fire(target, name, args) {
      return DOMUtils.DOM.fire(target, name, args);
    },
    innerHtml(elm, html) {
      DOMUtils.DOM.setHTML(elm, html);
    }
  };

  const isStatic = elm => funcs.getRuntimeStyle(elm, 'position') === 'static';
  const isFixed = ctrl => ctrl.state.get('fixed');
  function calculateRelativePosition(ctrl, targetElm, rel) {
    let ctrlElm, pos, x, y, selfW, selfH, targetW, targetH, viewport, size;
    viewport = getWindowViewPort();
    pos = funcs.getPos(targetElm, $_esvtriszjfjm4nba.getUiContainer(ctrl));
    x = pos.x;
    y = pos.y;
    if (isFixed(ctrl) && isStatic(document.body)) {
      x -= viewport.x;
      y -= viewport.y;
    }
    ctrlElm = ctrl.getEl();
    size = funcs.getSize(ctrlElm);
    selfW = size.width;
    selfH = size.height;
    size = funcs.getSize(targetElm);
    targetW = size.width;
    targetH = size.height;
    rel = (rel || '').split('');
    if (rel[0] === 'b') {
      y += targetH;
    }
    if (rel[1] === 'r') {
      x += targetW;
    }
    if (rel[0] === 'c') {
      y += Math.round(targetH / 2);
    }
    if (rel[1] === 'c') {
      x += Math.round(targetW / 2);
    }
    if (rel[3] === 'b') {
      y -= selfH;
    }
    if (rel[4] === 'r') {
      x -= selfW;
    }
    if (rel[3] === 'c') {
      y -= Math.round(selfH / 2);
    }
    if (rel[4] === 'c') {
      x -= Math.round(selfW / 2);
    }
    return {
      x,
      y,
      w: selfW,
      h: selfH
    };
  }
  const getUiContainerViewPort = customUiContainer => ({
    x: 0,
    y: 0,
    w: customUiContainer.scrollWidth - 1,
    h: customUiContainer.scrollHeight - 1
  });
  var getWindowViewPort = () => {
    const win = window;
    const x = Math.max(win.pageXOffset, document.body.scrollLeft, document.documentElement.scrollLeft);
    const y = Math.max(win.pageYOffset, document.body.scrollTop, document.documentElement.scrollTop);
    const w = win.innerWidth || document.documentElement.clientWidth;
    const h = win.innerHeight || document.documentElement.clientHeight;
    return {
      x,
      y,
      w: x + w,
      h: y + h
    };
  };
  const getViewPortRect = ctrl => {
    const customUiContainer = $_esvtriszjfjm4nba.getUiContainer(ctrl);
    return customUiContainer && !isFixed(ctrl) ? getUiContainerViewPort(customUiContainer) : getWindowViewPort();
  };
  const $_1ttsxttdjfjm4ncj = {
    testMoveRel(elm, rels) {
      const viewPortRect = getViewPortRect(this);
      for (let i = 0; i < rels.length; i++) {
        const pos = calculateRelativePosition(this, elm, rels[i]);
        if (isFixed(this)) {
          if (pos.x > 0 && pos.x + pos.w < viewPortRect.w && pos.y > 0 && pos.y + pos.h < viewPortRect.h) {
            return rels[i];
          }
        } else {
          if (pos.x > viewPortRect.x && pos.x + pos.w < viewPortRect.w && pos.y > viewPortRect.y && pos.y + pos.h < viewPortRect.h) {
            return rels[i];
          }
        }
      }
      return rels[0];
    },
    moveRel(elm, rel) {
      if (typeof rel !== 'string') {
        rel = this.testMoveRel(elm, rel);
      }
      const pos = calculateRelativePosition(this, elm, rel);
      return this.moveTo(pos.x, pos.y);
    },
    moveBy(dx, dy) {
      const self = this, rect = self.layoutRect();
      self.moveTo(rect.x + dx, rect.y + dy);
      return self;
    },
    moveTo(x, y) {
      const self = this;
      function constrain(value, max, size) {
        if (value < 0) {
          return 0;
        }
        if (value + size > max) {
          value = max - size;
          return value < 0 ? 0 : value;
        }
        return value;
      }
      if (self.settings.constrainToViewport) {
        const viewPortRect = getViewPortRect(this);
        const layoutRect = self.layoutRect();
        x = constrain(x, viewPortRect.w, layoutRect.w);
        y = constrain(y, viewPortRect.h, layoutRect.h);
      }
      const uiContainer = $_esvtriszjfjm4nba.getUiContainer(self);
      if (uiContainer && isStatic(uiContainer) && !isFixed(self)) {
        x -= uiContainer.scrollLeft;
        y -= uiContainer.scrollTop;
      }
      if (uiContainer) {
        x += 1;
        y += 1;
      }
      if (self.state.get('rendered')) {
        self.layoutRect({
          x,
          y
        }).repaint();
      } else {
        self.settings.x = x;
        self.settings.y = y;
      }
      self.fire('move', {
        x,
        y
      });
      return self;
    }
  };

  const global$10 = tinymce.util.Tools.resolve('tinymce.util.Class');

  const global$11 = tinymce.util.Tools.resolve('tinymce.util.EventDispatcher');

  const $_bty1r7tjjfjm4ndm = {
    parseBox(value) {
      let len;
      const radix = 10;
      if (!value) {
        return;
      }
      if (typeof value === 'number') {
        value = value || 0;
        return {
          top: value,
          left: value,
          bottom: value,
          right: value
        };
      }
      value = value.split(' ');
      len = value.length;
      if (len === 1) {
        value[1] = value[2] = value[3] = value[0];
      } else if (len === 2) {
        value[2] = value[0];
        value[3] = value[1];
      } else if (len === 3) {
        value[3] = value[1];
      }
      return {
        top: parseInt(value[0], radix) || 0,
        right: parseInt(value[1], radix) || 0,
        bottom: parseInt(value[2], radix) || 0,
        left: parseInt(value[3], radix) || 0
      };
    },
    measureBox(elm, prefix) {
      function getStyle(name) {
        const defaultView = elm.ownerDocument.defaultView;
        if (defaultView) {
          const computedStyle = defaultView.getComputedStyle(elm, null);
          if (computedStyle) {
            name = name.replace(/[A-Z]/g, a => '-' + a);
            return computedStyle.getPropertyValue(name);
          } else {
            return null;
          }
        }
        return elm.currentStyle[name];
      }
      function getSide(name) {
        const val = parseFloat(getStyle(name));
        return isNaN(val) ? 0 : val;
      }
      return {
        top: getSide(prefix + 'TopWidth'),
        right: getSide(prefix + 'RightWidth'),
        bottom: getSide(prefix + 'BottomWidth'),
        left: getSide(prefix + 'LeftWidth')
      };
    }
  };

  function noop$1() {
  }
  function ClassList(onchange) {
    this.cls = [];
    this.cls._map = {};
    this.onchange = onchange || noop$1;
    this.prefix = '';
  }
  Tools.extend(ClassList.prototype, {
    add(cls) {
      if (cls && !this.contains(cls)) {
        this.cls._map[cls] = true;
        this.cls.push(cls);
        this._change();
      }
      return this;
    },
    remove(cls) {
      if (this.contains(cls)) {
        let i = void 0;
        for (i = 0; i < this.cls.length; i++) {
          if (this.cls[i] === cls) {
            break;
          }
        }
        this.cls.splice(i, 1);
        delete this.cls._map[cls];
        this._change();
      }
      return this;
    },
    toggle(cls, state) {
      const curState = this.contains(cls);
      if (curState !== state) {
        if (curState) {
          this.remove(cls);
        } else {
          this.add(cls);
        }
        this._change();
      }
      return this;
    },
    contains(cls) {
      return !!this.cls._map[cls];
    },
    _change() {
      delete this.clsValue;
      this.onchange.call(this);
    }
  });
  ClassList.prototype.toString = function () {
    let value;
    if (this.clsValue) {
      return this.clsValue;
    }
    value = '';
    for (let i = 0; i < this.cls.length; i++) {
      if (i > 0) {
        value += ' ';
      }
      value += this.prefix + this.cls[i];
    }
    return value;
  };

  function unique(array) {
    const uniqueItems = [];
    let i = array.length, item;
    while (i--) {
      item = array[i];
      if (!item.__checked) {
        uniqueItems.push(item);
        item.__checked = 1;
      }
    }
    i = uniqueItems.length;
    while (i--) {
      delete uniqueItems[i].__checked;
    }
    return uniqueItems;
  }
  const expression = /^([\w\\*]+)?(?:#([\w\-\\]+))?(?:\.([\w\\\.]+))?(?:\[\@?([\w\\]+)([\^\$\*!~]?=)([\w\\]+)\])?(?:\:(.+))?/i;
  const chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^\[\]]*\]|['"][^'"]*['"]|[^\[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g;
  const whiteSpace = /^\s*|\s*$/g;
  let Collection;
  const Selector = global$10.extend({
    init(selector) {
      const match = this.match;
      function compileNameFilter(name) {
        if (name) {
          name = name.toLowerCase();
          return item => name === '*' || item.type === name;
        }
      }
      function compileIdFilter(id) {
        if (id) {
          return item => item._name === id;
        }
      }
      function compileClassesFilter(classes) {
        if (classes) {
          classes = classes.split('.');
          return item => {
            let i = classes.length;
            while (i--) {
              if (!item.classes.contains(classes[i])) {
                return false;
              }
            }
            return true;
          };
        }
      }
      function compileAttrFilter(name, cmp, check) {
        if (name) {
          return item => {
            const value = item[name] ? item[name]() : '';
            return !cmp ? !!check : cmp === '=' ? value === check : cmp === '*=' ? value.indexOf(check) >= 0 : cmp === '~=' ? (' ' + value + ' ').indexOf(' ' + check + ' ') >= 0 : cmp === '!=' ? value !== check : cmp === '^=' ? value.indexOf(check) === 0 : cmp === '$=' ? value.substr(value.length - check.length) === check : false;
          };
        }
      }
      function compilePsuedoFilter(name) {
        let notSelectors;
        if (name) {
          name = /(?:not\((.+)\))|(.+)/i.exec(name);
          if (!name[1]) {
            name = name[2];
            return (item, index, length) => name === 'first' ? index === 0 : name === 'last' ? index === length - 1 : name === 'even' ? index % 2 === 0 : name === 'odd' ? index % 2 === 1 : item[name] ? item[name]() : false;
          }
          notSelectors = parseChunks(name[1], []);
          return item => !match(item, notSelectors);
        }
      }
      function compile(selector, filters, direct) {
        let parts;
        function add(filter) {
          if (filter) {
            filters.push(filter);
          }
        }
        parts = expression.exec(selector.replace(whiteSpace, ''));
        add(compileNameFilter(parts[1]));
        add(compileIdFilter(parts[2]));
        add(compileClassesFilter(parts[3]));
        add(compileAttrFilter(parts[4], parts[5], parts[6]));
        add(compilePsuedoFilter(parts[7]));
        filters.pseudo = !!parts[7];
        filters.direct = direct;
        return filters;
      }
      function parseChunks(selector, selectors) {
        const parts = [];
        let extra, matches, i;
        do {
          chunker.exec('');
          matches = chunker.exec(selector);
          if (matches) {
            selector = matches[3];
            parts.push(matches[1]);
            if (matches[2]) {
              extra = matches[3];
              break;
            }
          }
        } while (matches);
        if (extra) {
          parseChunks(extra, selectors);
        }
        selector = [];
        for (i = 0; i < parts.length; i++) {
          if (parts[i] !== '>') {
            selector.push(compile(parts[i], [], parts[i - 1] === '>'));
          }
        }
        selectors.push(selector);
        return selectors;
      }
      this._selectors = parseChunks(selector, []);
    },
    match(control, selectors) {
      let i, l, si, sl, selector, fi, fl, filters, index, length, siblings, count, item;
      selectors = selectors || this._selectors;
      for (i = 0, l = selectors.length; i < l; i++) {
        selector = selectors[i];
        sl = selector.length;
        item = control;
        count = 0;
        for (si = sl - 1; si >= 0; si--) {
          filters = selector[si];
          while (item) {
            if (filters.pseudo) {
              siblings = item.parent().items();
              index = length = siblings.length;
              while (index--) {
                if (siblings[index] === item) {
                  break;
                }
              }
            }
            for (fi = 0, fl = filters.length; fi < fl; fi++) {
              if (!filters[fi](item, index, length)) {
                fi = fl + 1;
                break;
              }
            }
            if (fi === fl) {
              count++;
              break;
            } else {
              if (si === sl - 1) {
                break;
              }
            }
            item = item.parent();
          }
        }
        if (count === sl) {
          return true;
        }
      }
      return false;
    },
    find(container) {
      let matches = [], i, l;
      const selectors = this._selectors;
      function collect(items, selector, index) {
        let i, l, fi, fl, item;
        const filters = selector[index];
        for (i = 0, l = items.length; i < l; i++) {
          item = items[i];
          for (fi = 0, fl = filters.length; fi < fl; fi++) {
            if (!filters[fi](item, i, l)) {
              fi = fl + 1;
              break;
            }
          }
          if (fi === fl) {
            if (index === selector.length - 1) {
              matches.push(item);
            } else {
              if (item.items) {
                collect(item.items(), selector, index + 1);
              }
            }
          } else if (filters.direct) {
            return;
          }
          if (item.items) {
            collect(item.items(), selector, index);
          }
        }
      }
      if (container.items) {
        for (i = 0, l = selectors.length; i < l; i++) {
          collect(container.items(), selectors[i], 0);
        }
        if (l > 1) {
          matches = unique(matches);
        }
      }
      if (!Collection) {
        Collection = Selector.Collection;
      }
      return new Collection(matches);
    }
  });

  let Collection$1;
  let proto;
  const push$1 = Array.prototype.push;
  const slice$1 = Array.prototype.slice;
  proto = {
    length: 0,
    init(items) {
      if (items) {
        this.add(items);
      }
    },
    add(items) {
      const self = this;
      if (!Tools.isArray(items)) {
        if (items instanceof Collection$1) {
          self.add(items.toArray());
        } else {
          push$1.call(self, items);
        }
      } else {
        push$1.apply(self, items);
      }
      return self;
    },
    set(items) {
      const self = this;
      const len = self.length;
      let i;
      self.length = 0;
      self.add(items);
      for (i = self.length; i < len; i++) {
        delete self[i];
      }
      return self;
    },
    filter(selector) {
      const self = this;
      let i, l;
      const matches = [];
      let item, match;
      if (typeof selector === 'string') {
        selector = new Selector(selector);
        match = item => selector.match(item);
      } else {
        match = selector;
      }
      for (i = 0, l = self.length; i < l; i++) {
        item = self[i];
        if (match(item)) {
          matches.push(item);
        }
      }
      return new Collection$1(matches);
    },
    slice(...args) {
      return new Collection$1(slice$1.apply(this, args));
    },
    eq(index) {
      return index === -1 ? this.slice(index) : this.slice(index, +index + 1);
    },
    each(callback) {
      Tools.each(this, callback);
      return this;
    },
    toArray() {
      return Tools.toArray(this);
    },
    indexOf(ctrl) {
      const self = this;
      let i = self.length;
      while (i--) {
        if (self[i] === ctrl) {
          break;
        }
      }
      return i;
    },
    reverse() {
      return new Collection$1(Tools.toArray(this).reverse());
    },
    hasClass(cls) {
      return this[0] ? this[0].classes.contains(cls) : false;
    },
    prop(name, value) {
      const self = this;
      let item;
      if (value !== undefined) {
        self.each(item => {
          if (item[name]) {
            item[name](value);
          }
        });
        return self;
      }
      item = self[0];
      if (item && item[name]) {
        return item[name]();
      }
    },
    exec(name) {
      const self = this, args = Tools.toArray(arguments).slice(1);
      self.each(item => {
        if (item[name]) {
          item[name](...args);
        }
      });
      return self;
    },
    remove() {
      let i = this.length;
      while (i--) {
        this[i].remove();
      }
      return this;
    },
    addClass(cls) {
      return this.each(item => {
        item.classes.add(cls);
      });
    },
    removeClass(cls) {
      return this.each(item => {
        item.classes.remove(cls);
      });
    }
  };
  Tools.each('fire on off show hide append prepend before after reflow'.split(' '), name => {
    proto[name] = function () {
      const args = Tools.toArray(arguments);
      this.each(ctrl => {
        if (name in ctrl) {
          ctrl[name](...args);
        }
      });
      return this;
    };
  });
  Tools.each('text name disabled active selected checked visible parent value data'.split(' '), name => {
    proto[name] = function (value) {
      return this.prop(name, value);
    };
  });
  Collection$1 = global$10.extend(proto);
  Selector.Collection = Collection$1;
  const Collection$2 = Collection$1;

  const Binding = function (settings) {
    this.create = settings.create;
  };
  Binding.create = (model, name) => new Binding({
    create(otherModel, otherName) {
      let bindings;
      const fromSelfToOther = e => {
        otherModel.set(otherName, e.value);
      };
      const fromOtherToSelf = e => {
        model.set(name, e.value);
      };
      otherModel.on('change:' + otherName, fromOtherToSelf);
      model.on('change:' + name, fromSelfToOther);
      bindings = otherModel._bindings;
      if (!bindings) {
        bindings = otherModel._bindings = [];
        otherModel.on('destroy', () => {
          let i = bindings.length;
          while (i--) {
            bindings[i]();
          }
        });
      }
      bindings.push(() => {
        model.off('change:' + name, fromSelfToOther);
      });
      return model.get(name);
    }
  });

  const global$12 = tinymce.util.Tools.resolve('tinymce.util.Observable');

  function isNode(node) {
    return node.nodeType > 0;
  }
  function isEqual(a, b) {
    let k, checked;
    if (a === b) {
      return true;
    }
    if (a === null || b === null) {
      return a === b;
    }
    if (typeof a !== 'object' || typeof b !== 'object') {
      return a === b;
    }
    if (Tools.isArray(b)) {
      if (a.length !== b.length) {
        return false;
      }
      k = a.length;
      while (k--) {
        if (!isEqual(a[k], b[k])) {
          return false;
        }
      }
    }
    if (isNode(a) || isNode(b)) {
      return a === b;
    }
    checked = {};
    for (k in b) {
      if (!isEqual(a[k], b[k])) {
        return false;
      }
      checked[k] = true;
    }
    for (k in a) {
      if (!checked[k] && !isEqual(a[k], b[k])) {
        return false;
      }
    }
    return true;
  }
  const ObservableObject = global$10.extend({
    Mixins: [global$12],
    init(data) {
      let name, value;
      data = data || {};
      for (name in data) {
        value = data[name];
        if (value instanceof Binding) {
          data[name] = value.create(this, name);
        }
      }
      this.data = data;
    },
    set(name, value) {
      let key, args;
      const oldValue = this.data[name];
      if (value instanceof Binding) {
        value = value.create(this, name);
      }
      if (typeof name === 'object') {
        for (key in name) {
          this.set(key, name[key]);
        }
        return this;
      }
      if (!isEqual(oldValue, value)) {
        this.data[name] = value;
        args = {
          target: this,
          name,
          value,
          oldValue
        };
        this.fire('change:' + name, args);
        this.fire('change', args);
      }
      return this;
    },
    get(name) {
      return this.data[name];
    },
    has(name) {
      return name in this.data;
    },
    bind(name) {
      return Binding.create(this, name);
    },
    destroy() {
      this.fire('destroy');
    }
  });

  let dirtyCtrls = {};
  let animationFrameRequested;
  const $_8awxqftqjfjm4ne5 = {
    add(ctrl) {
      const parent = ctrl.parent();
      if (parent) {
        if (!parent._layout || parent._layout.isNative()) {
          return;
        }
        if (!dirtyCtrls[parent._id]) {
          dirtyCtrls[parent._id] = parent;
        }
        if (!animationFrameRequested) {
          animationFrameRequested = true;
          Delay.requestAnimationFrame(() => {
            let id, ctrl;
            animationFrameRequested = false;
            for (id in dirtyCtrls) {
              ctrl = dirtyCtrls[id];
              if (ctrl.state.get('rendered')) {
                ctrl.reflow();
              }
            }
            dirtyCtrls = {};
          }, document.body);
        }
      }
    },
    remove(ctrl) {
      if (dirtyCtrls[ctrl._id]) {
        delete dirtyCtrls[ctrl._id];
      }
    }
  };

  const hasMouseWheelEventSupport = 'onmousewheel' in document;
  const hasWheelEventSupport = false;
  const classPrefix = 'mce-';
  let Control;
  let idCounter = 0;
  const proto$1 = {
    Statics: { classPrefix },
    isRtl() {
      return Control.rtl;
    },
    classPrefix,
    init(settings) {
      const self = this;
      let classes, defaultClasses;
      function applyClasses(classes) {
        let i;
        classes = classes.split(' ');
        for (i = 0; i < classes.length; i++) {
          self.classes.add(classes[i]);
        }
      }
      self.settings = settings = Tools.extend({}, self.Defaults, settings);
      self._id = settings.id || 'mceu_' + idCounter++;
      self._aria = { role: settings.role };
      self._elmCache = {};
      self.$ = global$9;
      self.state = new ObservableObject({
        visible: true,
        active: false,
        disabled: false,
        value: ''
      });
      self.data = new ObservableObject(settings.data);
      self.classes = new ClassList(function () {
        if (self.state.get('rendered')) {
          self.getEl().className = this.toString();
        }
      });
      self.classes.prefix = self.classPrefix;
      classes = settings.classes;
      if (classes) {
        if (self.Defaults) {
          defaultClasses = self.Defaults.classes;
          if (defaultClasses && classes !== defaultClasses) {
            applyClasses(defaultClasses);
          }
        }
        applyClasses(classes);
      }
      Tools.each('title text name visible disabled active value'.split(' '), name => {
        if (name in settings) {
          self[name](settings[name]);
        }
      });
      self.on('click', () => {
        if (self.disabled()) {
          return false;
        }
      });
      self.settings = settings;
      self.borderBox = $_bty1r7tjjfjm4ndm.parseBox(settings.border);
      self.paddingBox = $_bty1r7tjjfjm4ndm.parseBox(settings.padding);
      self.marginBox = $_bty1r7tjjfjm4ndm.parseBox(settings.margin);
      if (settings.hidden) {
        self.hide();
      }
    },
    Properties: 'parent,name',
    getContainerElm() {
      const uiContainer = $_esvtriszjfjm4nba.getUiContainer(this);
      return uiContainer ? uiContainer : funcs.getContainer();
    },
    getParentCtrl(elm) {
      let ctrl;
      const lookup = this.getRoot().controlIdLookup;
      while (elm && lookup) {
        ctrl = lookup[elm.id];
        if (ctrl) {
          break;
        }
        elm = elm.parentNode;
      }
      return ctrl;
    },
    initLayoutRect() {
      const self = this;
      const settings = self.settings;
      let borderBox, layoutRect;
      const elm = self.getEl();
      let width, height, minWidth, minHeight, autoResize;
      let startMinWidth, startMinHeight, initialSize;
      borderBox = self.borderBox = self.borderBox || $_bty1r7tjjfjm4ndm.measureBox(elm, 'border');
      self.paddingBox = self.paddingBox || $_bty1r7tjjfjm4ndm.measureBox(elm, 'padding');
      self.marginBox = self.marginBox || $_bty1r7tjjfjm4ndm.measureBox(elm, 'margin');
      initialSize = funcs.getSize(elm);
      startMinWidth = settings.minWidth;
      startMinHeight = settings.minHeight;
      minWidth = startMinWidth || initialSize.width;
      minHeight = startMinHeight || initialSize.height;
      width = settings.width;
      height = settings.height;
      autoResize = settings.autoResize;
      autoResize = typeof autoResize !== 'undefined' ? autoResize : !width && !height;
      width = width || minWidth;
      height = height || minHeight;
      const deltaW = borderBox.left + borderBox.right;
      const deltaH = borderBox.top + borderBox.bottom;
      const maxW = settings.maxWidth || 65535;
      const maxH = settings.maxHeight || 65535;
      self._layoutRect = layoutRect = {
        x: settings.x || 0,
        y: settings.y || 0,
        w: width,
        h: height,
        deltaW,
        deltaH,
        contentW: width - deltaW,
        contentH: height - deltaH,
        innerW: width - deltaW,
        innerH: height - deltaH,
        startMinWidth: startMinWidth || 0,
        startMinHeight: startMinHeight || 0,
        minW: Math.min(minWidth, maxW),
        minH: Math.min(minHeight, maxH),
        maxW,
        maxH,
        autoResize,
        scrollW: 0
      };
      self._lastLayoutRect = {};
      return layoutRect;
    },
    layoutRect(newRect) {
      const self = this;
      let curRect = self._layoutRect, lastLayoutRect, size, deltaWidth, deltaHeight, repaintControls;
      if (!curRect) {
        curRect = self.initLayoutRect();
      }
      if (newRect) {
        deltaWidth = curRect.deltaW;
        deltaHeight = curRect.deltaH;
        if (newRect.x !== undefined) {
          curRect.x = newRect.x;
        }
        if (newRect.y !== undefined) {
          curRect.y = newRect.y;
        }
        if (newRect.minW !== undefined) {
          curRect.minW = newRect.minW;
        }
        if (newRect.minH !== undefined) {
          curRect.minH = newRect.minH;
        }
        size = newRect.w;
        if (size !== undefined) {
          size = size < curRect.minW ? curRect.minW : size;
          size = size > curRect.maxW ? curRect.maxW : size;
          curRect.w = size;
          curRect.innerW = size - deltaWidth;
        }
        size = newRect.h;
        if (size !== undefined) {
          size = size < curRect.minH ? curRect.minH : size;
          size = size > curRect.maxH ? curRect.maxH : size;
          curRect.h = size;
          curRect.innerH = size - deltaHeight;
        }
        size = newRect.innerW;
        if (size !== undefined) {
          size = size < curRect.minW - deltaWidth ? curRect.minW - deltaWidth : size;
          size = size > curRect.maxW - deltaWidth ? curRect.maxW - deltaWidth : size;
          curRect.innerW = size;
          curRect.w = size + deltaWidth;
        }
        size = newRect.innerH;
        if (size !== undefined) {
          size = size < curRect.minH - deltaHeight ? curRect.minH - deltaHeight : size;
          size = size > curRect.maxH - deltaHeight ? curRect.maxH - deltaHeight : size;
          curRect.innerH = size;
          curRect.h = size + deltaHeight;
        }
        if (newRect.contentW !== undefined) {
          curRect.contentW = newRect.contentW;
        }
        if (newRect.contentH !== undefined) {
          curRect.contentH = newRect.contentH;
        }
        lastLayoutRect = self._lastLayoutRect;
        if (lastLayoutRect.x !== curRect.x || lastLayoutRect.y !== curRect.y || lastLayoutRect.w !== curRect.w || lastLayoutRect.h !== curRect.h) {
          repaintControls = Control.repaintControls;
          if (repaintControls) {
            if (repaintControls.map && !repaintControls.map[self._id]) {
              repaintControls.push(self);
              repaintControls.map[self._id] = true;
            }
          }

          lastLayoutRect.x = curRect.x;
          lastLayoutRect.y = curRect.y;
          lastLayoutRect.w = curRect.w;
          lastLayoutRect.h = curRect.h;
        }
        return self;
      }
      return curRect;
    },
    repaint() {
      const self = this;
      let style, bodyStyle, bodyElm, rect, borderBox;
      let borderW, borderH, lastRepaintRect, round, value;
      round = !document.createRange ? Math.round : value => value;
      style = self.getEl().style;
      rect = self._layoutRect;
      lastRepaintRect = self._lastRepaintRect || {};
      borderBox = self.borderBox;
      borderW = borderBox.left + borderBox.right;
      borderH = borderBox.top + borderBox.bottom;

      if (rect.x !== lastRepaintRect.x) {
        style.left = round(rect.x) + 'px';
        lastRepaintRect.x = rect.x;
      }
      if (rect.y !== lastRepaintRect.y) {
        style.top = round(rect.y) + 'px';
        lastRepaintRect.y = rect.y;
      }
      if (rect.w !== lastRepaintRect.w) {
        value = round(rect.w - borderW);
        style.width = (value >= 0 ? value : 0) + 'px';
        lastRepaintRect.w = rect.w;
      }
      if (rect.h !== lastRepaintRect.h) {
        value = round(rect.h - borderH);
        style.height = (value >= 0 ? value : 0) + 'px';
        lastRepaintRect.h = rect.h;
      }
      if (self._hasBody && rect.innerW !== lastRepaintRect.innerW) {
        value = round(rect.innerW);
        bodyElm = self.getEl('body');
        if (bodyElm) {
          bodyStyle = bodyElm.style;
          bodyStyle.width = (value >= 0 ? value : 0) + 'px';
        }
        lastRepaintRect.innerW = rect.innerW;
      }
      if (self._hasBody && rect.innerH !== lastRepaintRect.innerH) {
        value = round(rect.innerH);
        bodyElm = bodyElm || self.getEl('body');
        if (bodyElm) {
          bodyStyle = bodyStyle || bodyElm.style;
          bodyStyle.height = (value >= 0 ? value : 0) + 'px';
        }
        lastRepaintRect.innerH = rect.innerH;
      }
      self._lastRepaintRect = lastRepaintRect;
      self.fire('repaint', {}, false);
    },
    updateLayoutRect() {
      const self = this;
      self.parent()._lastRect = null;
      funcs.css(self.getEl(), {
        width: '',
        height: ''
      });
      self._layoutRect = self._lastRepaintRect = self._lastLayoutRect = null;
      self.initLayoutRect();
    },
    on(name, callback) {
      const self = this;
      function resolveCallbackName(name) {
        let callback, scope;
        if (typeof name !== 'string') {
          return name;
        }
        return function (e) {
          if (!callback) {
            self.parentsAndSelf().each(ctrl => {
              const callbacks = ctrl.settings.callbacks;
              if (callbacks && (callback = callbacks[name])) {
                scope = ctrl;
                return false;
              }
            });
          }
          if (!callback) {
            e.action = name;
            this.fire('execute', e);
            return;
          }
          return callback.call(scope, e);
        };
      }
      getEventDispatcher(self).on(name, resolveCallbackName(callback));
      return self;
    },
    off(name, callback) {
      getEventDispatcher(this).off(name, callback);
      return this;
    },
    fire(name, args, bubble) {
      const self = this;
      args = args || {};
      if (!args.control) {
        args.control = self;
      }
      args = getEventDispatcher(self).fire(name, args);
      if (bubble !== false && self.parent) {
        let parent_1 = self.parent();
        while (parent_1 && !args.isPropagationStopped()) {
          parent_1.fire(name, args, false);
          parent_1 = parent_1.parent();
        }
      }
      return args;
    },
    hasEventListeners(name) {
      return getEventDispatcher(this).has(name);
    },
    parents(selector) {
      const self = this;
      let ctrl, parents = new Collection$2();
      for (ctrl = self.parent(); ctrl; ctrl = ctrl.parent()) {
        parents.add(ctrl);
      }
      if (selector) {
        parents = parents.filter(selector);
      }
      return parents;
    },
    parentsAndSelf(selector) {
      return new Collection$2(this).add(this.parents(selector));
    },
    next() {
      const parentControls = this.parent().items();
      return parentControls[parentControls.indexOf(this) + 1];
    },
    prev() {
      const parentControls = this.parent().items();
      return parentControls[parentControls.indexOf(this) - 1];
    },
    innerHtml(html) {
      this.$el.html(html);
      return this;
    },
    getEl(suffix) {
      const id = suffix ? this._id + '-' + suffix : this._id;
      if (!this._elmCache[id]) {
        this._elmCache[id] = global$9('#' + id)[0];
      }
      return this._elmCache[id];
    },
    show() {
      return this.visible(true);
    },
    hide() {
      return this.visible(false);
    },
    focus() {
      try {
        this.getEl().focus();
      } catch (ex) {
      }
      return this;
    },
    blur() {
      this.getEl().blur();
      return this;
    },
    aria(name, value) {
      const self = this, elm = self.getEl(self.ariaTarget);
      if (typeof value === 'undefined') {
        return self._aria[name];
      }
      self._aria[name] = value;
      if (self.state.get('rendered')) {
        elm.setAttribute(name === 'role' ? name : 'aria-' + name, value);
      }
      return self;
    },
    encode(text, translate) {
      if (translate !== false) {
        text = this.translate(text);
      }
      return (text || '').replace(/[&<>"]/g, match => '&#' + match.charCodeAt(0) + ';');
    },
    translate(text) {
      return Control.translate ? Control.translate(text) : text;
    },
    before(items) {
      const self = this, parent = self.parent();
      if (parent) {
        parent.insert(items, parent.items().indexOf(self), true);
      }
      return self;
    },
    after(items) {
      const self = this, parent = self.parent();
      if (parent) {
        parent.insert(items, parent.items().indexOf(self));
      }
      return self;
    },
    remove() {
      const self = this;
      const elm = self.getEl();
      const parent = self.parent();
      let newItems, i;
      if (self.items) {
        const controls = self.items().toArray();
        i = controls.length;
        while (i--) {
          controls[i].remove();
        }
      }
      if (parent && parent.items) {
        newItems = [];
        parent.items().each(item => {
          if (item !== self) {
            newItems.push(item);
          }
        });
        parent.items().set(newItems);
        parent._lastRect = null;
      }
      if (self._eventsRoot && self._eventsRoot === self) {
        global$9(elm).off();
      }
      const lookup = self.getRoot().controlIdLookup;
      if (lookup) {
        delete lookup[self._id];
      }
      if (elm && elm.parentNode) {
        elm.parentNode.removeChild(elm);
      }
      self.state.set('rendered', false);
      self.state.destroy();
      self.fire('remove');
      return self;
    },
    renderBefore(elm) {
      global$9(elm).before(this.renderHtml());
      this.postRender();
      return this;
    },
    renderTo(elm) {
      global$9(elm || this.getContainerElm()).append(this.renderHtml());
      this.postRender();
      return this;
    },
    preRender() {
    },
    render() {
    },
    renderHtml() {
      return '<div id="' + this._id + '" class="' + this.classes + '"></div>';
    },
    postRender() {
      const self = this;
      const settings = self.settings;
      let elm, box, parent, name, parentEventsRoot;
      self.$el = global$9(self.getEl());
      self.state.set('rendered', true);
      for (name in settings) {
        if (name.indexOf('on') === 0) {
          self.on(name.substr(2), settings[name]);
        }
      }
      if (self._eventsRoot) {
        for (parent = self.parent(); !parentEventsRoot && parent; parent = parent.parent()) {
          parentEventsRoot = parent._eventsRoot;
        }
        if (parentEventsRoot) {
          for (name in parentEventsRoot._nativeEvents) {
            self._nativeEvents[name] = true;
          }
        }
      }
      bindPendingEvents(self);
      if (settings.style) {
        elm = self.getEl();
        if (elm) {
          elm.setAttribute('style', settings.style);
          elm.style.cssText = settings.style;
        }
      }
      if (self.settings.border) {
        box = self.borderBox;
        self.$el.css({
          'border-top-width': box.top,
          'border-right-width': box.right,
          'border-bottom-width': box.bottom,
          'border-left-width': box.left
        });
      }
      const root = self.getRoot();
      if (!root.controlIdLookup) {
        root.controlIdLookup = {};
      }
      root.controlIdLookup[self._id] = self;
      for (const key in self._aria) {
        self.aria(key, self._aria[key]);
      }
      if (self.state.get('visible') === false) {
        self.getEl().style.display = 'none';
      }
      self.bindStates();
      self.state.on('change:visible', e => {
        const state = e.value;
        let parentCtrl;
        if (self.state.get('rendered')) {
          self.getEl().style.display = state === false ? 'none' : '';
          self.getEl().getBoundingClientRect();
        }
        parentCtrl = self.parent();
        if (parentCtrl) {
          parentCtrl._lastRect = null;
        }
        self.fire(state ? 'show' : 'hide');
        $_8awxqftqjfjm4ne5.add(self);
      });
      self.fire('postrender', {}, false);
    },
    bindStates() {
    },
    scrollIntoView(align) {
      function getOffset(elm, rootElm) {
        let x, y, parent = elm;
        x = y = 0;
        while (parent && parent !== rootElm && parent.nodeType) {
          x += parent.offsetLeft || 0;
          y += parent.offsetTop || 0;
          parent = parent.offsetParent;
        }
        return {
          x,
          y
        };
      }
      const elm = this.getEl(), parentElm = elm.parentNode;
      let x, y, width, height, parentWidth, parentHeight;
      const pos = getOffset(elm, parentElm);
      x = pos.x;
      y = pos.y;
      width = elm.offsetWidth;
      height = elm.offsetHeight;
      parentWidth = parentElm.clientWidth;
      parentHeight = parentElm.clientHeight;
      if (align === 'end') {
        x -= parentWidth - width;
        y -= parentHeight - height;
      } else if (align === 'center') {
        x -= parentWidth / 2 - width / 2;
        y -= parentHeight / 2 - height / 2;
      }
      parentElm.scrollLeft = x;
      parentElm.scrollTop = y;
      return this;
    },
    getRoot() {
      let ctrl = this, rootControl;
      const parents = [];
      while (ctrl) {
        if (ctrl.rootControl) {
          rootControl = ctrl.rootControl;
          break;
        }
        parents.push(ctrl);
        rootControl = ctrl;
        ctrl = ctrl.parent();
      }
      if (!rootControl) {
        rootControl = this;
      }
      let i = parents.length;
      while (i--) {
        parents[i].rootControl = rootControl;
      }
      return rootControl;
    },
    reflow() {
      $_8awxqftqjfjm4ne5.remove(this);
      const parent = this.parent();
      if (parent && parent._layout && !parent._layout.isNative()) {
        parent.reflow();
      }
      return this;
    }
  };
  Tools.each('text title visible disabled active value'.split(' '), name => {
    proto$1[name] = function (value) {
      if (arguments.length === 0) {
        return this.state.get(name);
      }
      if (typeof value !== 'undefined') {
        this.state.set(name, value);
      }
      return this;
    };
  });
  Control = global$10.extend(proto$1);
  function getEventDispatcher(obj) {
    if (!obj._eventDispatcher) {
      obj._eventDispatcher = new global$11({
        scope: obj,
        toggleEvent(name, state) {
          if (state && global$11.isNative(name)) {
            if (!obj._nativeEvents) {
              obj._nativeEvents = {};
            }
            obj._nativeEvents[name] = true;
            if (obj.state.get('rendered')) {
              bindPendingEvents(obj);
            }
          }
        }
      });
    }
    return obj._eventDispatcher;
  }
  function bindPendingEvents(eventCtrl) {
    let i, l, parents, eventRootCtrl, nativeEvents, name;
    function delegate(e) {
      const control = eventCtrl.getParentCtrl(e.target);
      if (control) {
        control.fire(e.type, e);
      }
    }
    function mouseLeaveHandler() {
      const ctrl = eventRootCtrl._lastHoverCtrl;
      if (ctrl) {
        ctrl.fire('mouseleave', { target: ctrl.getEl() });
        ctrl.parents().each(ctrl => {
          ctrl.fire('mouseleave', { target: ctrl.getEl() });
        });
        eventRootCtrl._lastHoverCtrl = null;
      }
    }
    function mouseEnterHandler(e) {
      let ctrl = eventCtrl.getParentCtrl(e.target), lastCtrl = eventRootCtrl._lastHoverCtrl, idx = 0, i, parents, lastParents;
      if (ctrl !== lastCtrl) {
        eventRootCtrl._lastHoverCtrl = ctrl;
        parents = ctrl.parents().toArray().reverse();
        parents.push(ctrl);
        if (lastCtrl) {
          lastParents = lastCtrl.parents().toArray().reverse();
          lastParents.push(lastCtrl);
          for (idx = 0; idx < lastParents.length; idx++) {
            if (parents[idx] !== lastParents[idx]) {
              break;
            }
          }
          for (i = lastParents.length - 1; i >= idx; i--) {
            lastCtrl = lastParents[i];
            lastCtrl.fire('mouseleave', { target: lastCtrl.getEl() });
          }
        }
        for (i = idx; i < parents.length; i++) {
          ctrl = parents[i];
          ctrl.fire('mouseenter', { target: ctrl.getEl() });
        }
      }
    }
    function fixWheelEvent(e) {
      e.preventDefault();
      if (e.type === 'mousewheel') {
        e.deltaY = -1 / 40 * e.wheelDelta;
        if (e.wheelDeltaX) {
          e.deltaX = -1 / 40 * e.wheelDeltaX;
        }
      } else {
        e.deltaX = 0;
        e.deltaY = e.detail;
      }
      e = eventCtrl.fire('wheel', e);
    }
    nativeEvents = eventCtrl._nativeEvents;
    if (nativeEvents) {
      parents = eventCtrl.parents().toArray();
      parents.unshift(eventCtrl);
      for (i = 0, l = parents.length; !eventRootCtrl && i < l; i++) {
        eventRootCtrl = parents[i]._eventsRoot;
      }
      if (!eventRootCtrl) {
        eventRootCtrl = parents[parents.length - 1] || eventCtrl;
      }
      eventCtrl._eventsRoot = eventRootCtrl;
      for (l = i, i = 0; i < l; i++) {
        parents[i]._eventsRoot = eventRootCtrl;
      }
      let eventRootDelegates = eventRootCtrl._delegates;
      if (!eventRootDelegates) {
        eventRootDelegates = eventRootCtrl._delegates = {};
      }
      for (name in nativeEvents) {
        if (!nativeEvents) {
          return false;
        }
        if (name === 'wheel' && !hasWheelEventSupport) {
          if (hasMouseWheelEventSupport) {
            global$9(eventCtrl.getEl()).on('mousewheel', fixWheelEvent);
          } else {
            global$9(eventCtrl.getEl()).on('DOMMouseScroll', fixWheelEvent);
          }
          continue;
        }
        if (name === 'mouseenter' || name === 'mouseleave') {
          if (!eventRootCtrl._hasMouseEnter) {
            global$9(eventRootCtrl.getEl()).on('mouseleave', mouseLeaveHandler).on('mouseover', mouseEnterHandler);
            eventRootCtrl._hasMouseEnter = 1;
          }
        } else if (!eventRootDelegates[name]) {
          global$9(eventRootCtrl.getEl()).on(name, delegate);
          eventRootDelegates[name] = true;
        }
        nativeEvents[name] = false;
      }
    }
  }
  const Control$1 = Control;

  const hasTabstopData = elm => elm.getAttribute('data-mce-tabstop') ? true : false;
  function KeyboardNavigation (settings) {
    const root = settings.root;
    let focusedElement, focusedControl;
    function isElement(node) {
      return node && node.nodeType === 1;
    }
    try {
      focusedElement = document.activeElement;
    } catch (ex) {
      focusedElement = document.body;
    }
    focusedControl = root.getParentCtrl(focusedElement);
    function getRole(elm) {
      elm = elm || focusedElement;
      if (isElement(elm)) {
        return elm.getAttribute('role');
      }
      return null;
    }
    function getParentRole(elm) {
      let role, parent = elm || focusedElement;
      while (parent = parent.parentNode) {
        if (role = getRole(parent)) {
          return role;
        }
      }
    }
    function getAriaProp(name) {
      const elm = focusedElement;
      if (isElement(elm)) {
        return elm.getAttribute('aria-' + name);
      }
    }
    function isTextInputElement(elm) {
      const tagName = elm.tagName.toUpperCase();
      return tagName === 'INPUT' || tagName === 'TEXTAREA' || tagName === 'SELECT';
    }
    function canFocus(elm) {
      if (isTextInputElement(elm) && !elm.hidden) {
        return true;
      }
      if (hasTabstopData(elm)) {
        return true;
      }
      if (/^(button|menuitem|checkbox|tab|menuitemcheckbox|option|gridcell|slider)$/.test(getRole(elm))) {
        return true;
      }
      return false;
    }
    function getFocusElements(elm) {
      const elements = [];
      function collect(elm) {
        if (elm.nodeType !== 1 || elm.style.display === 'none' || elm.disabled) {
          return;
        }
        if (canFocus(elm)) {
          elements.push(elm);
        }
        for (let i = 0; i < elm.childNodes.length; i++) {
          collect(elm.childNodes[i]);
        }
      }
      collect(elm || root.getEl());
      return elements;
    }
    function getNavigationRoot(targetControl) {
      let navigationRoot, controls;
      targetControl = targetControl || focusedControl;
      controls = targetControl.parents().toArray();
      controls.unshift(targetControl);
      for (let i = 0; i < controls.length; i++) {
        navigationRoot = controls[i];
        if (navigationRoot.settings.ariaRoot) {
          break;
        }
      }
      return navigationRoot;
    }
    function focusFirst(targetControl) {
      const navigationRoot = getNavigationRoot(targetControl);
      const focusElements = getFocusElements(navigationRoot.getEl());
      if (navigationRoot.settings.ariaRemember && 'lastAriaIndex' in navigationRoot) {
        moveFocusToIndex(navigationRoot.lastAriaIndex, focusElements);
      } else {
        moveFocusToIndex(0, focusElements);
      }
    }
    function moveFocusToIndex(idx, elements) {
      if (idx < 0) {
        idx = elements.length - 1;
      } else if (idx >= elements.length) {
        idx = 0;
      }
      if (elements[idx]) {
        elements[idx].focus();
      }
      return idx;
    }
    function moveFocus(dir, elements) {
      let idx = -1;
      const navigationRoot = getNavigationRoot();
      elements = elements || getFocusElements(navigationRoot.getEl());
      for (let i = 0; i < elements.length; i++) {
        if (elements[i] === focusedElement) {
          idx = i;
        }
      }
      idx += dir;
      navigationRoot.lastAriaIndex = moveFocusToIndex(idx, elements);
    }
    function left() {
      const parentRole = getParentRole();
      if (parentRole === 'tablist') {
        moveFocus(-1, getFocusElements(focusedElement.parentNode));
      } else if (focusedControl.parent().submenu) {
        cancel();
      } else {
        moveFocus(-1);
      }
    }
    function right() {
      const role = getRole(), parentRole = getParentRole();
      if (parentRole === 'tablist') {
        moveFocus(1, getFocusElements(focusedElement.parentNode));
      } else if (role === 'menuitem' && parentRole === 'menu' && getAriaProp('haspopup')) {
        enter();
      } else {
        moveFocus(1);
      }
    }
    function up() {
      moveFocus(-1);
    }
    function down() {
      const role = getRole(), parentRole = getParentRole();
      if (role === 'menuitem' && parentRole === 'menubar') {
        enter();
      } else if (role === 'button' && getAriaProp('haspopup')) {
        enter({ key: 'down' });
      } else {
        moveFocus(1);
      }
    }
    function tab(e) {
      const parentRole = getParentRole();
      if (parentRole === 'tablist') {
        const elm = getFocusElements(focusedControl.getEl('body'))[0];
        if (elm) {
          elm.focus();
        }
      } else {
        moveFocus(e.shiftKey ? -1 : 1);
      }
    }
    function cancel() {
      focusedControl.fire('cancel');
    }
    function enter(aria) {
      aria = aria || {};
      focusedControl.fire('click', {
        target: focusedElement,
        aria
      });
    }
    root.on('keydown', e => {
      function handleNonTabOrEscEvent(e, handler) {
        if (isTextInputElement(focusedElement) || hasTabstopData(focusedElement)) {
          return;
        }
        if (getRole(focusedElement) === 'slider') {
          return;
        }
        if (handler(e) !== false) {
          e.preventDefault();
        }
      }
      if (e.isDefaultPrevented()) {
        return;
      }
      switch (e.keyCode) {
      case 37:
        handleNonTabOrEscEvent(e, left);
        break;
      case 39:
        handleNonTabOrEscEvent(e, right);
        break;
      case 38:
        handleNonTabOrEscEvent(e, up);
        break;
      case 40:
        handleNonTabOrEscEvent(e, down);
        break;
      case 27:
        cancel();
        break;
      case 14:
      case 13:
      case 32:
        handleNonTabOrEscEvent(e, enter);
        break;
      case 9:
        tab(e);
        e.preventDefault();
        break;
      }
    });
    root.on('focusin', e => {
      focusedElement = e.target;
      focusedControl = e.control;
    });
    return { focusFirst };
  }

  const selectorCache = {};
  const Container = Control$1.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      settings = self.settings;
      if (settings.fixed) {
        self.state.set('fixed', true);
      }
      self._items = new Collection$2();
      if (self.isRtl()) {
        self.classes.add('rtl');
      }
      self.bodyClasses = new ClassList(function () {
        if (self.state.get('rendered')) {
          self.getEl('body').className = this.toString();
        }
      });
      self.bodyClasses.prefix = self.classPrefix;
      self.classes.add('container');
      self.bodyClasses.add('container-body');
      if (settings.containerCls) {
        self.classes.add(settings.containerCls);
      }
      self._layout = Factory.create((settings.layout || '') + 'layout');
      if (self.settings.items) {
        self.add(self.settings.items);
      } else {
        self.add(self.render());
      }
      self._hasBody = true;
    },
    items() {
      return this._items;
    },
    find(selector) {
      selector = selectorCache[selector] = selectorCache[selector] || new Selector(selector);
      return selector.find(this);
    },
    add(items) {
      const self = this;
      self.items().add(self.create(items)).parent(self);
      return self;
    },
    focus(keyboard) {
      const self = this;
      let focusCtrl, keyboardNav, items;
      if (keyboard) {
        keyboardNav = self.keyboardNav || self.parents().eq(-1)[0].keyboardNav;
        if (keyboardNav) {
          keyboardNav.focusFirst(self);
          return;
        }
      }
      items = self.find('*');
      if (self.statusbar) {
        items.add(self.statusbar.items());
      }
      items.each(ctrl => {
        if (ctrl.settings.autofocus) {
          focusCtrl = null;
          return false;
        }
        if (ctrl.canFocus) {
          focusCtrl = focusCtrl || ctrl;
        }
      });
      if (focusCtrl) {
        focusCtrl.focus();
      }
      return self;
    },
    replace(oldItem, newItem) {
      let ctrlElm;
      const items = this.items();
      let i = items.length;
      while (i--) {
        if (items[i] === oldItem) {
          items[i] = newItem;
          break;
        }
      }
      if (i >= 0) {
        ctrlElm = newItem.getEl();
        if (ctrlElm) {
          ctrlElm.parentNode.removeChild(ctrlElm);
        }
        ctrlElm = oldItem.getEl();
        if (ctrlElm) {
          ctrlElm.parentNode.removeChild(ctrlElm);
        }
      }
      newItem.parent(this);
    },
    create(items) {
      const self = this;
      let settings;
      const ctrlItems = [];
      if (!Tools.isArray(items)) {
        items = [items];
      }
      Tools.each(items, item => {
        if (item) {
          if (!(item instanceof Control$1)) {
            if (typeof item === 'string') {
              item = { type: item };
            }
            settings = Tools.extend({}, self.settings.defaults, item);
            item.type = settings.type = settings.type || item.type || self.settings.defaultType || (settings.defaults ? settings.defaults.type : null);
            item = Factory.create(settings);
          }
          ctrlItems.push(item);
        }
      });
      return ctrlItems;
    },
    renderNew() {
      const self = this;
      self.items().each((ctrl, index) => {
        let containerElm;
        ctrl.parent(self);
        if (!ctrl.state.get('rendered')) {
          containerElm = self.getEl('body');
          if (containerElm.hasChildNodes() && index <= containerElm.childNodes.length - 1) {
            global$9(containerElm.childNodes[index]).before(ctrl.renderHtml());
          } else {
            global$9(containerElm).append(ctrl.renderHtml());
          }
          ctrl.postRender();
          $_8awxqftqjfjm4ne5.add(ctrl);
        }
      });
      self._layout.applyClasses(self.items().filter(':visible'));
      self._lastRect = null;
      return self;
    },
    append(items) {
      return this.add(items).renderNew();
    },
    prepend(items) {
      const self = this;
      self.items().set(self.create(items).concat(self.items().toArray()));
      return self.renderNew();
    },
    insert(items, index, before) {
      const self = this;
      let curItems, beforeItems, afterItems;
      items = self.create(items);
      curItems = self.items();
      if (!before && index < curItems.length - 1) {
        index += 1;
      }
      if (index >= 0 && index < curItems.length) {
        beforeItems = curItems.slice(0, index).toArray();
        afterItems = curItems.slice(index).toArray();
        curItems.set(beforeItems.concat(items, afterItems));
      }
      return self.renderNew();
    },
    fromJSON(data) {
      const self = this;
      for (const name_1 in data) {
        self.find('#' + name_1).value(data[name_1]);
      }
      return self;
    },
    toJSON() {
      const self = this, data = {};
      self.find('*').each(ctrl => {
        const name = ctrl.name(), value = ctrl.value();
        if (name && typeof value !== 'undefined') {
          data[name] = value;
        }
      });
      return data;
    },
    renderHtml() {
      const self = this, layout = self._layout, role = this.settings.role;
      self.preRender();
      layout.preRender(self);
      return '<div id="' + self._id + '" class="' + self.classes + '"' + (role ? ' role="' + this.settings.role + '"' : '') + '>' + '<div id="' + self._id + '-body" class="' + self.bodyClasses + '">' + (self.settings.html || '') + layout.renderHtml(self) + '</div>' + '</div>';
    },
    postRender() {
      const self = this;
      let box;
      self.items().exec('postRender');
      self._super();
      self._layout.postRender(self);
      self.state.set('rendered', true);
      if (self.settings.style) {
        self.$el.css(self.settings.style);
      }
      if (self.settings.border) {
        box = self.borderBox;
        self.$el.css({
          'border-top-width': box.top,
          'border-right-width': box.right,
          'border-bottom-width': box.bottom,
          'border-left-width': box.left
        });
      }
      if (!self.parent()) {
        self.keyboardNav = KeyboardNavigation({ root: self });
      }
      return self;
    },
    initLayoutRect() {
      const self = this, layoutRect = self._super();
      self._layout.recalc(self);
      return layoutRect;
    },
    recalc() {
      const self = this;
      let rect = self._layoutRect;
      const lastRect = self._lastRect;
      if (!lastRect || lastRect.w !== rect.w || lastRect.h !== rect.h) {
        self._layout.recalc(self);
        rect = self.layoutRect();
        self._lastRect = {
          x: rect.x,
          y: rect.y,
          w: rect.w,
          h: rect.h
        };
        return true;
      }
    },
    reflow() {
      let i;
      $_8awxqftqjfjm4ne5.remove(this);
      if (this.visible()) {
        Control$1.repaintControls = [];
        Control$1.repaintControls.map = {};
        this.recalc();
        i = Control$1.repaintControls.length;
        while (i--) {
          Control$1.repaintControls[i].repaint();
        }
        if (this.settings.layout !== 'flow' && this.settings.layout !== 'stack') {
          this.repaint();
        }
        Control$1.repaintControls = [];
      }
      return this;
    }
  });

  function getDocumentSize(doc) {
    let documentElement, body, scrollWidth, clientWidth;
    let offsetWidth, scrollHeight, clientHeight, offsetHeight;
    const max = Math.max;
    documentElement = doc.documentElement;
    body = doc.body;
    scrollWidth = max(documentElement.scrollWidth, body.scrollWidth);
    clientWidth = max(documentElement.clientWidth, body.clientWidth);
    offsetWidth = max(documentElement.offsetWidth, body.offsetWidth);
    scrollHeight = max(documentElement.scrollHeight, body.scrollHeight);
    clientHeight = max(documentElement.clientHeight, body.clientHeight);
    offsetHeight = max(documentElement.offsetHeight, body.offsetHeight);
    return {
      width: scrollWidth < offsetWidth ? clientWidth : scrollWidth,
      height: scrollHeight < offsetHeight ? clientHeight : scrollHeight
    };
  }
  function updateWithTouchData(e) {
    let keys, i;
    if (e.changedTouches) {
      keys = 'screenX screenY pageX pageY clientX clientY'.split(' ');
      for (i = 0; i < keys.length; i++) {
        e[keys[i]] = e.changedTouches[0][keys[i]];
      }
    }
  }
  function DragHelper (id, settings) {
    let $eventOverlay;
    const doc = settings.document || document;
    let downButton;
    let start, stop, drag, startX, startY;
    settings = settings || {};
    const handleElement = doc.getElementById(settings.handle || id);
    start = e => {
      const docSize = getDocumentSize(doc);
      let handleElm, cursor;
      updateWithTouchData(e);
      e.preventDefault();
      downButton = e.button;
      handleElm = handleElement;
      startX = e.screenX;
      startY = e.screenY;
      if (window.getComputedStyle) {
        cursor = window.getComputedStyle(handleElm, null).getPropertyValue('cursor');
      } else {
        cursor = handleElm.runtimeStyle.cursor;
      }
      $eventOverlay = global$9('<div></div>').css({
        position: 'absolute',
        top: 0,
        left: 0,
        width: docSize.width,
        height: docSize.height,
        zIndex: 2147483647,
        opacity: 0.0001,
        cursor
      }).appendTo(doc.body);
      global$9(doc).on('mousemove touchmove', drag).on('mouseup touchend', stop);
      settings.start(e);
    };
    drag = e => {
      updateWithTouchData(e);
      if (e.button !== downButton) {
        return stop(e);
      }
      e.deltaX = e.screenX - startX;
      e.deltaY = e.screenY - startY;
      e.preventDefault();
      settings.drag(e);
    };
    stop = e => {
      updateWithTouchData(e);
      global$9(doc).off('mousemove touchmove', drag).off('mouseup touchend', stop);
      $eventOverlay.remove();
      if (settings.stop) {
        settings.stop(e);
      }
    };
    this.destroy = () => {
      global$9(handleElement).off();
    };
    global$9(handleElement).on('mousedown touchstart', start);
  }

  const $_rcleftsjfjm4nec = {
    init() {
      const self = this;
      self.on('repaint', self.renderScroll);
    },
    renderScroll() {
      const self = this, margin = 2;
      function repaintScroll() {
        let hasScrollH, hasScrollV, bodyElm;
        function repaintAxis(axisName, posName, sizeName, contentSizeName, hasScroll, ax) {
          let containerElm, scrollBarElm, scrollThumbElm;
          let containerSize, scrollSize, ratio, rect;
          let posNameLower, sizeNameLower;
          scrollBarElm = self.getEl('scroll' + axisName);
          if (scrollBarElm) {
            posNameLower = posName.toLowerCase();
            sizeNameLower = sizeName.toLowerCase();
            global$9(self.getEl('absend')).css(posNameLower, self.layoutRect()[contentSizeName] - 1);
            if (!hasScroll) {
              global$9(scrollBarElm).css('display', 'none');
              return;
            }
            global$9(scrollBarElm).css('display', 'block');
            containerElm = self.getEl('body');
            scrollThumbElm = self.getEl('scroll' + axisName + 't');
            containerSize = containerElm['client' + sizeName] - margin * 2;
            containerSize -= hasScrollH && hasScrollV ? scrollBarElm['client' + ax] : 0;
            scrollSize = containerElm['scroll' + sizeName];
            ratio = containerSize / scrollSize;
            rect = {};
            rect[posNameLower] = containerElm['offset' + posName] + margin;
            rect[sizeNameLower] = containerSize;
            global$9(scrollBarElm).css(rect);
            rect = {};
            rect[posNameLower] = containerElm['scroll' + posName] * ratio;
            rect[sizeNameLower] = containerSize * ratio;
            global$9(scrollThumbElm).css(rect);
          }
        }
        bodyElm = self.getEl('body');
        hasScrollH = bodyElm.scrollWidth > bodyElm.clientWidth;
        hasScrollV = bodyElm.scrollHeight > bodyElm.clientHeight;
        repaintAxis('h', 'Left', 'Width', 'contentW', hasScrollH, 'Height');
        repaintAxis('v', 'Top', 'Height', 'contentH', hasScrollV, 'Width');
      }
      function addScroll() {
        function addScrollAxis(axisName, posName, sizeName, deltaPosName, ax) {
          let scrollStart;
          const axisId = self._id + '-scroll' + axisName, prefix = self.classPrefix;
          global$9(self.getEl()).append('<div id="' + axisId + '" class="' + prefix + 'scrollbar ' + prefix + 'scrollbar-' + axisName + '">' + '<div id="' + axisId + 't" class="' + prefix + 'scrollbar-thumb"></div>' + '</div>');
          self.draghelper = new DragHelper(axisId + 't', {
            start() {
              scrollStart = self.getEl('body')['scroll' + posName];
              global$9('#' + axisId).addClass(prefix + 'active');
            },
            drag(e) {
              let ratio, hasScrollH, hasScrollV, containerSize;
              const layoutRect = self.layoutRect();
              hasScrollH = layoutRect.contentW > layoutRect.innerW;
              hasScrollV = layoutRect.contentH > layoutRect.innerH;
              containerSize = self.getEl('body')['client' + sizeName] - margin * 2;
              containerSize -= hasScrollH && hasScrollV ? self.getEl('scroll' + axisName)['client' + ax] : 0;
              ratio = containerSize / self.getEl('body')['scroll' + sizeName];
              self.getEl('body')['scroll' + posName] = scrollStart + e['delta' + deltaPosName] / ratio;
            },
            stop() {
              global$9('#' + axisId).removeClass(prefix + 'active');
            }
          });
        }
        self.classes.add('scroll');
        addScrollAxis('v', 'Top', 'Height', 'Y', 'Width');
        addScrollAxis('h', 'Left', 'Width', 'X', 'Height');
      }
      if (self.settings.autoScroll) {
        if (!self._hasScroll) {
          self._hasScroll = true;
          addScroll();
          self.on('wheel', e => {
            const bodyEl = self.getEl('body');
            bodyEl.scrollLeft += (e.deltaX || 0) * 10;
            bodyEl.scrollTop += e.deltaY * 10;
            repaintScroll();
          });
          global$9(self.getEl('body')).on('scroll', repaintScroll);
        }
        repaintScroll();
      }
    }
  };

  const Panel = Container.extend({
    Defaults: {
      layout: 'fit',
      containerCls: 'panel'
    },
    Mixins: [$_rcleftsjfjm4nec],
    renderHtml() {
      const self = this;
      const layout = self._layout;
      let innerHtml = self.settings.html;
      self.preRender();
      layout.preRender(self);
      if (typeof innerHtml === 'undefined') {
        innerHtml = '<div id="' + self._id + '-body" class="' + self.bodyClasses + '">' + layout.renderHtml(self) + '</div>';
      } else {
        if (typeof innerHtml === 'function') {
          innerHtml = innerHtml.call(self);
        }
        self._hasBody = false;
      }
      return '<div id="' + self._id + '" class="' + self.classes + '" hidefocus="1" tabindex="-1" role="group">' + (self._preBodyHtml || '') + innerHtml + '</div>';
    }
  });

  const $_c9f860tujfjm4nev = {
    resizeToContent() {
      this._layoutRect.autoResize = true;
      this._lastRect = null;
      this.reflow();
    },
    resizeTo(w, h) {
      if (w <= 1 || h <= 1) {
        const rect = funcs.getWindowSize();
        w = w <= 1 ? w * rect.w : w;
        h = h <= 1 ? h * rect.h : h;
      }
      this._layoutRect.autoResize = false;
      return this.layoutRect({
        minW: w,
        minH: h,
        w,
        h
      }).reflow();
    },
    resizeBy(dw, dh) {
      const self = this, rect = self.layoutRect();
      return self.resizeTo(rect.w + dw, rect.h + dh);
    }
  };

  let documentClickHandler;
  let documentScrollHandler;
  let windowResizeHandler;
  const visiblePanels = [];
  const zOrder = [];
  let hasModal;
  function isChildOf(ctrl, parent) {
    while (ctrl) {
      if (ctrl === parent) {
        return true;
      }
      ctrl = ctrl.parent();
    }
  }
  function skipOrHidePanels(e) {
    let i = visiblePanels.length;
    while (i--) {
      const panel = visiblePanels[i], clickCtrl = panel.getParentCtrl(e.target);
      if (panel.settings.autohide) {
        if (clickCtrl) {
          if (isChildOf(clickCtrl, panel) || panel.parent() === clickCtrl) {
            continue;
          }
        }
        e = panel.fire('autohide', { target: e.target });
        if (!e.isDefaultPrevented()) {
          panel.hide();
        }
      }
    }
  }
  function bindDocumentClickHandler() {
    if (!documentClickHandler) {
      documentClickHandler = e => {
        if (e.button === 2) {
          return;
        }
        skipOrHidePanels(e);
      };
      global$9(document).on('click touchstart', documentClickHandler);
    }
  }
  function bindDocumentScrollHandler() {
    if (!documentScrollHandler) {
      documentScrollHandler = () => {
        let i;
        i = visiblePanels.length;
        while (i--) {
          repositionPanel(visiblePanels[i]);
        }
      };
      global$9(window).on('scroll', documentScrollHandler);
    }
  }
  function bindWindowResizeHandler() {
    if (!windowResizeHandler) {
      const docElm_1 = document.documentElement;
      let clientWidth_1 = docElm_1.clientWidth, clientHeight_1 = docElm_1.clientHeight;
      windowResizeHandler = () => {
        if (!document.all || clientWidth_1 !== docElm_1.clientWidth || clientHeight_1 !== docElm_1.clientHeight) {
          clientWidth_1 = docElm_1.clientWidth;
          clientHeight_1 = docElm_1.clientHeight;
          FloatPanel.hideAll();
        }
      };
      global$9(window).on('resize', windowResizeHandler);
    }
  }
  function repositionPanel(panel) {
    const scrollY = funcs.getViewPort().y;
    function toggleFixedChildPanels(fixed, deltaY) {
      let parent;
      for (let i = 0; i < visiblePanels.length; i++) {
        if (visiblePanels[i] !== panel) {
          parent = visiblePanels[i].parent();
          while (parent && (parent = parent.parent())) {
            if (parent === panel) {
              visiblePanels[i].fixed(fixed).moveBy(0, deltaY).repaint();
            }
          }
        }
      }
    }
    if (panel.settings.autofix) {
      if (!panel.state.get('fixed')) {
        panel._autoFixY = panel.layoutRect().y;
        if (panel._autoFixY < scrollY) {
          panel.fixed(true).layoutRect({ y: 0 }).repaint();
          toggleFixedChildPanels(true, scrollY - panel._autoFixY);
        }
      } else {
        if (panel._autoFixY > scrollY) {
          panel.fixed(false).layoutRect({ y: panel._autoFixY }).repaint();
          toggleFixedChildPanels(false, panel._autoFixY - scrollY);
        }
      }
    }
  }
  function addRemove(add, ctrl) {
    let i, zIndex = FloatPanel.zIndex || 65535, topModal;
    if (add) {
      zOrder.push(ctrl);
    } else {
      i = zOrder.length;
      while (i--) {
        if (zOrder[i] === ctrl) {
          zOrder.splice(i, 1);
        }
      }
    }
    if (zOrder.length) {
      for (i = 0; i < zOrder.length; i++) {
        if (zOrder[i].modal) {
          zIndex++;
          topModal = zOrder[i];
        }
        zOrder[i].getEl().style.zIndex = zIndex;
        zOrder[i].zIndex = zIndex;
        zIndex++;
      }
    }
    const modalBlockEl = global$9('#' + ctrl.classPrefix + 'modal-block', ctrl.getContainerElm())[0];
    if (topModal) {
      global$9(modalBlockEl).css('z-index', topModal.zIndex - 1);
    } else if (modalBlockEl) {
      modalBlockEl.parentNode.removeChild(modalBlockEl);
      hasModal = false;
    }
    FloatPanel.currentZIndex = zIndex;
  }
  var FloatPanel = Panel.extend({
    Mixins: [
      $_1ttsxttdjfjm4ncj,
      $_c9f860tujfjm4nev
    ],
    init(settings) {
      const self = this;
      self._super(settings);
      self._eventsRoot = self;
      self.classes.add('floatpanel');
      if (settings.autohide) {
        bindDocumentClickHandler();
        bindWindowResizeHandler();
        visiblePanels.push(self);
      }
      if (settings.autofix) {
        bindDocumentScrollHandler();
        self.on('move', function () {
          repositionPanel(this);
        });
      }
      self.on('postrender show', e => {
        if (e.control === self) {
          let $modalBlockEl_1;
          const prefix_1 = self.classPrefix;
          if (self.modal && !hasModal) {
            $modalBlockEl_1 = global$9('#' + prefix_1 + 'modal-block', self.getContainerElm());
            if (!$modalBlockEl_1[0]) {
              $modalBlockEl_1 = global$9('<div id="' + prefix_1 + 'modal-block" class="' + prefix_1 + 'reset ' + prefix_1 + 'fade"></div>').appendTo(self.getContainerElm());
            }
            Delay.setTimeout(() => {
              $modalBlockEl_1.addClass(prefix_1 + 'in');
              global$9(self.getEl()).addClass(prefix_1 + 'in');
            });
            hasModal = true;
          }
          addRemove(true, self);
        }
      });
      self.on('show', () => {
        self.parents().each(ctrl => {
          if (ctrl.state.get('fixed')) {
            self.fixed(true);
            return false;
          }
        });
      });
      if (settings.popover) {
        self._preBodyHtml = '<div class="' + self.classPrefix + 'arrow"></div>';
        self.classes.add('popover').add('bottom').add(self.isRtl() ? 'end' : 'start');
      }
      self.aria('label', settings.ariaLabel);
      self.aria('labelledby', self._id);
      self.aria('describedby', self.describedBy || self._id + '-none');
    },
    fixed(state) {
      const self = this;
      if (self.state.get('fixed') !== state) {
        if (self.state.get('rendered')) {
          const viewport = funcs.getViewPort();
          if (state) {
            self.layoutRect().y -= viewport.y;
          } else {
            self.layoutRect().y += viewport.y;
          }
        }
        self.classes.toggle('fixed', state);
        self.state.set('fixed', state);
      }
      return self;
    },
    show() {
      const self = this;
      let i;
      const state = self._super();
      i = visiblePanels.length;
      while (i--) {
        if (visiblePanels[i] === self) {
          break;
        }
      }
      if (i === -1) {
        visiblePanels.push(self);
      }
      return state;
    },
    hide() {
      removeVisiblePanel(this);
      addRemove(false, this);
      return this._super();
    },
    hideAll() {
      FloatPanel.hideAll();
    },
    close() {
      const self = this;
      if (!self.fire('close').isDefaultPrevented()) {
        self.remove();
        addRemove(false, self);
      }
      return self;
    },
    remove() {
      removeVisiblePanel(this);
      this._super();
    },
    postRender() {
      const self = this;
      if (self.settings.bodyRole) {
        this.getEl('body').setAttribute('role', self.settings.bodyRole);
      }
      return self._super();
    }
  });
  FloatPanel.hideAll = () => {
    let i = visiblePanels.length;
    while (i--) {
      const panel = visiblePanels[i];
      if (panel && panel.settings.autohide) {
        panel.hide();
        visiblePanels.splice(i, 1);
      }
    }
  };
  function removeVisiblePanel(panel) {
    let i;
    i = visiblePanels.length;
    while (i--) {
      if (visiblePanels[i] === panel) {
        visiblePanels.splice(i, 1);
      }
    }
    i = zOrder.length;
    while (i--) {
      if (zOrder[i] === panel) {
        zOrder.splice(i, 1);
      }
    }
  }

  const isFixed$1 = (inlineToolbarContainer, editor) => !!(inlineToolbarContainer && !editor.settings.ui_container);
  const render$1 = (editor, theme, args) => {
    let panel, inlineToolbarContainer;
    const DOM = DOMUtils.DOM;
    const fixedToolbarContainer = getFixedToolbarContainer(editor);
    if (fixedToolbarContainer) {
      inlineToolbarContainer = DOM.select(fixedToolbarContainer)[0];
    }
    const reposition = () => {
      if (panel && panel.moveRel && panel.visible() && !panel._fixed) {
        const scrollContainer = editor.selection.getScrollContainer(), body = editor.getBody();
        let deltaX = 0, deltaY = 0;
        if (scrollContainer) {
          const bodyPos = DOM.getPos(body), scrollContainerPos = DOM.getPos(scrollContainer);
          deltaX = Math.max(0, scrollContainerPos.x - bodyPos.x);
          deltaY = Math.max(0, scrollContainerPos.y - bodyPos.y);
        }
        panel.fixed(false).moveRel(body, editor.rtl ? [
          'tr-br',
          'br-tr'
        ] : [
          'tl-bl',
          'bl-tl',
          'tr-br'
        ]).moveBy(deltaX, deltaY);
      }
    };
    const show = () => {
      if (panel) {
        panel.show();
        reposition();
        DOM.addClass(editor.getBody(), 'mce-edit-focus');
      }
    };
    const hide = () => {
      if (panel) {
        panel.hide();
        FloatPanel.hideAll();
        DOM.removeClass(editor.getBody(), 'mce-edit-focus');
      }
    };
    const render = () => {
      if (panel) {
        if (!panel.visible()) {
          show();
        }
        return;
      }
      panel = theme.panel = Factory.create({
        type: inlineToolbarContainer ? 'panel' : 'floatpanel',
        role: 'application',
        classes: 'tinymce tinymce-inline',
        layout: 'flex',
        direction: 'column',
        align: 'stretch',
        autohide: false,
        autofix: isFixed$1(inlineToolbarContainer, editor),
        fixed: isFixed$1(inlineToolbarContainer, editor),
        border: 1,
        items: [
          hasMenubar(editor) === false ? null : {
            type: 'menubar',
            border: '0 0 1 0',
            items: $_zri7tt3jfjm4nbl.createMenuButtons(editor)
          },
          Toolbar.createToolbars(editor, getToolbarSize(editor))
        ]
      });
      $_esvtriszjfjm4nba.setUiContainer(editor, panel);
      SkinLoadedEvents.fireBeforeRenderUI(editor);
      if (inlineToolbarContainer) {
        panel.renderTo(inlineToolbarContainer).reflow();
      } else {
        panel.renderTo().reflow();
      }
      Shortcuts.addKeys(editor, panel);
      show();
      ContextToolbars.addContextualToolbars(editor);
      editor.on('nodeChange', reposition);
      editor.on('ResizeWindow', reposition);
      editor.on('activate', show);
      editor.on('deactivate', hide);
      editor.nodeChanged();
    };
    editor.settings.content_editable = true;
    editor.on('focus', () => {
      if (isSkinDisabled(editor) === false && args.skinUiCss) {
        DOM.styleSheetLoader.load(args.skinUiCss, render, render);
      } else {
        render();
      }
    });
    editor.on('blur hide', hide);
    editor.on('remove', () => {
      if (panel) {
        panel.remove();
        panel = null;
      }
    });
    if (isSkinDisabled(editor) === false && args.skinUiCss) {
      DOM.styleSheetLoader.load(args.skinUiCss, $_9j2wbtt8jfjm4nc1.fireSkinLoaded(editor));
    } else {
      $_9j2wbtt8jfjm4nc1.fireSkinLoaded(editor)();
    }
    return {};
  };
  const $_ev44nkt9jfjm4nc2 = { render: render$1 };

  function Throbber (elm, inline) {
    const self = this;
    let state;
    const classPrefix = Control$1.classPrefix;
    let timer;
    self.show = (time, callback) => {
      function render() {
        if (state) {
          global$9(elm).append('<div class="' + classPrefix + 'throbber' + (inline ? ' ' + classPrefix + 'throbber-inline' : '') + '"></div>');
          if (callback) {
            callback();
          }
        }
      }
      self.hide();
      state = true;
      if (time) {
        timer = Delay.setTimeout(render, time);
      } else {
        render();
      }
      return self;
    };
    self.hide = () => {
      const child = elm.lastChild;
      Delay.clearTimeout(timer);
      if (child && child.className.indexOf('throbber') !== -1) {
        child.parentNode.removeChild(child);
      }
      state = false;
      return self;
    };
  }

  const setup = (editor, theme) => {
    let throbber;
    editor.on('ProgressState', e => {
      throbber = throbber || new Throbber(theme.panel.getEl('body'));
      if (e.state) {
        throbber.show(e.time);
      } else {
        throbber.hide();
      }
    });
  };
  const $_4jt9ektvjfjm4nex = { setup };

  const renderUI = (editor, theme, args) => {
    const skinUrl = getSkinUrl(editor);
    if (skinUrl) {
      args.skinUiCss = skinUrl + '/skin.min.css';
      editor.contentCSS.push(skinUrl + '/content' + (editor.inline ? '.inline' : '') + '.min.css');
    }
    $_4jt9ektvjfjm4nex.setup(editor, theme);
    return isInline(editor) ? $_ev44nkt9jfjm4nc2.render(editor, theme, args) : $_5f2n0osqjfjm4nao.render(editor, theme, args);
  };
  const $_d9fsposmjfjm4nai = { renderUI };

  const Tooltip = Control$1.extend({
    Mixins: [$_1ttsxttdjfjm4ncj],
    Defaults: { classes: 'widget tooltip tooltip-n' },
    renderHtml() {
      const self = this, prefix = self.classPrefix;
      return '<div id="' + self._id + '" class="' + self.classes + '" role="presentation">' + '<div class="' + prefix + 'tooltip-arrow"></div>' + '<div class="' + prefix + 'tooltip-inner">' + self.encode(self.state.get('text')) + '</div>' + '</div>';
    },
    bindStates() {
      const self = this;
      self.state.on('change:text', e => {
        self.getEl().lastChild.innerHTML = self.encode(e.value);
      });
      return self._super();
    },
    repaint() {
      const self = this;
      let style, rect;
      style = self.getEl().style;
      rect = self._layoutRect;
      style.left = rect.x + 'px';
      style.top = rect.y + 'px';
      style.zIndex = 65535 + 65535;
    }
  });

  const Widget = Control$1.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      settings = self.settings;
      self.canFocus = true;
      if (settings.tooltip && Widget.tooltips !== false) {
        self.on('mouseenter', e => {
          const tooltip = self.tooltip().moveTo(-65535);
          if (e.control === self) {
            const rel = tooltip.text(settings.tooltip).show().testMoveRel(self.getEl(), [
              'bc-tc',
              'bc-tl',
              'bc-tr'
            ]);
            tooltip.classes.toggle('tooltip-n', rel === 'bc-tc');
            tooltip.classes.toggle('tooltip-nw', rel === 'bc-tl');
            tooltip.classes.toggle('tooltip-ne', rel === 'bc-tr');
            tooltip.moveRel(self.getEl(), rel);
          } else {
            tooltip.hide();
          }
        });
        self.on('mouseleave mousedown click', () => {
          self.tooltip().remove();
          self._tooltip = null;
        });
      }
      self.aria('label', settings.ariaLabel || settings.tooltip);
    },
    tooltip() {
      if (!this._tooltip) {
        this._tooltip = new Tooltip({ type: 'tooltip' });
        $_esvtriszjfjm4nba.inheritUiContainer(this, this._tooltip);
        this._tooltip.renderTo();
      }
      return this._tooltip;
    },
    postRender() {
      const self = this, settings = self.settings;
      self._super();
      if (!self.parent() && (settings.width || settings.height)) {
        self.initLayoutRect();
        self.repaint();
      }
      if (settings.autofocus) {
        self.focus();
      }
    },
    bindStates() {
      const self = this;
      function disable(state) {
        self.aria('disabled', state);
        self.classes.toggle('disabled', state);
      }
      function active(state) {
        self.aria('pressed', state);
        self.classes.toggle('active', state);
      }
      self.state.on('change:disabled', e => {
        disable(e.value);
      });
      self.state.on('change:active', e => {
        active(e.value);
      });
      if (self.state.get('disabled')) {
        disable(true);
      }
      if (self.state.get('active')) {
        active(true);
      }
      return self._super();
    },
    remove() {
      this._super();
      if (this._tooltip) {
        this._tooltip.remove();
        this._tooltip = null;
      }
    }
  });

  const Progress = Widget.extend({
    Defaults: { value: 0 },
    init(settings) {
      const self = this;
      self._super(settings);
      self.classes.add('progress');
      if (!self.settings.filter) {
        self.settings.filter = value => Math.round(value);
      }
    },
    renderHtml() {
      const self = this, id = self._id, prefix = this.classPrefix;
      return '<div id="' + id + '" class="' + self.classes + '">' + '<div class="' + prefix + 'bar-container">' + '<div class="' + prefix + 'bar"></div>' + '</div>' + '<div class="' + prefix + 'text">0%</div>' + '</div>';
    },
    postRender() {
      const self = this;
      self._super();
      self.value(self.settings.value);
      return self;
    },
    bindStates() {
      const self = this;
      function setValue(value) {
        value = self.settings.filter(value);
        self.getEl().lastChild.innerHTML = value + '%';
        self.getEl().firstChild.firstChild.style.width = value + '%';
      }
      self.state.on('change:value', e => {
        setValue(e.value);
      });
      setValue(self.state.get('value'));
      return self._super();
    }
  });

  const updateLiveRegion = (ctx, text) => {
    ctx.getEl().lastChild.textContent = text + (ctx.progressBar ? ' ' + ctx.progressBar.value() + '%' : '');
  };
  const Notification = Control$1.extend({
    Mixins: [$_1ttsxttdjfjm4ncj],
    Defaults: { classes: 'widget notification' },
    init(settings) {
      const self = this;
      self._super(settings);
      self.maxWidth = settings.maxWidth;
      if (settings.text) {
        self.text(settings.text);
      }
      if (settings.icon) {
        self.icon = settings.icon;
      }
      if (settings.color) {
        self.color = settings.color;
      }
      if (settings.type) {
        self.classes.add('notification-' + settings.type);
      }
      if (settings.timeout && (settings.timeout < 0 || settings.timeout > 0) && !settings.closeButton) {
        self.closeButton = false;
      } else {
        self.classes.add('has-close');
        self.closeButton = true;
      }
      if (settings.progressBar) {
        self.progressBar = new Progress();
      }
      self.on('click', e => {
        if (e.target.className.indexOf(self.classPrefix + 'close') !== -1) {
          self.close();
        }
      });
    },
    renderHtml() {
      const self = this;
      const prefix = self.classPrefix;
      let icon = '', closeButton = '', progressBar = '', notificationStyle = '';
      if (self.icon) {
        icon = '<i class="' + prefix + 'ico' + ' ' + prefix + 'i-' + self.icon + '"></i>';
      }
      notificationStyle = ' style="max-width: ' + self.maxWidth + 'px;' + (self.color ? 'background-color: ' + self.color + ';"' : '"');
      if (self.closeButton) {
        closeButton = '<button type="button" class="' + prefix + 'close" aria-hidden="true">\xD7</button>';
      }
      if (self.progressBar) {
        progressBar = self.progressBar.renderHtml();
      }
      return '<div id="' + self._id + '" class="' + self.classes + '"' + notificationStyle + ' role="presentation">' + icon + '<div class="' + prefix + 'notification-inner">' + self.state.get('text') + '</div>' + progressBar + closeButton + '<div style="clip: rect(1px, 1px, 1px, 1px);height: 1px;overflow: hidden;position: absolute;width: 1px;"' + ' aria-live="assertive" aria-relevant="additions" aria-atomic="true"></div>' + '</div>';
    },
    postRender() {
      const self = this;
      Delay.setTimeout(() => {
        self.$el.addClass(self.classPrefix + 'in');
        updateLiveRegion(self, self.state.get('text'));
      }, 100);
      return self._super();
    },
    bindStates() {
      const self = this;
      self.state.on('change:text', e => {
        self.getEl().firstChild.innerHTML = e.value;
        updateLiveRegion(self, e.value);
      });
      if (self.progressBar) {
        self.progressBar.bindStates();
        self.progressBar.state.on('change:value', e => {
          updateLiveRegion(self, self.state.get('text'));
        });
      }
      return self._super();
    },
    close() {
      const self = this;
      if (!self.fire('close').isDefaultPrevented()) {
        self.remove();
      }
      return self;
    },
    repaint() {
      const self = this;
      let style, rect;
      style = self.getEl().style;
      rect = self._layoutRect;
      style.left = rect.x + 'px';
      style.top = rect.y + 'px';
      style.zIndex = 65535 - 1;
    }
  });

  function NotificationManagerImpl (editor) {
    const getEditorContainer = editor => editor.inline ? editor.getElement() : editor.getContentAreaContainer();
    const getContainerWidth = () => {
      const container = getEditorContainer(editor);
      return funcs.getSize(container).width;
    };
    const prePositionNotifications = notifications => {
      Arr.each(notifications, notification => {
        notification.moveTo(0, 0);
      });
    };
    const positionNotifications = notifications => {
      if (notifications.length > 0) {
        const firstItem = notifications.slice(0, 1)[0];
        const container = getEditorContainer(editor);
        firstItem.moveRel(container, 'tc-tc');
        Arr.each(notifications, (notification, index) => {
          if (index > 0) {
            notification.moveRel(notifications[index - 1].getEl(), 'bc-tc');
          }
        });
      }
    };
    const reposition = notifications => {
      prePositionNotifications(notifications);
      positionNotifications(notifications);
    };
    const open = (args, closeCallback) => {
      const extendedArgs = Tools.extend(args, { maxWidth: getContainerWidth() });
      const notif = new Notification(extendedArgs);
      notif.args = extendedArgs;
      if (extendedArgs.timeout > 0) {
        notif.timer = setTimeout(() => {
          notif.close();
          closeCallback();
        }, extendedArgs.timeout);
      }
      notif.on('close', () => {
        closeCallback();
      });
      notif.renderTo();
      return notif;
    };
    const close = notification => {
      notification.close();
    };
    const getArgs = notification => notification.args;
    return {
      open,
      close,
      reposition,
      getArgs
    };
  }

  const windows = [];
  let oldMetaValue = '';
  function toggleFullScreenState(state) {
    const noScaleMetaValue = 'width=device-width,initial-scale=1.0,user-scalable=0,minimum-scale=1.0,maximum-scale=1.0';
    let viewport = global$9('meta[name=viewport]')[0], contentValue;
    if (global$8.overrideViewPort === false) {
      return;
    }
    if (!viewport) {
      viewport = document.createElement('meta');
      viewport.setAttribute('name', 'viewport');
      document.getElementsByTagName('head')[0].appendChild(viewport);
    }
    contentValue = viewport.getAttribute('content');
    if (contentValue && typeof oldMetaValue !== 'undefined') {
      oldMetaValue = contentValue;
    }
    viewport.setAttribute('content', state ? noScaleMetaValue : oldMetaValue);
  }
  function toggleBodyFullScreenClasses(classPrefix, state) {
    if (checkFullscreenWindows() && state === false) {
      global$9([
        document.documentElement,
        document.body
      ]).removeClass(classPrefix + 'fullscreen');
    }
  }
  function checkFullscreenWindows() {
    for (let i = 0; i < windows.length; i++) {
      if (windows[i]._fullscreen) {
        return true;
      }
    }
    return false;
  }
  function handleWindowResize() {
    if (!global$8.desktop) {
      let lastSize_1 = {
        w: window.innerWidth,
        h: window.innerHeight
      };
      Delay.setInterval(() => {
        const w = window.innerWidth, h = window.innerHeight;
        if (lastSize_1.w !== w || lastSize_1.h !== h) {
          lastSize_1 = {
            w,
            h
          };
          global$9(window).trigger('resize');
        }
      }, 100);
    }
    function reposition() {
      let i;
      const rect = funcs.getWindowSize();
      let layoutRect;
      for (i = 0; i < windows.length; i++) {
        layoutRect = windows[i].layoutRect();
        windows[i].moveTo(windows[i].settings.x || Math.max(0, rect.w / 2 - layoutRect.w / 2), windows[i].settings.y || Math.max(0, rect.h / 2 - layoutRect.h / 2));
      }
    }
    global$9(window).on('resize', reposition);
  }
  const Window = FloatPanel.extend({
    modal: true,
    Defaults: {
      border: 1,
      layout: 'flex',
      containerCls: 'panel',
      role: 'dialog',
      callbacks: {
        submit() {
          this.fire('submit', { data: this.toJSON() });
        },
        close() {
          this.close();
        }
      }
    },
    init(settings) {
      const self = this;
      self._super(settings);
      if (self.isRtl()) {
        self.classes.add('rtl');
      }
      self.classes.add('window');
      self.bodyClasses.add('window-body');
      self.state.set('fixed', true);
      if (settings.buttons) {
        self.statusbar = new Panel({
          layout: 'flex',
          border: '1 0 0 0',
          spacing: 3,
          padding: 10,
          align: 'center',
          pack: self.isRtl() ? 'start' : 'end',
          defaults: { type: 'button' },
          items: settings.buttons
        });
        self.statusbar.classes.add('foot');
        self.statusbar.parent(self);
      }
      self.on('click', e => {
        const closeClass = self.classPrefix + 'close';
        if (funcs.hasClass(e.target, closeClass) || funcs.hasClass(e.target.parentNode, closeClass)) {
          self.close();
        }
      });
      self.on('cancel', () => {
        self.close();
      });
      self.aria('describedby', self.describedBy || self._id + '-none');
      self.aria('label', settings.title);
      self._fullscreen = false;
    },
    recalc() {
      const self = this;
      const statusbar = self.statusbar;
      let layoutRect, width, x, needsRecalc;
      if (self._fullscreen) {
        self.layoutRect(funcs.getWindowSize());
        self.layoutRect().contentH = self.layoutRect().innerH;
      }
      self._super();
      layoutRect = self.layoutRect();
      if (self.settings.title && !self._fullscreen) {
        width = layoutRect.headerW;
        if (width > layoutRect.w) {
          x = layoutRect.x - Math.max(0, width / 2);
          self.layoutRect({
            w: width,
            x
          });
          needsRecalc = true;
        }
      }
      if (statusbar) {
        statusbar.layoutRect({ w: self.layoutRect().innerW }).recalc();
        width = statusbar.layoutRect().minW + layoutRect.deltaW;
        if (width > layoutRect.w) {
          x = layoutRect.x - Math.max(0, width - layoutRect.w);
          self.layoutRect({
            w: width,
            x
          });
          needsRecalc = true;
        }
      }
      if (needsRecalc) {
        self.recalc();
      }
    },
    initLayoutRect() {
      const self = this;
      const layoutRect = self._super();
      let deltaH = 0, headEl;
      if (self.settings.title && !self._fullscreen) {
        headEl = self.getEl('head');
        const size = funcs.getSize(headEl);
        layoutRect.headerW = size.width;
        layoutRect.headerH = size.height;
        deltaH += layoutRect.headerH;
      }
      if (self.statusbar) {
        deltaH += self.statusbar.layoutRect().h;
      }
      layoutRect.deltaH += deltaH;
      layoutRect.minH += deltaH;
      layoutRect.h += deltaH;
      const rect = funcs.getWindowSize();
      layoutRect.x = self.settings.x || Math.max(0, rect.w / 2 - layoutRect.w / 2);
      layoutRect.y = self.settings.y || Math.max(0, rect.h / 2 - layoutRect.h / 2);
      return layoutRect;
    },
    renderHtml() {
      const self = this, layout = self._layout, id = self._id, prefix = self.classPrefix;
      const settings = self.settings;
      let headerHtml = '', footerHtml = '', html = settings.html;
      self.preRender();
      layout.preRender(self);
      if (settings.title) {
        headerHtml = '<div id="' + id + '-head" class="' + prefix + 'window-head">' + '<div id="' + id + '-title" class="' + prefix + 'title">' + self.encode(settings.title) + '</div>' + '<div id="' + id + '-dragh" class="' + prefix + 'dragh"></div>' + '<button type="button" class="' + prefix + 'close" aria-hidden="true">' + '<i class="mce-ico mce-i-remove"></i>' + '</button>' + '</div>';
      }
      if (settings.url) {
        html = '<iframe src="' + settings.url + '" tabindex="-1"></iframe>';
      }
      if (typeof html === 'undefined') {
        html = layout.renderHtml(self);
      }
      if (self.statusbar) {
        footerHtml = self.statusbar.renderHtml();
      }
      return '<div id="' + id + '" class="' + self.classes + '" hidefocus="1">' + '<div class="' + self.classPrefix + 'reset" role="application">' + headerHtml + '<div id="' + id + '-body" class="' + self.bodyClasses + '">' + html + '</div>' + footerHtml + '</div>' + '</div>';
    },
    fullscreen(state) {
      const self = this;
      const documentElement = document.documentElement;
      let slowRendering;
      const prefix = self.classPrefix;
      let layoutRect;
      if (state !== self._fullscreen) {
        global$9(window).on('resize', () => {
          let time;
          if (self._fullscreen) {
            if (!slowRendering) {
              time = new Date().getTime();
              const rect = funcs.getWindowSize();
              self.moveTo(0, 0).resizeTo(rect.w, rect.h);
              if (new Date().getTime() - time > 50) {
                slowRendering = true;
              }
            } else {
              if (!self._timer) {
                self._timer = Delay.setTimeout(() => {
                  const rect = funcs.getWindowSize();
                  self.moveTo(0, 0).resizeTo(rect.w, rect.h);
                  self._timer = 0;
                }, 50);
              }
            }
          }
        });
        layoutRect = self.layoutRect();
        self._fullscreen = state;
        if (!state) {
          self.borderBox = $_bty1r7tjjfjm4ndm.parseBox(self.settings.border);
          self.getEl('head').style.display = '';
          layoutRect.deltaH += layoutRect.headerH;
          global$9([
            documentElement,
            document.body
          ]).removeClass(prefix + 'fullscreen');
          self.classes.remove('fullscreen');
          self.moveTo(self._initial.x, self._initial.y).resizeTo(self._initial.w, self._initial.h);
        } else {
          self._initial = {
            x: layoutRect.x,
            y: layoutRect.y,
            w: layoutRect.w,
            h: layoutRect.h
          };
          self.borderBox = $_bty1r7tjjfjm4ndm.parseBox('0');
          self.getEl('head').style.display = 'none';
          layoutRect.deltaH -= layoutRect.headerH + 2;
          global$9([
            documentElement,
            document.body
          ]).addClass(prefix + 'fullscreen');
          self.classes.add('fullscreen');
          const rect = funcs.getWindowSize();
          self.moveTo(0, 0).resizeTo(rect.w, rect.h);
        }
      }
      return self.reflow();
    },
    postRender() {
      const self = this;
      let startPos;
      setTimeout(() => {
        self.classes.add('in');
        self.fire('open');
      }, 0);
      self._super();
      if (self.statusbar) {
        self.statusbar.postRender();
      }
      self.focus();
      this.dragHelper = new DragHelper(self._id + '-dragh', {
        start() {
          startPos = {
            x: self.layoutRect().x,
            y: self.layoutRect().y
          };
        },
        drag(e) {
          self.moveTo(startPos.x + e.deltaX, startPos.y + e.deltaY);
        }
      });
      self.on('submit', e => {
        if (!e.isDefaultPrevented()) {
          self.close();
        }
      });
      windows.push(self);
      toggleFullScreenState(true);
    },
    submit() {
      return this.fire('submit', { data: this.toJSON() });
    },
    remove() {
      const self = this;
      let i;
      self.dragHelper.destroy();
      self._super();
      if (self.statusbar) {
        this.statusbar.remove();
      }
      toggleBodyFullScreenClasses(self.classPrefix, false);
      i = windows.length;
      while (i--) {
        if (windows[i] === self) {
          windows.splice(i, 1);
        }
      }
      toggleFullScreenState(windows.length > 0);
    },
    getContentWindow() {
      const ifr = this.getEl().getElementsByTagName('iframe')[0];
      return ifr ? ifr.contentWindow : null;
    }
  });
  handleWindowResize();

  const MessageBox = Window.extend({
    init(settings) {
      settings = {
        border: 1,
        padding: 20,
        layout: 'flex',
        pack: 'center',
        align: 'center',
        containerCls: 'panel',
        autoScroll: true,
        buttons: {
          type: 'button',
          text: 'Ok',
          action: 'ok'
        },
        items: {
          type: 'label',
          multiline: true,
          maxWidth: 500,
          maxHeight: 200
        }
      };
      this._super(settings);
    },
    Statics: {
      OK: 1,
      OK_CANCEL: 2,
      YES_NO: 3,
      YES_NO_CANCEL: 4,
      msgBox(settings) {
        let buttons;
        const callback = settings.callback || (() => {
        });
        function createButton(text, status, primary) {
          return {
            type: 'button',
            text,
            subtype: primary ? 'primary' : '',
            onClick(e) {
              e.control.parents()[1].close();
              callback(status);
            }
          };
        }
        switch (settings.buttons) {
        case MessageBox.OK_CANCEL:
          buttons = [
            createButton('Ok', true, true),
            createButton('Cancel', false)
          ];
          break;
        case MessageBox.YES_NO:
        case MessageBox.YES_NO_CANCEL:
          buttons = [
            createButton('Yes', 1, true),
            createButton('No', 0)
          ];
          if (settings.buttons === MessageBox.YES_NO_CANCEL) {
            buttons.push(createButton('Cancel', -1));
          }
          break;
        default:
          buttons = [createButton('Ok', true, true)];
          break;
        }
        return new Window({
          padding: 20,
          x: settings.x,
          y: settings.y,
          minWidth: 300,
          minHeight: 100,
          layout: 'flex',
          pack: 'center',
          align: 'center',
          buttons,
          title: settings.title,
          role: 'alertdialog',
          items: {
            type: 'label',
            multiline: true,
            maxWidth: 500,
            maxHeight: 200,
            text: settings.text
          },
          onPostRender() {
            this.aria('describedby', this.items()[0]._id);
          },
          onClose: settings.onClose,
          onCancel() {
            callback(false);
          }
        }).renderTo(document.body).reflow();
      },
      alert(settings, callback) {
        if (typeof settings === 'string') {
          settings = { text: settings };
        }
        settings.callback = callback;
        return MessageBox.msgBox(settings);
      },
      confirm(settings, callback) {
        if (typeof settings === 'string') {
          settings = { text: settings };
        }
        settings.callback = callback;
        settings.buttons = MessageBox.OK_CANCEL;
        return MessageBox.msgBox(settings);
      }
    }
  });

  function WindowManagerImpl (editor) {
    const open = (args, params, closeCallback) => {
      let win;
      args.title = args.title || ' ';
      args.url = args.url || args.file;
      if (args.url) {
        args.width = parseInt(args.width || 320, 10);
        args.height = parseInt(args.height || 240, 10);
      }
      if (args.body) {
        args.items = {
          defaults: args.defaults,
          type: args.bodyType || 'form',
          items: args.body,
          data: args.data,
          callbacks: args.commands
        };
      }
      if (!args.url && !args.buttons) {
        args.buttons = [
          {
            text: 'Ok',
            subtype: 'primary',
            onclick() {
              win.find('form')[0].submit();
            }
          },
          {
            text: 'Cancel',
            onclick() {
              win.close();
            }
          }
        ];
      }
      win = new Window(args);
      win.on('close', () => {
        closeCallback(win);
      });
      if (args.data) {
        win.on('postRender', function () {
          this.find('*').each(ctrl => {
            const name = ctrl.name();
            if (name in args.data) {
              ctrl.value(args.data[name]);
            }
          });
        });
      }
      win.features = args || {};
      win.params = params || {};
      win = win.renderTo(document.body).reflow();
      return win;
    };
    const alert = (message, choiceCallback, closeCallback) => {
      let win;
      win = MessageBox.alert(message, () => {
        choiceCallback();
      });
      win.on('close', () => {
        closeCallback(win);
      });
      return win;
    };
    const confirm = (message, choiceCallback, closeCallback) => {
      let win;
      win = MessageBox.confirm(message, state => {
        choiceCallback(state);
      });
      win.on('close', () => {
        closeCallback(win);
      });
      return win;
    };
    const close = window => {
      window.close();
    };
    const getParams = window => window.params;
    const setParams = (window, params) => {
      window.params = params;
    };
    return {
      open,
      alert,
      confirm,
      close,
      getParams,
      setParams
    };
  }

  const get = editor => {
    const renderUI = function (args) {
      return $_d9fsposmjfjm4nai.renderUI(editor, this, args);
    };
    const resizeTo = (w, h) => $_5ku2twt5jfjm4nbw.resizeTo(editor, w, h);
    const resizeBy = (dw, dh) => $_5ku2twt5jfjm4nbw.resizeBy(editor, dw, dh);
    const getNotificationManagerImpl = () => NotificationManagerImpl(editor);
    const getWindowManagerImpl = () => WindowManagerImpl(editor);
    return {
      renderUI,
      resizeTo,
      resizeBy,
      getNotificationManagerImpl,
      getWindowManagerImpl
    };
  };
  const $_dtr9igsljfjm4nah = { get };

  const Layout = global$10.extend({
    Defaults: {
      firstControlClass: 'first',
      lastControlClass: 'last'
    },
    init(settings) {
      this.settings = Tools.extend({}, this.Defaults, settings);
    },
    preRender(container) {
      container.bodyClasses.add(this.settings.containerClass);
    },
    applyClasses(items) {
      const self = this;
      const settings = self.settings;
      let firstClass, lastClass, firstItem, lastItem;
      firstClass = settings.firstControlClass;
      lastClass = settings.lastControlClass;
      items.each(item => {
        item.classes.remove(firstClass).remove(lastClass).add(settings.controlClass);
        if (item.visible()) {
          if (!firstItem) {
            firstItem = item;
          }
          lastItem = item;
        }
      });
      if (firstItem) {
        firstItem.classes.add(firstClass);
      }
      if (lastItem) {
        lastItem.classes.add(lastClass);
      }
    },
    renderHtml(container) {
      const self = this;
      let html = '';
      self.applyClasses(container.items());
      container.items().each(item => {
        html += item.renderHtml();
      });
      return html;
    },
    recalc() {
    },
    postRender() {
    },
    isNative() {
      return false;
    }
  });

  const AbsoluteLayout = Layout.extend({
    Defaults: {
      containerClass: 'abs-layout',
      controlClass: 'abs-layout-item'
    },
    recalc(container) {
      container.items().filter(':visible').each(ctrl => {
        const settings = ctrl.settings;
        ctrl.layoutRect({
          x: settings.x,
          y: settings.y,
          w: settings.w,
          h: settings.h
        });
        if (ctrl.recalc) {
          ctrl.recalc();
        }
      });
    },
    renderHtml(container) {
      return '<div id="' + container._id + '-absend" class="' + container.classPrefix + 'abs-end"></div>' + this._super(container);
    }
  });

  const Button = Widget.extend({
    Defaults: {
      classes: 'widget btn',
      role: 'button'
    },
    init(settings) {
      const self = this;
      let size;
      self._super(settings);
      settings = self.settings;
      size = self.settings.size;
      self.on('click mousedown', e => {
        e.preventDefault();
      });
      self.on('touchstart', e => {
        self.fire('click', e);
        e.preventDefault();
      });
      if (settings.subtype) {
        self.classes.add(settings.subtype);
      }
      if (size) {
        self.classes.add('btn-' + size);
      }
      if (settings.icon) {
        self.icon(settings.icon);
      }
    },
    icon(icon) {
      if (!arguments.length) {
        return this.state.get('icon');
      }
      this.state.set('icon', icon);
      return this;
    },
    repaint() {
      const btnElm = this.getEl().firstChild;
      let btnStyle;
      if (btnElm) {
        btnStyle = btnElm.style;
        btnStyle.width = btnStyle.height = '100%';
      }
      this._super();
    },
    renderHtml() {
      const self = this, id = self._id, prefix = self.classPrefix;
      let icon = self.state.get('icon'), image;
      const text = self.state.get('text');
      let textHtml = '';
      let ariaPressed;
      const settings = self.settings;
      image = settings.image;
      if (image) {
        icon = 'none';
        if (typeof image !== 'string') {
          image = window.getSelection ? image[0] : image[1];
        }
        image = ' style="background-image: url(\'' + image + '\')"';
      } else {
        image = '';
      }
      if (text) {
        self.classes.add('btn-has-text');
        textHtml = '<span class="' + prefix + 'txt">' + self.encode(text) + '</span>';
      }
      icon = icon ? prefix + 'ico ' + prefix + 'i-' + icon : '';
      ariaPressed = typeof settings.active === 'boolean' ? ' aria-pressed="' + settings.active + '"' : '';
      return '<div id="' + id + '" class="' + self.classes + '" tabindex="-1"' + ariaPressed + '>' + '<button id="' + id + '-button" role="presentation" type="button" tabindex="-1">' + (icon ? '<i class="' + icon + '"' + image + '></i>' : '') + textHtml + '</button>' + '</div>';
    },
    bindStates() {
      const self = this, $ = self.$, textCls = self.classPrefix + 'txt';
      function setButtonText(text) {
        let $span = $('span.' + textCls, self.getEl());
        if (text) {
          if (!$span[0]) {
            $('button:first', self.getEl()).append('<span class="' + textCls + '"></span>');
            $span = $('span.' + textCls, self.getEl());
          }
          $span.html(self.encode(text));
        } else {
          $span.remove();
        }
        self.classes.toggle('btn-has-text', !!text);
      }
      self.state.on('change:text', e => {
        setButtonText(e.value);
      });
      self.state.on('change:icon', e => {
        let icon = e.value;
        const prefix = self.classPrefix;
        self.settings.icon = icon;
        icon = icon ? prefix + 'ico ' + prefix + 'i-' + self.settings.icon : '';
        const btnElm = self.getEl().firstChild;
        let iconElm = btnElm.getElementsByTagName('i')[0];
        if (icon) {
          if (!iconElm || iconElm !== btnElm.firstChild) {
            iconElm = document.createElement('i');
            btnElm.insertBefore(iconElm, btnElm.firstChild);
          }
          iconElm.className = icon;
        } else if (iconElm) {
          btnElm.removeChild(iconElm);
        }
        setButtonText(self.state.get('text'));
      });
      return self._super();
    }
  });

  const BrowseButton = Button.extend({
    init(settings) {
      const self = this;
      settings = Tools.extend({
        text: 'Browse...',
        multiple: false,
        accept: null
      }, settings);
      self._super(settings);
      self.classes.add('browsebutton');
      if (settings.multiple) {
        self.classes.add('multiple');
      }
    },
    postRender() {
      const self = this;
      const input = funcs.create('input', {
        type: 'file',
        id: self._id + '-browse',
        accept: self.settings.accept
      });
      self._super();
      global$9(input).on('change', e => {
        const files = e.target.files;
        self.value = () => {
          if (!files.length) {
            return null;
          } else if (self.settings.multiple) {
            return files;
          } else {
            return files[0];
          }
        };
        e.preventDefault();
        if (files.length) {
          self.fire('change', e);
        }
      });
      global$9(input).on('click', e => {
        e.stopPropagation();
      });
      global$9(self.getEl('button')).on('click', e => {
        e.stopPropagation();
        input.click();
      });
      self.getEl().appendChild(input);
    },
    remove() {
      global$9(this.getEl('button')).off();
      global$9(this.getEl('input')).off();
      this._super();
    }
  });

  const ButtonGroup = Container.extend({
    Defaults: {
      defaultType: 'button',
      role: 'group'
    },
    renderHtml() {
      const self = this, layout = self._layout;
      self.classes.add('btn-group');
      self.preRender();
      layout.preRender(self);
      return '<div id="' + self._id + '" class="' + self.classes + '">' + '<div id="' + self._id + '-body">' + (self.settings.html || '') + layout.renderHtml(self) + '</div>' + '</div>';
    }
  });

  const Checkbox = Widget.extend({
    Defaults: {
      classes: 'checkbox',
      role: 'checkbox',
      checked: false
    },
    init(settings) {
      const self = this;
      self._super(settings);
      self.on('click mousedown', e => {
        e.preventDefault();
      });
      self.on('click', e => {
        e.preventDefault();
        if (!self.disabled()) {
          self.checked(!self.checked());
        }
      });
      self.checked(self.settings.checked);
    },
    checked(state) {
      if (!arguments.length) {
        return this.state.get('checked');
      }
      this.state.set('checked', state);
      return this;
    },
    value(state) {
      if (!arguments.length) {
        return this.checked();
      }
      return this.checked(state);
    },
    renderHtml() {
      const self = this, id = self._id, prefix = self.classPrefix;
      return '<div id="' + id + '" class="' + self.classes + '" unselectable="on" aria-labelledby="' + id + '-al" tabindex="-1">' + '<i class="' + prefix + 'ico ' + prefix + 'i-checkbox"></i>' + '<span id="' + id + '-al" class="' + prefix + 'label">' + self.encode(self.state.get('text')) + '</span>' + '</div>';
    },
    bindStates() {
      const self = this;
      function checked(state) {
        self.classes.toggle('checked', state);
        self.aria('checked', state);
      }
      self.state.on('change:text', e => {
        self.getEl('al').firstChild.data = self.translate(e.value);
      });
      self.state.on('change:checked change:value', e => {
        self.fire('change');
        checked(e.value);
      });
      self.state.on('change:icon', e => {
        let icon = e.value;
        const prefix = self.classPrefix;
        if (typeof icon === 'undefined') {
          return self.settings.icon;
        }
        self.settings.icon = icon;
        icon = icon ? prefix + 'ico ' + prefix + 'i-' + self.settings.icon : '';
        const btnElm = self.getEl().firstChild;
        let iconElm = btnElm.getElementsByTagName('i')[0];
        if (icon) {
          if (!iconElm || iconElm !== btnElm.firstChild) {
            iconElm = document.createElement('i');
            btnElm.insertBefore(iconElm, btnElm.firstChild);
          }
          iconElm.className = icon;
        } else if (iconElm) {
          btnElm.removeChild(iconElm);
        }
      });
      if (self.state.get('checked')) {
        checked(true);
      }
      return self._super();
    }
  });

  const global$13 = tinymce.util.Tools.resolve('tinymce.util.VK');

  const ComboBox = Widget.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      settings = self.settings;
      self.classes.add('combobox');
      self.subinput = true;
      self.ariaTarget = 'inp';
      settings.menu = settings.menu || settings.values;
      if (settings.menu) {
        settings.icon = 'caret';
      }
      self.on('click', e => {
        let elm = e.target;
        const root = self.getEl();
        if (!global$9.contains(root, elm) && elm !== root) {
          return;
        }
        while (elm && elm !== root) {
          if (elm.id && elm.id.indexOf('-open') !== -1) {
            self.fire('action');
            if (settings.menu) {
              self.showMenu();
              if (e.aria) {
                self.menu.items()[0].focus();
              }
            }
          }
          elm = elm.parentNode;
        }
      });
      self.on('keydown', e => {
        let rootControl;
        if (e.keyCode === 13 && e.target.nodeName === 'INPUT') {
          e.preventDefault();
          self.parents().reverse().each(ctrl => {
            if (ctrl.toJSON) {
              rootControl = ctrl;
              return false;
            }
          });
          self.fire('submit', { data: rootControl.toJSON() });
        }
      });
      self.on('keyup', e => {
        if (e.target.nodeName === 'INPUT') {
          const oldValue = self.state.get('value');
          const newValue = e.target.value;
          if (newValue !== oldValue) {
            self.state.set('value', newValue);
            self.fire('autocomplete', e);
          }
        }
      });
      self.on('mouseover', e => {
        const tooltip = self.tooltip().moveTo(-65535);
        if (self.statusLevel() && e.target.className.indexOf(self.classPrefix + 'status') !== -1) {
          const statusMessage = self.statusMessage() || 'Ok';
          const rel = tooltip.text(statusMessage).show().testMoveRel(e.target, [
            'bc-tc',
            'bc-tl',
            'bc-tr'
          ]);
          tooltip.classes.toggle('tooltip-n', rel === 'bc-tc');
          tooltip.classes.toggle('tooltip-nw', rel === 'bc-tl');
          tooltip.classes.toggle('tooltip-ne', rel === 'bc-tr');
          tooltip.moveRel(e.target, rel);
        }
      });
    },
    statusLevel(value) {
      if (arguments.length > 0) {
        this.state.set('statusLevel', value);
      }
      return this.state.get('statusLevel');
    },
    statusMessage(value) {
      if (arguments.length > 0) {
        this.state.set('statusMessage', value);
      }
      return this.state.get('statusMessage');
    },
    showMenu() {
      const self = this;
      const settings = self.settings;
      let menu;
      if (!self.menu) {
        menu = settings.menu || [];
        if (menu.length) {
          menu = {
            type: 'menu',
            items: menu
          };
        } else {
          menu.type = menu.type || 'menu';
        }
        self.menu = Factory.create(menu).parent(self).renderTo(self.getContainerElm());
        self.fire('createmenu');
        self.menu.reflow();
        self.menu.on('cancel', e => {
          if (e.control === self.menu) {
            self.focus();
          }
        });
        self.menu.on('show hide', e => {
          e.control.items().each(ctrl => {
            ctrl.active(ctrl.value() === self.value());
          });
        }).fire('show');
        self.menu.on('select', e => {
          self.value(e.control.value());
        });
        self.on('focusin', e => {
          if (e.target.tagName.toUpperCase() === 'INPUT') {
            self.menu.hide();
          }
        });
        self.aria('expanded', true);
      }
      self.menu.show();
      self.menu.layoutRect({ w: self.layoutRect().w });
      self.menu.moveRel(self.getEl(), self.isRtl() ? [
        'br-tr',
        'tr-br'
      ] : [
        'bl-tl',
        'tl-bl'
      ]);
    },
    focus() {
      this.getEl('inp').focus();
    },
    repaint() {
      const self = this, elm = self.getEl(), openElm = self.getEl('open'), rect = self.layoutRect();
      let width, lineHeight, innerPadding = 0;
      const inputElm = elm.firstChild;
      if (self.statusLevel() && self.statusLevel() !== 'none') {
        innerPadding = parseInt(funcs.getRuntimeStyle(inputElm, 'padding-right'), 10) - parseInt(funcs.getRuntimeStyle(inputElm, 'padding-left'), 10);
      }
      if (openElm) {
        width = rect.w - funcs.getSize(openElm).width - 10;
      } else {
        width = rect.w - 10;
      }
      const doc = document;
      if (doc.all && (!doc.documentMode || doc.documentMode <= 8)) {
        lineHeight = self.layoutRect().h - 2 + 'px';
      }
      global$9(inputElm).css({
        width: width - innerPadding,
        lineHeight
      });
      self._super();
      return self;
    },
    postRender() {
      const self = this;
      global$9(this.getEl('inp')).on('change', e => {
        self.state.set('value', e.target.value);
        self.fire('change', e);
      });
      return self._super();
    },
    renderHtml() {
      const self = this, id = self._id, settings = self.settings, prefix = self.classPrefix;
      const value = self.state.get('value') || '';
      let icon, text, openBtnHtml = '', extraAttrs = '', statusHtml = '';
      if ('spellcheck' in settings) {
        extraAttrs += ' spellcheck="' + settings.spellcheck + '"';
      }
      if (settings.maxLength) {
        extraAttrs += ' maxlength="' + settings.maxLength + '"';
      }
      if (settings.size) {
        extraAttrs += ' size="' + settings.size + '"';
      }
      if (settings.subtype) {
        extraAttrs += ' type="' + settings.subtype + '"';
      }
      statusHtml = '<i id="' + id + '-status" class="mce-status mce-ico" style="display: none"></i>';
      if (self.disabled()) {
        extraAttrs += ' disabled="disabled"';
      }
      icon = settings.icon;
      if (icon && icon !== 'caret') {
        icon = prefix + 'ico ' + prefix + 'i-' + settings.icon;
      }
      text = self.state.get('text');
      if (icon || text) {
        openBtnHtml = '<div id="' + id + '-open" class="' + prefix + 'btn ' + prefix + 'open" tabIndex="-1" role="button">' + '<button id="' + id + '-action" type="button" hidefocus="1" tabindex="-1">' + (icon !== 'caret' ? '<i class="' + icon + '"></i>' : '<i class="' + prefix + 'caret"></i>') + (text ? (icon ? ' ' : '') + text : '') + '</button>' + '</div>';
        self.classes.add('has-open');
      }
      return '<div id="' + id + '" class="' + self.classes + '">' + '<input id="' + id + '-inp" class="' + prefix + 'textbox" value="' + self.encode(value, false) + '" hidefocus="1"' + extraAttrs + ' placeholder="' + self.encode(settings.placeholder) + '" />' + statusHtml + openBtnHtml + '</div>';
    },
    value(value) {
      if (arguments.length) {
        this.state.set('value', value);
        return this;
      }
      if (this.state.get('rendered')) {
        this.state.set('value', this.getEl('inp').value);
      }
      return this.state.get('value');
    },
    showAutoComplete(items, term) {
      const self = this;
      if (items.length === 0) {
        self.hideMenu();
        return;
      }
      const insert = (value, title) => () => {
        self.fire('selectitem', {
          title,
          value
        });
      };
      if (self.menu) {
        self.menu.items().remove();
      } else {
        self.menu = Factory.create({
          type: 'menu',
          classes: 'combobox-menu',
          layout: 'flow'
        }).parent(self).renderTo();
      }
      Tools.each(items, item => {
        self.menu.add({
          text: item.title,
          url: item.previewUrl,
          match: term,
          classes: 'menu-item-ellipsis',
          onclick: insert(item.value, item.title)
        });
      });
      self.menu.renderNew();
      self.hideMenu();
      self.menu.on('cancel', e => {
        if (e.control.parent() === self.menu) {
          e.stopPropagation();
          self.focus();
          self.hideMenu();
        }
      });
      self.menu.on('select', () => {
        self.focus();
      });
      const maxW = self.layoutRect().w;
      self.menu.layoutRect({
        w: maxW,
        minW: 0,
        maxW
      });
      self.menu.repaint();
      self.menu.reflow();
      self.menu.show();
      self.menu.moveRel(self.getEl(), self.isRtl() ? [
        'br-tr',
        'tr-br'
      ] : [
        'bl-tl',
        'tl-bl'
      ]);
    },
    hideMenu() {
      if (this.menu) {
        this.menu.hide();
      }
    },
    bindStates() {
      const self = this;
      self.state.on('change:value', e => {
        if (self.getEl('inp').value !== e.value) {
          self.getEl('inp').value = e.value;
        }
      });
      self.state.on('change:disabled', e => {
        self.getEl('inp').disabled = e.value;
      });
      self.state.on('change:statusLevel', e => {
        const statusIconElm = self.getEl('status');
        const prefix = self.classPrefix, value = e.value;
        funcs.css(statusIconElm, 'display', value === 'none' ? 'none' : '');
        funcs.toggleClass(statusIconElm, prefix + 'i-checkmark', value === 'ok');
        funcs.toggleClass(statusIconElm, prefix + 'i-warning', value === 'warn');
        funcs.toggleClass(statusIconElm, prefix + 'i-error', value === 'error');
        self.classes.toggle('has-status', value !== 'none');
        self.repaint();
      });
      funcs.on(self.getEl('status'), 'mouseleave', () => {
        self.tooltip().hide();
      });
      self.on('cancel', e => {
        if (self.menu && self.menu.visible()) {
          e.stopPropagation();
          self.hideMenu();
        }
      });
      const focusIdx = (idx, menu) => {
        if (menu && menu.items().length > 0) {
          menu.items().eq(idx)[0].focus();
        }
      };
      self.on('keydown', e => {
        const keyCode = e.keyCode;
        if (e.target.nodeName === 'INPUT') {
          if (keyCode === global$13.DOWN) {
            e.preventDefault();
            self.fire('autocomplete');
            focusIdx(0, self.menu);
          } else if (keyCode === global$13.UP) {
            e.preventDefault();
            focusIdx(-1, self.menu);
          }
        }
      });
      return self._super();
    },
    remove() {
      global$9(this.getEl('inp')).off();
      if (this.menu) {
        this.menu.remove();
      }
      this._super();
    }
  });

  const ColorBox = ComboBox.extend({
    init(settings) {
      const self = this;
      settings.spellcheck = false;
      if (settings.onaction) {
        settings.icon = 'none';
      }
      self._super(settings);
      self.classes.add('colorbox');
      self.on('change keyup postrender', () => {
        self.repaintColor(self.value());
      });
    },
    repaintColor(value) {
      const openElm = this.getEl('open');
      const elm = openElm ? openElm.getElementsByTagName('i')[0] : null;
      if (elm) {
        try {
          elm.style.background = value;
        } catch (ex) {
        }
      }
    },
    bindStates() {
      const self = this;
      self.state.on('change:value', e => {
        if (self.state.get('rendered')) {
          self.repaintColor(e.value);
        }
      });
      return self._super();
    }
  });

  const PanelButton = Button.extend({
    showPanel() {
      const self = this, settings = self.settings;
      self.classes.add('opened');
      if (!self.panel) {
        let panelSettings = settings.panel;
        if (panelSettings.type) {
          panelSettings = {
            layout: 'grid',
            items: panelSettings
          };
        }
        panelSettings.role = panelSettings.role || 'dialog';
        panelSettings.popover = true;
        panelSettings.autohide = true;
        panelSettings.ariaRoot = true;
        self.panel = new FloatPanel(panelSettings).on('hide', () => {
          self.classes.remove('opened');
        }).on('cancel', e => {
          e.stopPropagation();
          self.focus();
          self.hidePanel();
        }).parent(self).renderTo(self.getContainerElm());
        self.panel.fire('show');
        self.panel.reflow();
      } else {
        self.panel.show();
      }
      const rtlRels = [
        'bc-tc',
        'bc-tl',
        'bc-tr'
      ];
      const ltrRels = [
        'bc-tc',
        'bc-tr',
        'bc-tl',
        'tc-bc',
        'tc-br',
        'tc-bl'
      ];
      const rel = self.panel.testMoveRel(self.getEl(), settings.popoverAlign || (self.isRtl() ? rtlRels : ltrRels));
      self.panel.classes.toggle('start', rel.substr(-1) === 'l');
      self.panel.classes.toggle('end', rel.substr(-1) === 'r');
      const isTop = rel.substr(0, 1) === 't';
      self.panel.classes.toggle('bottom', !isTop);
      self.panel.classes.toggle('top', isTop);
      self.panel.moveRel(self.getEl(), rel);
    },
    hidePanel() {
      const self = this;
      if (self.panel) {
        self.panel.hide();
      }
    },
    postRender() {
      const self = this;
      self.aria('haspopup', true);
      self.on('click', e => {
        if (e.control === self) {
          if (self.panel && self.panel.visible()) {
            self.hidePanel();
          } else {
            self.showPanel();
            self.panel.focus(!!e.aria);
          }
        }
      });
      return self._super();
    },
    remove() {
      if (this.panel) {
        this.panel.remove();
        this.panel = null;
      }
      return this._super();
    }
  });

  const ColorButton = PanelButton.extend({
    init(settings) {
      this._super(settings);
      this.classes.add('splitbtn');
      this.classes.add('colorbutton');
    },
    color(color) {
      if (color) {
        this._color = color;
        this.getEl('preview').style.backgroundColor = color;
        return this;
      }
      return this._color;
    },
    resetColor() {
      this._color = null;
      this.getEl('preview').style.backgroundColor = null;
      return this;
    },
    renderHtml() {
      const self = this, id = self._id, prefix = self.classPrefix, text = self.state.get('text');
      const icon = self.settings.icon ? prefix + 'ico ' + prefix + 'i-' + self.settings.icon : '';
      const image = self.settings.image ? ' style="background-image: url(\'' + self.settings.image + '\')"' : '';
      let textHtml = '';
      if (text) {
        self.classes.add('btn-has-text');
        textHtml = '<span class="' + prefix + 'txt">' + self.encode(text) + '</span>';
      }
      return '<div id="' + id + '" class="' + self.classes + '" role="button" tabindex="-1" aria-haspopup="true">' + '<button role="presentation" hidefocus="1" type="button" tabindex="-1">' + (icon ? '<i class="' + icon + '"' + image + '></i>' : '') + '<span id="' + id + '-preview" class="' + prefix + 'preview"></span>' + textHtml + '</button>' + '<button type="button" class="' + prefix + 'open" hidefocus="1" tabindex="-1">' + ' <i class="' + prefix + 'caret"></i>' + '</button>' + '</div>';
    },
    postRender() {
      const self = this, onClickHandler = self.settings.onclick;
      self.on('click', e => {
        if (e.aria && e.aria.key === 'down') {
          return;
        }
        if (e.control === self && !DOM.getParent(e.target, '.' + self.classPrefix + 'open')) {
          e.stopImmediatePropagation();
          onClickHandler.call(self, e);
        }
      });
      delete self.settings.onclick;
      return self._super();
    }
  });

  const global$14 = tinymce.util.Tools.resolve('tinymce.util.Color');

  const ColorPicker = Widget.extend({
    Defaults: { classes: 'widget colorpicker' },
    init(settings) {
      this._super(settings);
    },
    postRender() {
      const self = this;
      const color = self.color();
      let hsv, hueRootElm, huePointElm, svRootElm, svPointElm;
      hueRootElm = self.getEl('h');
      huePointElm = self.getEl('hp');
      svRootElm = self.getEl('sv');
      svPointElm = self.getEl('svp');
      function getPos(elm, event) {
        const pos = funcs.getPos(elm);
        let x, y;
        x = event.pageX - pos.x;
        y = event.pageY - pos.y;
        x = Math.max(0, Math.min(x / elm.clientWidth, 1));
        y = Math.max(0, Math.min(y / elm.clientHeight, 1));
        return {
          x,
          y
        };
      }
      function updateColor(hsv, hueUpdate) {
        const hue = (360 - hsv.h) / 360;
        funcs.css(huePointElm, { top: hue * 100 + '%' });
        if (!hueUpdate) {
          funcs.css(svPointElm, {
            left: hsv.s + '%',
            top: 100 - hsv.v + '%'
          });
        }
        svRootElm.style.background = global$14({
          s: 100,
          v: 100,
          h: hsv.h
        }).toHex();
        self.color().parse({
          s: hsv.s,
          v: hsv.v,
          h: hsv.h
        });
      }
      function updateSaturationAndValue(e) {
        let pos;
        pos = getPos(svRootElm, e);
        hsv.s = pos.x * 100;
        hsv.v = (1 - pos.y) * 100;
        updateColor(hsv);
        self.fire('change');
      }
      function updateHue(e) {
        let pos;
        pos = getPos(hueRootElm, e);
        hsv = color.toHsv();
        hsv.h = (1 - pos.y) * 360;
        updateColor(hsv, true);
        self.fire('change');
      }
      self._repaint = () => {
        hsv = color.toHsv();
        updateColor(hsv);
      };
      self._super();
      self._svdraghelper = new DragHelper(self._id + '-sv', {
        start: updateSaturationAndValue,
        drag: updateSaturationAndValue
      });
      self._hdraghelper = new DragHelper(self._id + '-h', {
        start: updateHue,
        drag: updateHue
      });
      self._repaint();
    },
    rgb() {
      return this.color().toRgb();
    },
    value(value) {
      const self = this;
      if (arguments.length) {
        self.color().parse(value);
        if (self._rendered) {
          self._repaint();
        }
      } else {
        return self.color().toHex();
      }
    },
    color() {
      if (!this._color) {
        this._color = global$14();
      }
      return this._color;
    },
    renderHtml() {
      const self = this;
      const id = self._id;
      const prefix = self.classPrefix;
      let hueHtml;
      const stops = '#ff0000,#ff0080,#ff00ff,#8000ff,#0000ff,#0080ff,#00ffff,#00ff80,#00ff00,#80ff00,#ffff00,#ff8000,#ff0000';
      function getOldIeFallbackHtml() {
        let i, l, html = '', gradientPrefix, stopsList;
        gradientPrefix = 'filter:progid:DXImageTransform.Microsoft.gradient(GradientType=0,startColorstr=';
        stopsList = stops.split(',');
        for (i = 0, l = stopsList.length - 1; i < l; i++) {
          html += '<div class="' + prefix + 'colorpicker-h-chunk" style="' + 'height:' + 100 / l + '%;' + gradientPrefix + stopsList[i] + ',endColorstr=' + stopsList[i + 1] + ');' + '-ms-' + gradientPrefix + stopsList[i] + ',endColorstr=' + stopsList[i + 1] + ')' + '"></div>';
        }
        return html;
      }
      const gradientCssText = 'background: -ms-linear-gradient(top,' + stops + ');' + 'background: linear-gradient(to bottom,' + stops + ');';
      hueHtml = '<div id="' + id + '-h" class="' + prefix + 'colorpicker-h" style="' + gradientCssText + '">' + getOldIeFallbackHtml() + '<div id="' + id + '-hp" class="' + prefix + 'colorpicker-h-marker"></div>' + '</div>';
      return '<div id="' + id + '" class="' + self.classes + '">' + '<div id="' + id + '-sv" class="' + prefix + 'colorpicker-sv">' + '<div class="' + prefix + 'colorpicker-overlay1">' + '<div class="' + prefix + 'colorpicker-overlay2">' + '<div id="' + id + '-svp" class="' + prefix + 'colorpicker-selector1">' + '<div class="' + prefix + 'colorpicker-selector2"></div>' + '</div>' + '</div>' + '</div>' + '</div>' + hueHtml + '</div>';
    }
  });

  const DropZone = Widget.extend({
    init(settings) {
      const self = this;
      settings = Tools.extend({
        height: 100,
        text: 'Drop an image here',
        multiple: false,
        accept: null
      }, settings);
      self._super(settings);
      self.classes.add('dropzone');
      if (settings.multiple) {
        self.classes.add('multiple');
      }
    },
    renderHtml() {
      const self = this;
      let attrs, elm;
      const cfg = self.settings;
      attrs = {
        id: self._id,
        hidefocus: '1'
      };
      elm = funcs.create('div', attrs, '<span>' + this.translate(cfg.text) + '</span>');
      if (cfg.height) {
        funcs.css(elm, 'height', cfg.height + 'px');
      }
      if (cfg.width) {
        funcs.css(elm, 'width', cfg.width + 'px');
      }
      elm.className = self.classes;
      return elm.outerHTML;
    },
    postRender() {
      const self = this;
      const toggleDragClass = e => {
        e.preventDefault();
        self.classes.toggle('dragenter');
        self.getEl().className = self.classes;
      };
      const filter = files => {
        const accept = self.settings.accept;
        if (typeof accept !== 'string') {
          return files;
        }
        const re = new RegExp('(' + accept.split(/\s*,\s*/).join('|') + ')$', 'i');
        return Tools.grep(files, file => re.test(file.name));
      };
      self._super();
      self.$el.on('dragover', e => {
        e.preventDefault();
      });
      self.$el.on('dragenter', toggleDragClass);
      self.$el.on('dragleave', toggleDragClass);
      self.$el.on('drop', e => {
        e.preventDefault();
        if (self.state.get('disabled')) {
          return;
        }
        const files = filter(e.dataTransfer.files);
        self.value = () => {
          if (!files.length) {
            return null;
          } else if (self.settings.multiple) {
            return files;
          } else {
            return files[0];
          }
        };
        if (files.length) {
          self.fire('change', e);
        }
      });
    },
    remove() {
      this.$el.off();
      this._super();
    }
  });

  const Path = Widget.extend({
    init(settings) {
      const self = this;
      if (!settings.delimiter) {
        settings.delimiter = '\xBB';
      }
      self._super(settings);
      self.classes.add('path');
      self.canFocus = true;
      self.on('click', e => {
        let index;
        const target = e.target;
        if (index = target.getAttribute('data-index')) {
          self.fire('select', {
            value: self.row()[index],
            index
          });
        }
      });
      self.row(self.settings.row);
    },
    focus() {
      const self = this;
      self.getEl().firstChild.focus();
      return self;
    },
    row(row) {
      if (!arguments.length) {
        return this.state.get('row');
      }
      this.state.set('row', row);
      return this;
    },
    renderHtml() {
      const self = this;
      return '<div id="' + self._id + '" class="' + self.classes + '">' + self._getDataPathHtml(self.state.get('row')) + '</div>';
    },
    bindStates() {
      const self = this;
      self.state.on('change:row', e => {
        self.innerHtml(self._getDataPathHtml(e.value));
      });
      return self._super();
    },
    _getDataPathHtml(data) {
      const self = this;
      const parts = data || [];
      let i, l, html = '';
      const prefix = self.classPrefix;
      for (i = 0, l = parts.length; i < l; i++) {
        html += (i > 0 ? '<div class="' + prefix + 'divider" aria-hidden="true"> ' + self.settings.delimiter + ' </div>' : '') + '<div role="button" class="' + prefix + 'path-item' + (i === l - 1 ? ' ' + prefix + 'last' : '') + '" data-index="' + i + '" tabindex="-1" id="' + self._id + '-' + i + '" aria-level="' + (i + 1) + '">' + parts[i].name + '</div>';
      }
      if (!html) {
        html = '<div class="' + prefix + 'path-item">\xA0</div>';
      }
      return html;
    }
  });

  const ElementPath = Path.extend({
    postRender() {
      const self = this, editor = self.settings.editor;
      function isHidden(elm) {
        if (elm.nodeType === 1) {
          if (elm.nodeName === 'BR' || !!elm.getAttribute('data-mce-bogus')) {
            return true;
          }
          if (elm.getAttribute('data-mce-type') === 'bookmark') {
            return true;
          }
        }
        return false;
      }
      if (editor.settings.elementpath !== false) {
        self.on('select', function (e) {
          editor.focus();
          editor.selection.select(this.row()[e.index].element);
          editor.nodeChanged();
        });
        editor.on('nodeChange', e => {
          const outParents = [];
          const parents = e.parents;
          let i = parents.length;
          while (i--) {
            if (parents[i].nodeType === 1 && !isHidden(parents[i])) {
              const args = editor.fire('ResolveName', {
                name: parents[i].nodeName.toLowerCase(),
                target: parents[i]
              });
              if (!args.isDefaultPrevented()) {
                outParents.push({
                  name: args.name,
                  element: parents[i]
                });
              }
              if (args.isPropagationStopped()) {
                break;
              }
            }
          }
          self.row(outParents);
        });
      }
      return self._super();
    }
  });

  const FormItem = Container.extend({
    Defaults: {
      layout: 'flex',
      align: 'center',
      defaults: { flex: 1 }
    },
    renderHtml() {
      const self = this, layout = self._layout, prefix = self.classPrefix;
      self.classes.add('formitem');
      layout.preRender(self);
      return '<div id="' + self._id + '" class="' + self.classes + '" hidefocus="1" tabindex="-1">' + (self.settings.title ? '<div id="' + self._id + '-title" class="' + prefix + 'title">' + self.settings.title + '</div>' : '') + '<div id="' + self._id + '-body" class="' + self.bodyClasses + '">' + (self.settings.html || '') + layout.renderHtml(self) + '</div>' + '</div>';
    }
  });

  const Form = Container.extend({
    Defaults: {
      containerCls: 'form',
      layout: 'flex',
      direction: 'column',
      align: 'stretch',
      flex: 1,
      padding: 15,
      labelGap: 30,
      spacing: 10,
      callbacks: {
        submit() {
          this.submit();
        }
      }
    },
    preRender() {
      const self = this, items = self.items();
      if (!self.settings.formItemDefaults) {
        self.settings.formItemDefaults = {
          layout: 'flex',
          autoResize: 'overflow',
          defaults: { flex: 1 }
        };
      }
      items.each(ctrl => {
        let formItem;
        const label = ctrl.settings.label;
        if (label) {
          formItem = new FormItem(Tools.extend({
            items: {
              type: 'label',
              id: ctrl._id + '-l',
              text: label,
              flex: 0,
              forId: ctrl._id,
              disabled: ctrl.disabled()
            }
          }, self.settings.formItemDefaults));
          formItem.type = 'formitem';
          ctrl.aria('labelledby', ctrl._id + '-l');
          if (typeof ctrl.settings.flex === 'undefined') {
            ctrl.settings.flex = 1;
          }
          self.replace(ctrl, formItem);
          formItem.add(ctrl);
        }
      });
    },
    submit() {
      return this.fire('submit', { data: this.toJSON() });
    },
    postRender() {
      const self = this;
      self._super();
      self.fromJSON(self.settings.data);
    },
    bindStates() {
      const self = this;
      self._super();
      function recalcLabels() {
        let maxLabelWidth = 0;
        const labels = [];
        let i, labelGap, items;
        if (self.settings.labelGapCalc === false) {
          return;
        }
        if (self.settings.labelGapCalc === 'children') {
          items = self.find('formitem');
        } else {
          items = self.items();
        }
        items.filter('formitem').each(item => {
          const labelCtrl = item.items()[0], labelWidth = labelCtrl.getEl().clientWidth;
          maxLabelWidth = labelWidth > maxLabelWidth ? labelWidth : maxLabelWidth;
          labels.push(labelCtrl);
        });
        labelGap = self.settings.labelGap || 0;
        i = labels.length;
        while (i--) {
          labels[i].settings.minWidth = maxLabelWidth + labelGap;
        }
      }
      self.on('show', recalcLabels);
      recalcLabels();
    }
  });

  const FieldSet = Form.extend({
    Defaults: {
      containerCls: 'fieldset',
      layout: 'flex',
      direction: 'column',
      align: 'stretch',
      flex: 1,
      padding: '25 15 5 15',
      labelGap: 30,
      spacing: 10,
      border: 1
    },
    renderHtml() {
      const self = this, layout = self._layout, prefix = self.classPrefix;
      self.preRender();
      layout.preRender(self);
      return '<fieldset id="' + self._id + '" class="' + self.classes + '" hidefocus="1" tabindex="-1">' + (self.settings.title ? '<legend id="' + self._id + '-title" class="' + prefix + 'fieldset-title">' + self.settings.title + '</legend>' : '') + '<div id="' + self._id + '-body" class="' + self.bodyClasses + '">' + (self.settings.html || '') + layout.renderHtml(self) + '</div>' + '</fieldset>';
    }
  });

  let unique$1 = 0;
  const generate = prefix => {
    const date = new Date();
    const time = date.getTime();
    const random = Math.floor(Math.random() * 1000000000);
    unique$1++;
    return prefix + '_' + random + unique$1 + String(time);
  };
  const $_gah803urjfjm4nhq = { generate };

  const fromHtml = (html, scope) => {
    const doc = scope || document;
    const div = doc.createElement('div');
    div.innerHTML = html;
    if (!div.hasChildNodes() || div.childNodes.length > 1) {
      console.error('HTML does not have a single root node', html);
      throw 'HTML must have a single root node';
    }
    return fromDom(div.childNodes[0]);
  };
  const fromTag = (tag, scope) => {
    const doc = scope || document;
    const node = doc.createElement(tag);
    return fromDom(node);
  };
  const fromText = (text, scope) => {
    const doc = scope || document;
    const node = doc.createTextNode(text);
    return fromDom(node);
  };
  var fromDom = node => {
    if (node === null || node === undefined)
      throw new Error('Node cannot be null or undefined');
    return { dom: Fun.constant(node) };
  };
  const fromPoint = (doc, x, y) => Option.from(doc.dom().elementFromPoint(x, y)).map(fromDom);
  const $_1zx2m1usjfjm4nhs = {
    fromHtml,
    fromTag,
    fromText,
    fromDom,
    fromPoint
  };

  const cached = f => {
    let called = false;
    let r;
    return function(...args) {
      if (!called) {
        called = true;
        r = f(...args);
      }
      return r;
    };
  };
  const $_a7xvtruwjfjm4ni1 = { cached };

  const $_5oou0suyjfjm4nie = {
    ATTRIBUTE: 2,
    CDATA_SECTION: 4,
    COMMENT: 8,
    DOCUMENT: 9,
    DOCUMENT_TYPE: 10,
    DOCUMENT_FRAGMENT: 11,
    ELEMENT: 1,
    TEXT: 3,
    PROCESSING_INSTRUCTION: 7,
    ENTITY_REFERENCE: 5,
    ENTITY: 6,
    NOTATION: 12
  };

  const name = element => {
    const r = element.dom().nodeName;
    return r.toLowerCase();
  };
  const type = element => element.dom().nodeType;
  const value = element => element.dom().nodeValue;
  const isType = t => element => type(element) === t;
  const isComment = element => type(element) === $_5oou0suyjfjm4nie.COMMENT || name(element) === '#comment';
  const isElement = isType($_5oou0suyjfjm4nie.ELEMENT);
  const isText = isType($_5oou0suyjfjm4nie.TEXT);
  const isDocument = isType($_5oou0suyjfjm4nie.DOCUMENT);
  const $_1rmgj9uxjfjm4ni2 = {
    name,
    type,
    value,
    isElement,
    isText,
    isDocument,
    isComment
  };

  const inBody = element => {
    const dom = $_1rmgj9uxjfjm4ni2.isText(element) ? element.dom().parentNode : element.dom();
    return dom !== undefined && dom !== null && dom.ownerDocument.body.contains(dom);
  };
  const body = $_a7xvtruwjfjm4ni1.cached(() => getBody($_1zx2m1usjfjm4nhs.fromDom(document)));
  var getBody = doc => {
    const body = doc.dom().body;
    if (body === null || body === undefined)
      throw 'Body is not available yet';
    return $_1zx2m1usjfjm4nhs.fromDom(body);
  };
  const $_d4kplmuvjfjm4nhz = {
    body,
    getBody,
    inBody
  };

  const typeOf = x => {
    if (x === null)
      return 'null';
    const t = typeof x;
    if (t === 'object' && Array.prototype.isPrototypeOf(x))
      return 'array';
    if (t === 'object' && String.prototype.isPrototypeOf(x))
      return 'string';
    return t;
  };
  const isType$1 = type => value => typeOf(value) === type;
  const $_dt75eqv0jfjm4nio = {
    isString: isType$1('string'),
    isObject: isType$1('object'),
    isArray: isType$1('array'),
    isNull: isType$1('null'),
    isBoolean: isType$1('boolean'),
    isUndefined: isType$1('undefined'),
    isFunction: isType$1('function'),
    isNumber: isType$1('number')
  };

  function Immutable(...args) {
    const fields = [];
    for (let _i = 0; _i < args.length; _i++) {
      fields[_i] = args[_i];
    }
    return function(...args) {
      const values = [];
      for (let _i = 0; _i < args.length; _i++) {
        values[_i] = args[_i];
      }
      if (fields.length !== values.length) {
        throw new Error('Wrong number of arguments to struct. Expected "[' + fields.length + ']", got ' + values.length + ' arguments');
      }
      const struct = {};
      Arr.each(fields, (name, i) => {
        struct[name] = Fun.constant(values[i]);
      });
      return struct;
    };
  }

  const keys = (() => {
    const fastKeys = Object.keys;
    const slowKeys = o => {
      const r = [];
      for (const i in o) {
        if (o.hasOwnProperty(i)) {
          r.push(i);
        }
      }
      return r;
    };
    return fastKeys === undefined ? slowKeys : fastKeys;
  })();
  const each$1 = (obj, f) => {
    const props = keys(obj);
    for (let k = 0, len = props.length; k < len; k++) {
      const i = props[k];
      const x = obj[i];
      f(x, i, obj);
    }
  };
  const objectMap = (obj, f) => tupleMap(obj, (x, i, obj) => ({
    k: i,
    v: f(x, i, obj)
  }));
  var tupleMap = (obj, f) => {
    const r = {};
    each$1(obj, (x, i) => {
      const tuple = f(x, i, obj);
      r[tuple.k] = tuple.v;
    });
    return r;
  };
  const bifilter = (obj, pred) => {
    const t = {};
    const f = {};
    each$1(obj, (x, i) => {
      const branch = pred(x, i) ? t : f;
      branch[i] = x;
    });
    return {
      t,
      f
    };
  };
  const mapToArray = (obj, f) => {
    const r = [];
    each$1(obj, (value, name) => {
      r.push(f(value, name));
    });
    return r;
  };
  const find$1 = (obj, pred) => {
    const props = keys(obj);
    for (let k = 0, len = props.length; k < len; k++) {
      const i = props[k];
      const x = obj[i];
      if (pred(x, i, obj)) {
        return Option.some(x);
      }
    }
    return Option.none();
  };
  const values = obj => mapToArray(obj, v => v);
  const size = obj => values(obj).length;
  const $_9qvgkwv4jfjm4niu = {
    bifilter,
    each: each$1,
    map: objectMap,
    mapToArray,
    tupleMap,
    find: find$1,
    keys,
    values,
    size
  };

  const sort$1 = arr => arr.slice(0).sort();
  const reqMessage = (required, keys) => {
    throw new Error('All required keys (' + sort$1(required).join(', ') + ') were not specified. Specified keys were: ' + sort$1(keys).join(', ') + '.');
  };
  const unsuppMessage = unsupported => {
    throw new Error('Unsupported keys for object: ' + sort$1(unsupported).join(', '));
  };
  const validateStrArr = (label, array) => {
    if (!$_dt75eqv0jfjm4nio.isArray(array))
      throw new Error('The ' + label + ' fields must be an array. Was: ' + array + '.');
    Arr.each(array, a => {
      if (!$_dt75eqv0jfjm4nio.isString(a))
        throw new Error('The value ' + a + ' in the ' + label + ' fields was not a string.');
    });
  };
  const invalidTypeMessage = (incorrect, type) => {
    throw new Error('All values need to be of type: ' + type + '. Keys (' + sort$1(incorrect).join(', ') + ') were not.');
  };
  const checkDupes = everything => {
    const sorted = sort$1(everything);
    const dupe = Arr.find(sorted, (s, i) => i < sorted.length - 1 && s === sorted[i + 1]);
    dupe.each(d => {
      throw new Error('The field: ' + d + ' occurs more than once in the combined fields: [' + sorted.join(', ') + '].');
    });
  };
  const $_8nz9w7v5jfjm4niw = {
    sort: sort$1,
    reqMessage,
    unsuppMessage,
    validateStrArr,
    invalidTypeMessage,
    checkDupes
  };

  function MixedBag (required, optional) {
    const everything = required.concat(optional);
    if (everything.length === 0)
      throw new Error('You must specify at least one required or optional field.');
    $_8nz9w7v5jfjm4niw.validateStrArr('required', required);
    $_8nz9w7v5jfjm4niw.validateStrArr('optional', optional);
    $_8nz9w7v5jfjm4niw.checkDupes(everything);
    return obj => {
      const keys = $_9qvgkwv4jfjm4niu.keys(obj);
      const allReqd = Arr.forall(required, req => Arr.contains(keys, req));
      if (!allReqd)
        $_8nz9w7v5jfjm4niw.reqMessage(required, keys);
      const unsupported = Arr.filter(keys, key => !Arr.contains(everything, key));
      if (unsupported.length > 0)
        $_8nz9w7v5jfjm4niw.unsuppMessage(unsupported);
      const r = {};
      Arr.each(required, req => {
        r[req] = Fun.constant(obj[req]);
      });
      Arr.each(optional, opt => {
        r[opt] = Fun.constant(Object.prototype.hasOwnProperty.call(obj, opt) ? Option.some(obj[opt]) : Option.none());
      });
      return r;
    };
  }

  const $_axeqtpv1jfjm4nip = {
    immutable: Immutable,
    immutableBag: MixedBag
  };

  const toArray = (target, f) => {
    const r = [];
    const recurse = e => {
      r.push(e);
      return f(e);
    };
    let cur = f(target);
    do {
      cur = cur.bind(recurse);
    } while (cur.isSome());
    return r;
  };
  const $_dx04dnv6jfjm4niy = { toArray };

  const global$15 = typeof window !== 'undefined' ? window : Function('return this;')();

  const path = (parts, scope) => {
    let o = scope !== undefined && scope !== null ? scope : global$15;
    for (let i = 0; i < parts.length && o !== undefined && o !== null; ++i)
      o = o[parts[i]];
    return o;
  };
  const resolve = (p, scope) => {
    const parts = p.split('.');
    return path(parts, scope);
  };
  const step = (o, part) => {
    if (o[part] === undefined || o[part] === null)
      o[part] = {};
    return o[part];
  };
  const forge = (parts, target) => {
    let o = target !== undefined ? target : global$15;
    for (let i = 0; i < parts.length; ++i)
      o = step(o, parts[i]);
    return o;
  };
  const namespace = (name, target) => {
    const parts = name.split('.');
    return forge(parts, target);
  };
  const $_3yh035vajfjm4nj9 = {
    path,
    resolve,
    forge,
    namespace
  };

  const unsafe = (name, scope) => $_3yh035vajfjm4nj9.resolve(name, scope);
  const getOrDie = (name, scope) => {
    const actual = unsafe(name, scope);
    if (actual === undefined || actual === null)
      throw name + ' not available on this browser';
    return actual;
  };
  const $_gct1eov9jfjm4nj7 = { getOrDie };

  const node = () => {
    const f = $_gct1eov9jfjm4nj7.getOrDie('Node');
    return f;
  };
  const compareDocumentPosition = (a, b, match) => (a.compareDocumentPosition(b) & match) !== 0;
  const documentPositionPreceding = (a, b) => compareDocumentPosition(a, b, node().DOCUMENT_POSITION_PRECEDING);
  const documentPositionContainedBy = (a, b) => compareDocumentPosition(a, b, node().DOCUMENT_POSITION_CONTAINED_BY);
  const $_k0yekv8jfjm4nj6 = {
    documentPositionPreceding,
    documentPositionContainedBy
  };

  const firstMatch = (regexes, s) => {
    for (let i = 0; i < regexes.length; i++) {
      const x = regexes[i];
      if (x.test(s))
        return x;
    }
    return undefined;
  };
  const find$2 = (regexes, agent) => {
    const r = firstMatch(regexes, agent);
    if (!r)
      return {
        major: 0,
        minor: 0
      };
    const group = i => Number(agent.replace(r, '$' + i));
    return nu(group(1), group(2));
  };
  const detect = (versionRegexes, agent) => {
    const cleanedAgent = String(agent).toLowerCase();
    if (versionRegexes.length === 0)
      return unknown();
    return find$2(versionRegexes, cleanedAgent);
  };
  var unknown = () => nu(0, 0);
  var nu = (major, minor) => ({
    major,
    minor
  });
  const $_a9lq2nvfjfjm4njg = {
    nu,
    detect,
    unknown
  };

  const edge = 'Edge';
  const chrome = 'Chrome';
  const ie = 'IE';
  const opera = 'Opera';
  const firefox = 'Firefox';
  const safari = 'Safari';
  const isBrowser = (name, current) => () => current === name;
  const unknown$1 = () => nu$1({
    current: undefined,
    version: $_a9lq2nvfjfjm4njg.unknown()
  });
  var nu$1 = info => {
    const current = info.current;
    const version = info.version;
    return {
      current,
      version,
      isEdge: isBrowser(edge, current),
      isChrome: isBrowser(chrome, current),
      isIE: isBrowser(ie, current),
      isOpera: isBrowser(opera, current),
      isFirefox: isBrowser(firefox, current),
      isSafari: isBrowser(safari, current)
    };
  };
  const $_2dyz5ovejfjm4nje = {
    unknown: unknown$1,
    nu: nu$1,
    edge: Fun.constant(edge),
    chrome: Fun.constant(chrome),
    ie: Fun.constant(ie),
    opera: Fun.constant(opera),
    firefox: Fun.constant(firefox),
    safari: Fun.constant(safari)
  };

  const windows$1 = 'Windows';
  const ios = 'iOS';
  const android = 'Android';
  const linux = 'Linux';
  const osx = 'OSX';
  const solaris = 'Solaris';
  const freebsd = 'FreeBSD';
  const isOS = (name, current) => () => current === name;
  const unknown$2 = () => nu$2({
    current: undefined,
    version: $_a9lq2nvfjfjm4njg.unknown()
  });
  var nu$2 = info => {
    const current = info.current;
    const version = info.version;
    return {
      current,
      version,
      isWindows: isOS(windows$1, current),
      isiOS: isOS(ios, current),
      isAndroid: isOS(android, current),
      isOSX: isOS(osx, current),
      isLinux: isOS(linux, current),
      isSolaris: isOS(solaris, current),
      isFreeBSD: isOS(freebsd, current)
    };
  };
  const $_a65rkbvgjfjm4nji = {
    unknown: unknown$2,
    nu: nu$2,
    windows: Fun.constant(windows$1),
    ios: Fun.constant(ios),
    android: Fun.constant(android),
    linux: Fun.constant(linux),
    osx: Fun.constant(osx),
    solaris: Fun.constant(solaris),
    freebsd: Fun.constant(freebsd)
  };

  function DeviceType (os, browser, userAgent) {
    const isiPad = os.isiOS() && /ipad/i.test(userAgent) === true;
    const isiPhone = os.isiOS() && !isiPad;
    const isAndroid3 = os.isAndroid() && os.version.major === 3;
    const isAndroid4 = os.isAndroid() && os.version.major === 4;
    const isTablet = isiPad || isAndroid3 || isAndroid4 && /mobile/i.test(userAgent) === true;
    const isTouch = os.isiOS() || os.isAndroid();
    const isPhone = isTouch && !isTablet;
    const iOSwebview = browser.isSafari() && os.isiOS() && /safari/i.test(userAgent) === false;
    return {
      isiPad: Fun.constant(isiPad),
      isiPhone: Fun.constant(isiPhone),
      isTablet: Fun.constant(isTablet),
      isPhone: Fun.constant(isPhone),
      isTouch: Fun.constant(isTouch),
      isAndroid: os.isAndroid,
      isiOS: os.isiOS,
      isWebView: Fun.constant(iOSwebview)
    };
  }

  const detect$1 = (candidates, userAgent) => {
    const agent = String(userAgent).toLowerCase();
    return Arr.find(candidates, candidate => candidate.search(agent));
  };
  const detectBrowser = (browsers, userAgent) => detect$1(browsers, userAgent).map(browser => {
    const version = $_a9lq2nvfjfjm4njg.detect(browser.versionRegexes, userAgent);
    return {
      current: browser.name,
      version
    };
  });
  const detectOs = (oses, userAgent) => detect$1(oses, userAgent).map(os => {
    const version = $_a9lq2nvfjfjm4njg.detect(os.versionRegexes, userAgent);
    return {
      current: os.name,
      version
    };
  });
  const $_cdi74pvijfjm4njn = {
    detectBrowser,
    detectOs
  };

  const addToStart = (str, prefix) => prefix + str;
  const addToEnd = (str, suffix) => str + suffix;
  const removeFromStart = (str, numChars) => str.substring(numChars);
  const removeFromEnd = (str, numChars) => str.substring(0, str.length - numChars);
  const $_94ajbgvljfjm4njw = {
    addToStart,
    addToEnd,
    removeFromStart,
    removeFromEnd
  };

  const first = (str, count) => str.substr(0, count);
  const last$1 = (str, count) => str.substr(str.length - count, str.length);
  const head$1 = str => str === '' ? Option.none() : Option.some(str.substr(0, 1));
  const tail = str => str === '' ? Option.none() : Option.some(str.substring(1));
  const $_crkqi8vmjfjm4njx = {
    first,
    last: last$1,
    head: head$1,
    tail
  };

  const checkRange = (str, substr, start) => {
    if (substr === '')
      return true;
    if (str.length < substr.length)
      return false;
    const x = str.substr(start, start + substr.length);
    return x === substr;
  };
  const supplant = (str, obj) => {
    const isStringOrNumber = a => {
      const t = typeof a;
      return t === 'string' || t === 'number';
    };
    return str.replace(/\${([^{}]*)}/g, (a, b) => {
      const value = obj[b];
      return isStringOrNumber(value) ? value : a;
    });
  };
  const removeLeading = (str, prefix) => startsWith(str, prefix) ? $_94ajbgvljfjm4njw.removeFromStart(str, prefix.length) : str;
  const removeTrailing = (str, prefix) => endsWith(str, prefix) ? $_94ajbgvljfjm4njw.removeFromEnd(str, prefix.length) : str;
  const ensureLeading = (str, prefix) => startsWith(str, prefix) ? str : $_94ajbgvljfjm4njw.addToStart(str, prefix);
  const ensureTrailing = (str, prefix) => endsWith(str, prefix) ? str : $_94ajbgvljfjm4njw.addToEnd(str, prefix);
  const contains$1 = (str, substr) => str.indexOf(substr) !== -1;
  const capitalize = str => $_crkqi8vmjfjm4njx.head(str).bind(head => $_crkqi8vmjfjm4njx.tail(str).map(tail => head.toUpperCase() + tail)).getOr(str);
  var startsWith = (str, prefix) => checkRange(str, prefix, 0);
  var endsWith = (str, suffix) => checkRange(str, suffix, str.length - suffix.length);
  const trim = str => str.replace(/^\s+|\s+$/g, '');
  const lTrim = str => str.replace(/^\s+/g, '');
  const rTrim = str => str.replace(/\s+$/g, '');
  const $_8l8cgovkjfjm4nju = {
    supplant,
    startsWith,
    removeLeading,
    removeTrailing,
    ensureLeading,
    ensureTrailing,
    endsWith,
    contains: contains$1,
    trim,
    lTrim,
    rTrim,
    capitalize
  };

  const normalVersionRegex = /.*?version\/\ ?([0-9]+)\.([0-9]+).*/;
  const checkContains = target => uastring => $_8l8cgovkjfjm4nju.contains(uastring, target);
  const browsers = [
    {
      name: 'Edge',
      versionRegexes: [/.*?edge\/ ?([0-9]+)\.([0-9]+)$/],
      search(uastring) {
        const monstrosity = $_8l8cgovkjfjm4nju.contains(uastring, 'edge/') && $_8l8cgovkjfjm4nju.contains(uastring, 'chrome') && $_8l8cgovkjfjm4nju.contains(uastring, 'safari') && $_8l8cgovkjfjm4nju.contains(uastring, 'applewebkit');
        return monstrosity;
      }
    },
    {
      name: 'Chrome',
      versionRegexes: [
        /.*?chrome\/([0-9]+)\.([0-9]+).*/,
        normalVersionRegex
      ],
      search(uastring) {
        return $_8l8cgovkjfjm4nju.contains(uastring, 'chrome') && !$_8l8cgovkjfjm4nju.contains(uastring, 'chromeframe');
      }
    },
    {
      name: 'IE',
      versionRegexes: [
        /.*?msie\ ?([0-9]+)\.([0-9]+).*/,
        /.*?rv:([0-9]+)\.([0-9]+).*/
      ],
      search(uastring) {
        return $_8l8cgovkjfjm4nju.contains(uastring, 'msie') || $_8l8cgovkjfjm4nju.contains(uastring, 'trident');
      }
    },
    {
      name: 'Opera',
      versionRegexes: [
        normalVersionRegex,
        /.*?opera\/([0-9]+)\.([0-9]+).*/
      ],
      search: checkContains('opera')
    },
    {
      name: 'Firefox',
      versionRegexes: [/.*?firefox\/\ ?([0-9]+)\.([0-9]+).*/],
      search: checkContains('firefox')
    },
    {
      name: 'Safari',
      versionRegexes: [
        normalVersionRegex,
        /.*?cpu os ([0-9]+)_([0-9]+).*/
      ],
      search(uastring) {
        return ($_8l8cgovkjfjm4nju.contains(uastring, 'safari') || $_8l8cgovkjfjm4nju.contains(uastring, 'mobile/')) && $_8l8cgovkjfjm4nju.contains(uastring, 'applewebkit');
      }
    }
  ];
  const oses = [
    {
      name: 'Windows',
      search: checkContains('win'),
      versionRegexes: [/.*?windows\ nt\ ?([0-9]+)\.([0-9]+).*/]
    },
    {
      name: 'iOS',
      search(uastring) {
        return $_8l8cgovkjfjm4nju.contains(uastring, 'iphone') || $_8l8cgovkjfjm4nju.contains(uastring, 'ipad');
      },
      versionRegexes: [
        /.*?version\/\ ?([0-9]+)\.([0-9]+).*/,
        /.*cpu os ([0-9]+)_([0-9]+).*/,
        /.*cpu iphone os ([0-9]+)_([0-9]+).*/
      ]
    },
    {
      name: 'Android',
      search: checkContains('android'),
      versionRegexes: [/.*?android\ ?([0-9]+)\.([0-9]+).*/]
    },
    {
      name: 'OSX',
      search: checkContains('os x'),
      versionRegexes: [/.*?os\ x\ ?([0-9]+)_([0-9]+).*/]
    },
    {
      name: 'Linux',
      search: checkContains('linux'),
      versionRegexes: []
    },
    {
      name: 'Solaris',
      search: checkContains('sunos'),
      versionRegexes: []
    },
    {
      name: 'FreeBSD',
      search: checkContains('freebsd'),
      versionRegexes: []
    }
  ];
  const $_5r39zxvjjfjm4njp = {
    browsers: Fun.constant(browsers),
    oses: Fun.constant(oses)
  };

  const detect$2 = userAgent => {
    const browsers = $_5r39zxvjjfjm4njp.browsers();
    const oses = $_5r39zxvjjfjm4njp.oses();
    const browser = $_cdi74pvijfjm4njn.detectBrowser(browsers, userAgent).fold($_2dyz5ovejfjm4nje.unknown, $_2dyz5ovejfjm4nje.nu);
    const os = $_cdi74pvijfjm4njn.detectOs(oses, userAgent).fold($_a65rkbvgjfjm4nji.unknown, $_a65rkbvgjfjm4nji.nu);
    const deviceType = DeviceType(os, browser, userAgent);
    return {
      browser,
      os,
      deviceType
    };
  };
  const $_3mag30vdjfjm4njd = { detect: detect$2 };

  const detect$3 = $_a7xvtruwjfjm4ni1.cached(() => {
    const userAgent = navigator.userAgent;
    return $_3mag30vdjfjm4njd.detect(userAgent);
  });
  const $_cim1f3vcjfjm4njb = { detect: detect$3 };

  const ELEMENT = $_5oou0suyjfjm4nie.ELEMENT;
  const DOCUMENT = $_5oou0suyjfjm4nie.DOCUMENT;
  const is = (element, selector) => {
    const elem = element.dom();
    if (elem.nodeType !== ELEMENT)
      return false;
    else if (elem.matches !== undefined)
      return elem.matches(selector);
    else if (elem.msMatchesSelector !== undefined)
      return elem.msMatchesSelector(selector);
    else if (elem.webkitMatchesSelector !== undefined)
      return elem.webkitMatchesSelector(selector);
    else if (elem.mozMatchesSelector !== undefined)
      return elem.mozMatchesSelector(selector);
    else
      throw new Error('Browser lacks native selectors');
  };
  const bypassSelector = dom => dom.nodeType !== ELEMENT && dom.nodeType !== DOCUMENT || dom.childElementCount === 0;
  const all = (selector, scope) => {
    const base = scope === undefined ? document : scope.dom();
    return bypassSelector(base) ? [] : Arr.map(base.querySelectorAll(selector), $_1zx2m1usjfjm4nhs.fromDom);
  };
  const one = (selector, scope) => {
    const base = scope === undefined ? document : scope.dom();
    return bypassSelector(base) ? Option.none() : Option.from(base.querySelector(selector)).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const $_6h5b4svnjfjm4njy = {
    all,
    is,
    one
  };

  const eq = (e1, e2) => e1.dom() === e2.dom();
  const isEqualNode = (e1, e2) => e1.dom().isEqualNode(e2.dom());
  const member = (element, elements) => Arr.exists(elements, Fun.curry(eq, element));
  const regularContains = (e1, e2) => {
    const d1 = e1.dom(), d2 = e2.dom();
    return d1 === d2 ? false : d1.contains(d2);
  };
  const ieContains = (e1, e2) => $_k0yekv8jfjm4nj6.documentPositionContainedBy(e1.dom(), e2.dom());
  const browser = $_cim1f3vcjfjm4njb.detect().browser;
  const contains$2 = browser.isIE() ? ieContains : regularContains;
  const $_enmgxuv7jfjm4niz = {
    eq,
    isEqualNode,
    member,
    contains: contains$2,
    is: $_6h5b4svnjfjm4njy.is
  };

  const owner = element => $_1zx2m1usjfjm4nhs.fromDom(element.dom().ownerDocument);
  const documentElement = element => {
    const doc = owner(element);
    return $_1zx2m1usjfjm4nhs.fromDom(doc.dom().documentElement);
  };
  const defaultView = element => {
    const el = element.dom();
    const defaultView = el.ownerDocument.defaultView;
    return $_1zx2m1usjfjm4nhs.fromDom(defaultView);
  };
  const parent = element => {
    const dom = element.dom();
    return Option.from(dom.parentNode).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const findIndex$1 = element => parent(element).bind(p => {
    const kin = children(p);
    return Arr.findIndex(kin, elem => $_enmgxuv7jfjm4niz.eq(element, elem));
  });
  const parents = (element, isRoot) => {
    const stop = $_dt75eqv0jfjm4nio.isFunction(isRoot) ? isRoot : Fun.constant(false);
    let dom = element.dom();
    const ret = [];
    while (dom.parentNode !== null && dom.parentNode !== undefined) {
      const rawParent = dom.parentNode;
      const parent = $_1zx2m1usjfjm4nhs.fromDom(rawParent);
      ret.push(parent);
      if (stop(parent) === true)
        break;
      else
        dom = rawParent;
    }
    return ret;
  };
  const siblings = element => {
    const filterSelf = elements => Arr.filter(elements, x => !$_enmgxuv7jfjm4niz.eq(element, x));
    return parent(element).map(children).map(filterSelf).getOr([]);
  };
  const offsetParent = element => {
    const dom = element.dom();
    return Option.from(dom.offsetParent).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const prevSibling = element => {
    const dom = element.dom();
    return Option.from(dom.previousSibling).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const nextSibling = element => {
    const dom = element.dom();
    return Option.from(dom.nextSibling).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const prevSiblings = element => Arr.reverse($_dx04dnv6jfjm4niy.toArray(element, prevSibling));
  const nextSiblings = element => $_dx04dnv6jfjm4niy.toArray(element, nextSibling);
  var children = element => {
    const dom = element.dom();
    return Arr.map(dom.childNodes, $_1zx2m1usjfjm4nhs.fromDom);
  };
  const child = (element, index) => {
    const children = element.dom().childNodes;
    return Option.from(children[index]).map($_1zx2m1usjfjm4nhs.fromDom);
  };
  const firstChild = element => child(element, 0);
  const lastChild = element => child(element, element.dom().childNodes.length - 1);
  const childNodesCount = element => element.dom().childNodes.length;
  const hasChildNodes = element => element.dom().hasChildNodes();
  const spot = $_axeqtpv1jfjm4nip.immutable('element', 'offset');
  const leaf = (element, offset) => {
    const cs = children(element);
    return cs.length > 0 && offset < cs.length ? spot(cs[offset], 0) : spot(element, offset);
  };
  const $_2ocgbsuzjfjm4nig = {
    owner,
    defaultView,
    documentElement,
    parent,
    findIndex: findIndex$1,
    parents,
    siblings,
    prevSibling,
    offsetParent,
    prevSiblings,
    nextSibling,
    nextSiblings,
    children,
    child,
    firstChild,
    lastChild,
    childNodesCount,
    hasChildNodes,
    leaf
  };

  const all$1 = predicate => descendants($_d4kplmuvjfjm4nhz.body(), predicate);
  const ancestors = (scope, predicate, isRoot) => Arr.filter($_2ocgbsuzjfjm4nig.parents(scope, isRoot), predicate);
  const siblings$1 = (scope, predicate) => Arr.filter($_2ocgbsuzjfjm4nig.siblings(scope), predicate);
  const children$1 = (scope, predicate) => Arr.filter($_2ocgbsuzjfjm4nig.children(scope), predicate);
  var descendants = (scope, predicate) => {
    let result = [];
    Arr.each($_2ocgbsuzjfjm4nig.children(scope), x => {
      if (predicate(x)) {
        result = result.concat([x]);
      }
      result = result.concat(descendants(x, predicate));
    });
    return result;
  };
  const $_6mpgj6uujfjm4nhx = {
    all: all$1,
    ancestors,
    siblings: siblings$1,
    children: children$1,
    descendants
  };

  const all$2 = selector => $_6h5b4svnjfjm4njy.all(selector);
  const ancestors$1 = (scope, selector, isRoot) => $_6mpgj6uujfjm4nhx.ancestors(scope, e => $_6h5b4svnjfjm4njy.is(e, selector), isRoot);
  const siblings$2 = (scope, selector) => $_6mpgj6uujfjm4nhx.siblings(scope, e => $_6h5b4svnjfjm4njy.is(e, selector));
  const children$2 = (scope, selector) => $_6mpgj6uujfjm4nhx.children(scope, e => $_6h5b4svnjfjm4njy.is(e, selector));
  const descendants$1 = (scope, selector) => $_6h5b4svnjfjm4njy.all(selector, scope);
  const $_d3fvhlutjfjm4nhv = {
    all: all$2,
    ancestors: ancestors$1,
    siblings: siblings$2,
    children: children$2,
    descendants: descendants$1
  };

  const trim$1 = Tools.trim;
  const hasContentEditableState = value => node => {
    if (node && node.nodeType === 1) {
      if (node.contentEditable === value) {
        return true;
      }
      if (node.getAttribute('data-mce-contenteditable') === value) {
        return true;
      }
    }
    return false;
  };
  const isContentEditableTrue = hasContentEditableState('true');
  const isContentEditableFalse = hasContentEditableState('false');
  const create = (type, title, url, level, attach) => ({
    type,
    title,
    url,
    level,
    attach
  });
  const isChildOfContentEditableTrue = node => {
    while (node = node.parentNode) {
      const value = node.contentEditable;
      if (value && value !== 'inherit') {
        return isContentEditableTrue(node);
      }
    }
    return false;
  };
  const select = (selector, root) => Arr.map($_d3fvhlutjfjm4nhv.descendants($_1zx2m1usjfjm4nhs.fromDom(root), selector), element => element.dom());
  const getElementText = elm => elm.innerText || elm.textContent;
  const getOrGenerateId = elm => elm.id ? elm.id : $_gah803urjfjm4nhq.generate('h');
  const isAnchor = elm => elm && elm.nodeName === 'A' && (elm.id || elm.name);
  const isValidAnchor = elm => isAnchor(elm) && isEditable(elm);
  const isHeader = elm => elm && /^(H[1-6])$/.test(elm.nodeName);
  var isEditable = elm => isChildOfContentEditableTrue(elm) && !isContentEditableFalse(elm);
  const isValidHeader = elm => isHeader(elm) && isEditable(elm);
  const getLevel = elm => isHeader(elm) ? parseInt(elm.nodeName.substr(1), 10) : 0;
  const headerTarget = elm => {
    const headerId = getOrGenerateId(elm);
    const attach = () => {
      elm.id = headerId;
    };
    return create('header', getElementText(elm), '#' + headerId, getLevel(elm), attach);
  };
  const anchorTarget = elm => {
    const anchorId = elm.id || elm.name;
    const anchorText = getElementText(elm);
    return create('anchor', anchorText ? anchorText : '#' + anchorId, '#' + anchorId, 0, Fun.noop);
  };
  const getHeaderTargets = elms => Arr.map(Arr.filter(elms, isValidHeader), headerTarget);
  const getAnchorTargets = elms => Arr.map(Arr.filter(elms, isValidAnchor), anchorTarget);
  const getTargetElements = elm => {
    const elms = select('h1,h2,h3,h4,h5,h6,a:not([href])', elm);
    return elms;
  };
  const hasTitle = target => trim$1(target.title).length > 0;
  const find$3 = elm => {
    const elms = getTargetElements(elm);
    return Arr.filter(getHeaderTargets(elms).concat(getAnchorTargets(elms)), hasTitle);
  };
  const $_g1kcinuqjfjm4nhk = { find: find$3 };

  const getActiveEditor = () => window.tinymce ? window.tinymce.activeEditor : EditorManager.activeEditor;
  let history = {};
  const HISTORY_LENGTH = 5;
  const clearHistory = () => {
    history = {};
  };
  const toMenuItem = target => ({
    title: target.title,

    value: {
      title: { raw: target.title },
      url: target.url,
      attach: target.attach
    }
  });
  const toMenuItems = targets => Tools.map(targets, toMenuItem);
  const staticMenuItem = (title, url) => ({
    title,

    value: {
      title,
      url,
      attach: Fun.noop
    }
  });
  const isUniqueUrl = (url, targets) => {
    const foundTarget = Arr.exists(targets, target => target.url === url);
    return !foundTarget;
  };
  const getSetting = (editorSettings, name, defaultValue) => {
    const value = name in editorSettings ? editorSettings[name] : defaultValue;
    return value === false ? null : value;
  };
  const createMenuItems = (term, targets, fileType, editorSettings) => {
    const separator = { title: '-' };
    const fromHistoryMenuItems = history => {
      const historyItems = history.hasOwnProperty(fileType) ? history[fileType] : [];
      const uniqueHistory = Arr.filter(historyItems, url => isUniqueUrl(url, targets));
      return Tools.map(uniqueHistory, url => ({
        title: url,

        value: {
          title: url,
          url,
          attach: Fun.noop
        }
      }));
    };
    const fromMenuItems = type => {
      const filteredTargets = Arr.filter(targets, target => target.type === type);
      return toMenuItems(filteredTargets);
    };
    const anchorMenuItems = () => {
      const anchorMenuItems = fromMenuItems('anchor');
      const topAnchor = getSetting(editorSettings, 'anchor_top', '#top');
      const bottomAchor = getSetting(editorSettings, 'anchor_bottom', '#bottom');
      if (topAnchor !== null) {
        anchorMenuItems.unshift(staticMenuItem('<top>', topAnchor));
      }
      if (bottomAchor !== null) {
        anchorMenuItems.push(staticMenuItem('<bottom>', bottomAchor));
      }
      return anchorMenuItems;
    };
    const join = items => Arr.foldl(items, (a, b) => {
      const bothEmpty = a.length === 0 || b.length === 0;
      return bothEmpty ? a.concat(b) : a.concat(separator, b);
    }, []);
    if (editorSettings.typeahead_urls === false) {
      return [];
    }
    return fileType === 'file' ? join([
      filterByQuery(term, fromHistoryMenuItems(history)),
      filterByQuery(term, fromMenuItems('header')),
      filterByQuery(term, anchorMenuItems())
    ]) : filterByQuery(term, fromHistoryMenuItems(history));
  };
  const addToHistory = (url, fileType) => {
    const items = history[fileType];
    if (!/^https?/.test(url)) {
      return;
    }
    if (items) {
      if (Arr.indexOf(items, url) === -1) {
        history[fileType] = items.slice(0, HISTORY_LENGTH).concat(url);
      }
    } else {
      history[fileType] = [url];
    }
  };
  var filterByQuery = (term, menuItems) => {
    const lowerCaseTerm = term.toLowerCase();
    const result = Tools.grep(menuItems, item => item.title.toLowerCase().indexOf(lowerCaseTerm) !== -1);
    return result.length === 1 && result[0].title === term ? [] : result;
  };
  const getTitle = linkDetails => {
    const title = linkDetails.title;
    return title.raw ? title.raw : title;
  };
  const setupAutoCompleteHandler = (ctrl, editorSettings, bodyElm, fileType) => {
    const autocomplete = term => {
      const linkTargets = $_g1kcinuqjfjm4nhk.find(bodyElm);
      const menuItems = createMenuItems(term, linkTargets, fileType, editorSettings);
      ctrl.showAutoComplete(menuItems, term);
    };
    ctrl.on('autocomplete', () => {
      autocomplete(ctrl.value());
    });
    ctrl.on('selectitem', e => {
      const linkDetails = e.value;
      ctrl.value(linkDetails.url);
      const title = getTitle(linkDetails);
      if (fileType === 'image') {
        ctrl.fire('change', {
          meta: {
            alt: title,
            attach: linkDetails.attach
          }
        });
      } else {
        ctrl.fire('change', {
          meta: {
            text: title,
            attach: linkDetails.attach
          }
        });
      }
      ctrl.focus();
    });
    ctrl.on('click', e => {
      if (ctrl.value().length === 0 && e.target.nodeName === 'INPUT') {
        autocomplete('');
      }
    });
    ctrl.on('PostRender', () => {
      ctrl.getRoot().on('submit', e => {
        if (!e.isDefaultPrevented()) {
          addToHistory(ctrl.value(), fileType);
        }
      });
    });
  };
  const statusToUiState = result => {
    const status = result.status, message = result.message;
    if (status === 'valid') {
      return {
        status: 'ok',
        message
      };
    } else if (status === 'unknown') {
      return {
        status: 'warn',
        message
      };
    } else if (status === 'invalid') {
      return {
        status: 'warn',
        message
      };
    } else {
      return {
        status: 'none',
        message: ''
      };
    }
  };
  const setupLinkValidatorHandler = (ctrl, editorSettings, fileType) => {
    const validatorHandler = editorSettings.filepicker_validator_handler;
    if (validatorHandler) {
      const validateUrl_1 = url => {
        if (url.length === 0) {
          ctrl.statusLevel('none');
          return;
        }
        validatorHandler({
          url,
          type: fileType
        }, result => {
          const uiState = statusToUiState(result);
          ctrl.statusMessage(uiState.message);
          ctrl.statusLevel(uiState.status);
        });
      };
      ctrl.state.on('change:value', e => {
        validateUrl_1(e.value);
      });
    }
  };
  const FilePicker = ComboBox.extend({
    Statics: { clearHistory },
    init(settings) {
      const self = this, editor = getActiveEditor(), editorSettings = editor.settings;
      let actionCallback, fileBrowserCallback, fileBrowserCallbackTypes;
      const fileType = settings.filetype;
      settings.spellcheck = false;
      fileBrowserCallbackTypes = editorSettings.file_picker_types || editorSettings.file_browser_callback_types;
      if (fileBrowserCallbackTypes) {
        fileBrowserCallbackTypes = Tools.makeMap(fileBrowserCallbackTypes, /[, ]/);
      }
      if (!fileBrowserCallbackTypes || fileBrowserCallbackTypes[fileType]) {
        fileBrowserCallback = editorSettings.file_picker_callback;
        if (fileBrowserCallback && (!fileBrowserCallbackTypes || fileBrowserCallbackTypes[fileType])) {
          actionCallback = () => {
            let meta = self.fire('beforecall').meta;
            meta = Tools.extend({ filetype: fileType }, meta);
            fileBrowserCallback.call(editor, (value, meta) => {
              self.value(value).fire('change', { meta });
            }, self.value(), meta);
          };
        } else {
          fileBrowserCallback = editorSettings.file_browser_callback;
          if (fileBrowserCallback && (!fileBrowserCallbackTypes || fileBrowserCallbackTypes[fileType])) {
            actionCallback = () => {
              fileBrowserCallback(self.getEl('inp').id, self.value(), fileType, window);
            };
          }
        }
      }
      if (actionCallback) {
        settings.icon = 'browse';
        settings.onaction = actionCallback;
      }
      self._super(settings);
      self.classes.add('filepicker');
      setupAutoCompleteHandler(self, editorSettings, editor.getBody(), fileType);
      setupLinkValidatorHandler(self, editorSettings, fileType);
    }
  });

  const FitLayout = AbsoluteLayout.extend({
    recalc(container) {
      const contLayoutRect = container.layoutRect(), paddingBox = container.paddingBox;
      container.items().filter(':visible').each(ctrl => {
        ctrl.layoutRect({
          x: paddingBox.left,
          y: paddingBox.top,
          w: contLayoutRect.innerW - paddingBox.right - paddingBox.left,
          h: contLayoutRect.innerH - paddingBox.top - paddingBox.bottom
        });
        if (ctrl.recalc) {
          ctrl.recalc();
        }
      });
    }
  });

  const FlexLayout = AbsoluteLayout.extend({
    recalc(container) {
      let i, l, items, contLayoutRect, contPaddingBox, contSettings, align, pack, spacing, totalFlex, availableSpace, direction;
      let ctrl, ctrlLayoutRect, ctrlSettings, flex;
      const maxSizeItems = [];
      let size, maxSize, ratio, rect, pos, maxAlignEndPos;
      let sizeName, minSizeName, posName, maxSizeName, beforeName, innerSizeName, deltaSizeName, contentSizeName;
      let alignAxisName, alignInnerSizeName, alignSizeName, alignMinSizeName, alignBeforeName, alignAfterName;
      let alignDeltaSizeName, alignContentSizeName;
      const max = Math.max, min = Math.min;
      items = container.items().filter(':visible');
      contLayoutRect = container.layoutRect();
      contPaddingBox = container.paddingBox;
      contSettings = container.settings;
      direction = container.isRtl() ? contSettings.direction || 'row-reversed' : contSettings.direction;
      align = contSettings.align;
      pack = container.isRtl() ? contSettings.pack || 'end' : contSettings.pack;
      spacing = contSettings.spacing || 0;
      if (direction === 'row-reversed' || direction === 'column-reverse') {
        items = items.set(items.toArray().reverse());
        direction = direction.split('-')[0];
      }
      if (direction === 'column') {
        posName = 'y';
        sizeName = 'h';
        minSizeName = 'minH';
        maxSizeName = 'maxH';
        innerSizeName = 'innerH';
        beforeName = 'top';
        deltaSizeName = 'deltaH';
        contentSizeName = 'contentH';
        alignBeforeName = 'left';
        alignSizeName = 'w';
        alignAxisName = 'x';
        alignInnerSizeName = 'innerW';
        alignMinSizeName = 'minW';
        alignAfterName = 'right';
        alignDeltaSizeName = 'deltaW';
        alignContentSizeName = 'contentW';
      } else {
        posName = 'x';
        sizeName = 'w';
        minSizeName = 'minW';
        maxSizeName = 'maxW';
        innerSizeName = 'innerW';
        beforeName = 'left';
        deltaSizeName = 'deltaW';
        contentSizeName = 'contentW';
        alignBeforeName = 'top';
        alignSizeName = 'h';
        alignAxisName = 'y';
        alignInnerSizeName = 'innerH';
        alignMinSizeName = 'minH';
        alignAfterName = 'bottom';
        alignDeltaSizeName = 'deltaH';
        alignContentSizeName = 'contentH';
      }
      availableSpace = contLayoutRect[innerSizeName] - contPaddingBox[beforeName] - contPaddingBox[beforeName];
      maxAlignEndPos = totalFlex = 0;
      for (i = 0, l = items.length; i < l; i++) {
        ctrl = items[i];
        ctrlLayoutRect = ctrl.layoutRect();
        ctrlSettings = ctrl.settings;
        flex = ctrlSettings.flex;
        availableSpace -= i < l - 1 ? spacing : 0;
        if (flex > 0) {
          totalFlex += flex;
          if (ctrlLayoutRect[maxSizeName]) {
            maxSizeItems.push(ctrl);
          }
          ctrlLayoutRect.flex = flex;
        }
        availableSpace -= ctrlLayoutRect[minSizeName];
        size = contPaddingBox[alignBeforeName] + ctrlLayoutRect[alignMinSizeName] + contPaddingBox[alignAfterName];
        if (size > maxAlignEndPos) {
          maxAlignEndPos = size;
        }
      }
      rect = {};
      if (availableSpace < 0) {
        rect[minSizeName] = contLayoutRect[minSizeName] - availableSpace + contLayoutRect[deltaSizeName];
      } else {
        rect[minSizeName] = contLayoutRect[innerSizeName] - availableSpace + contLayoutRect[deltaSizeName];
      }
      rect[alignMinSizeName] = maxAlignEndPos + contLayoutRect[alignDeltaSizeName];
      rect[contentSizeName] = contLayoutRect[innerSizeName] - availableSpace;
      rect[alignContentSizeName] = maxAlignEndPos;
      rect.minW = min(rect.minW, contLayoutRect.maxW);
      rect.minH = min(rect.minH, contLayoutRect.maxH);
      rect.minW = max(rect.minW, contLayoutRect.startMinWidth);
      rect.minH = max(rect.minH, contLayoutRect.startMinHeight);
      if (contLayoutRect.autoResize && (rect.minW !== contLayoutRect.minW || rect.minH !== contLayoutRect.minH)) {
        rect.w = rect.minW;
        rect.h = rect.minH;
        container.layoutRect(rect);
        this.recalc(container);
        if (container._lastRect === null) {
          const parentCtrl = container.parent();
          if (parentCtrl) {
            parentCtrl._lastRect = null;
            parentCtrl.recalc();
          }
        }
        return;
      }
      ratio = availableSpace / totalFlex;
      for (i = 0, l = maxSizeItems.length; i < l; i++) {
        ctrl = maxSizeItems[i];
        ctrlLayoutRect = ctrl.layoutRect();
        maxSize = ctrlLayoutRect[maxSizeName];
        size = ctrlLayoutRect[minSizeName] + ctrlLayoutRect.flex * ratio;
        if (size > maxSize) {
          availableSpace -= ctrlLayoutRect[maxSizeName] - ctrlLayoutRect[minSizeName];
          totalFlex -= ctrlLayoutRect.flex;
          ctrlLayoutRect.flex = 0;
          ctrlLayoutRect.maxFlexSize = maxSize;
        } else {
          ctrlLayoutRect.maxFlexSize = 0;
        }
      }
      ratio = availableSpace / totalFlex;
      pos = contPaddingBox[beforeName];
      rect = {};
      if (totalFlex === 0) {
        if (pack === 'end') {
          pos = availableSpace + contPaddingBox[beforeName];
        } else if (pack === 'center') {
          pos = Math.round(contLayoutRect[innerSizeName] / 2 - (contLayoutRect[innerSizeName] - availableSpace) / 2) + contPaddingBox[beforeName];
          if (pos < 0) {
            pos = contPaddingBox[beforeName];
          }
        } else if (pack === 'justify') {
          pos = contPaddingBox[beforeName];
          spacing = Math.floor(availableSpace / (items.length - 1));
        }
      }
      rect[alignAxisName] = contPaddingBox[alignBeforeName];
      for (i = 0, l = items.length; i < l; i++) {
        ctrl = items[i];
        ctrlLayoutRect = ctrl.layoutRect();
        size = ctrlLayoutRect.maxFlexSize || ctrlLayoutRect[minSizeName];
        if (align === 'center') {
          rect[alignAxisName] = Math.round(contLayoutRect[alignInnerSizeName] / 2 - ctrlLayoutRect[alignSizeName] / 2);
        } else if (align === 'stretch') {
          rect[alignSizeName] = max(ctrlLayoutRect[alignMinSizeName] || 0, contLayoutRect[alignInnerSizeName] - contPaddingBox[alignBeforeName] - contPaddingBox[alignAfterName]);
          rect[alignAxisName] = contPaddingBox[alignBeforeName];
        } else if (align === 'end') {
          rect[alignAxisName] = contLayoutRect[alignInnerSizeName] - ctrlLayoutRect[alignSizeName] - contPaddingBox.top;
        }
        if (ctrlLayoutRect.flex > 0) {
          size += ctrlLayoutRect.flex * ratio;
        }
        rect[sizeName] = size;
        rect[posName] = pos;
        ctrl.layoutRect(rect);
        if (ctrl.recalc) {
          ctrl.recalc();
        }
        pos += size + spacing;
      }
    }
  });

  const FlowLayout = Layout.extend({
    Defaults: {
      containerClass: 'flow-layout',
      controlClass: 'flow-layout-item',
      endClass: 'break'
    },
    recalc(container) {
      container.items().filter(':visible').each(ctrl => {
        if (ctrl.recalc) {
          ctrl.recalc();
        }
      });
    },
    isNative() {
      return true;
    }
  });

  function ClosestOrAncestor (is, ancestor, scope, a, isRoot) {
    return is(scope, a) ? Option.some(scope) : $_dt75eqv0jfjm4nio.isFunction(isRoot) && isRoot(scope) ? Option.none() : ancestor(scope, a, isRoot);
  }

  const first$1 = predicate => descendant($_d4kplmuvjfjm4nhz.body(), predicate);
  const ancestor = (scope, predicate, isRoot) => {
    let element = scope.dom();
    const stop = $_dt75eqv0jfjm4nio.isFunction(isRoot) ? isRoot : Fun.constant(false);
    while (element.parentNode) {
      element = element.parentNode;
      const el = $_1zx2m1usjfjm4nhs.fromDom(element);
      if (predicate(el))
        return Option.some(el);
      else if (stop(el))
        break;
    }
    return Option.none();
  };
  const closest = (scope, predicate, isRoot) => {
    const is = scope => predicate(scope);
    return ClosestOrAncestor(is, ancestor, scope, predicate, isRoot);
  };
  const sibling = (scope, predicate) => {
    const element = scope.dom();
    if (!element.parentNode)
      return Option.none();
    return child$1($_1zx2m1usjfjm4nhs.fromDom(element.parentNode), x => !$_enmgxuv7jfjm4niz.eq(scope, x) && predicate(x));
  };
  var child$1 = (scope, predicate) => {
    const result = Arr.find(scope.dom().childNodes, Fun.compose(predicate, $_1zx2m1usjfjm4nhs.fromDom));
    return result.map($_1zx2m1usjfjm4nhs.fromDom);
  };
  var descendant = (scope, predicate) => {
    const descend = element => {
      for (let i = 0; i < element.childNodes.length; i++) {
        if (predicate($_1zx2m1usjfjm4nhs.fromDom(element.childNodes[i])))
          return Option.some($_1zx2m1usjfjm4nhs.fromDom(element.childNodes[i]));
        const res = descend(element.childNodes[i]);
        if (res.isSome())
          return res;
      }
      return Option.none();
    };
    return descend(scope.dom());
  };
  const $_fawrj3vtjfjm4nkn = {
    first: first$1,
    ancestor,
    closest,
    sibling,
    child: child$1,
    descendant
  };

  const first$2 = selector => $_6h5b4svnjfjm4njy.one(selector);
  const ancestor$1 = (scope, selector, isRoot) => $_fawrj3vtjfjm4nkn.ancestor(scope, e => $_6h5b4svnjfjm4njy.is(e, selector), isRoot);
  const sibling$1 = (scope, selector) => $_fawrj3vtjfjm4nkn.sibling(scope, e => $_6h5b4svnjfjm4njy.is(e, selector));
  const child$2 = (scope, selector) => $_fawrj3vtjfjm4nkn.child(scope, e => $_6h5b4svnjfjm4njy.is(e, selector));
  const descendant$1 = (scope, selector) => $_6h5b4svnjfjm4njy.one(selector, scope);
  const closest$1 = (scope, selector, isRoot) => ClosestOrAncestor($_6h5b4svnjfjm4njy.is, ancestor$1, scope, selector, isRoot);
  const $_29in5lvsjfjm4nkm = {
    first: first$2,
    ancestor: ancestor$1,
    sibling: sibling$1,
    child: child$2,
    descendant: descendant$1,
    closest: closest$1
  };

  const toggleFormat = (editor, fmt) => () => {
    editor.execCommand('mceToggleFormat', false, fmt);
  };
  const postRenderFormat = (editor, name) => function () {
    const self = this;
    if (editor.formatter) {
      editor.formatter.formatChanged(name, state => {
        self.active(state);
      });
    } else {
      editor.on('init', () => {
        editor.formatter.formatChanged(name, state => {
          self.active(state);
        });
      });
    }
  };
  const $_fb7c10vwjfjm4nkz = {
    toggleFormat,
    postRenderFormat
  };

  const register = editor => {
    editor.addMenuItem('align', {
      text: 'Align',
      menu: [
        {
          text: 'Left',
          icon: 'alignleft',
          onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, 'alignleft')
        },
        {
          text: 'Center',
          icon: 'aligncenter',
          onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, 'aligncenter')
        },
        {
          text: 'Right',
          icon: 'alignright',
          onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, 'alignright')
        },
        {
          text: 'Justify',
          icon: 'alignjustify',
          onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, 'alignjustify')
        }
      ]
    });
    Tools.each({
      alignleft: [
        'Align left',
        'JustifyLeft'
      ],
      aligncenter: [
        'Align center',
        'JustifyCenter'
      ],
      alignright: [
        'Align right',
        'JustifyRight'
      ],
      alignjustify: [
        'Justify',
        'JustifyFull'
      ],
      alignnone: [
        'No alignment',
        'JustifyNone'
      ]
    }, (item, name) => {
      editor.addButton(name, {
        active: false,
        tooltip: item[0],
        cmd: item[1],
        onPostRender: $_fb7c10vwjfjm4nkz.postRenderFormat(editor, name)
      });
    });
  };
  const $_2fcn33vvjfjm4nkx = { register };

  const getFirstFont = fontFamily => fontFamily ? fontFamily.split(',')[0] : '';
  const findMatchingValue = (items, fontFamily) => {
    let value;
    Tools.each(items, item => {
      if (item.value.toLowerCase() === fontFamily.toLowerCase()) {
        value = item.value;
      }
    });
    Tools.each(items, item => {
      if (!value && getFirstFont(item.value).toLowerCase() === getFirstFont(fontFamily).toLowerCase()) {
        value = item.value;
      }
    });
    return value;
  };
  const createFontNameListBoxChangeHandler = (editor, items) => function () {
    const self = this;
    editor.on('init nodeChange', e => {
      const fontFamily = editor.queryCommandValue('FontName');
      const match = findMatchingValue(items, fontFamily);
      self.value(match ? match : null);
      if (!match && fontFamily) {
        self.text(getFirstFont(fontFamily));
      }
    });
  };
  const createFormats = formats => {
    formats = formats.replace(/;$/, '').split(';');
    let i = formats.length;
    while (i--) {
      formats[i] = formats[i].split('=');
    }
    return formats;
  };
  const getFontItems = editor => {
    const defaultFontsFormats = 'Andale Mono=andale mono,monospace;' + 'Arial=arial,helvetica,sans-serif;' + 'Arial Black=arial black,sans-serif;' + 'Book Antiqua=book antiqua,palatino,serif;' + 'Comic Sans MS=comic sans ms,sans-serif;' + 'Courier New=courier new,courier,monospace;' + 'Georgia=georgia,palatino,serif;' + 'Helvetica=helvetica,arial,sans-serif;' + 'Impact=impact,sans-serif;' + 'Symbol=symbol;' + 'Tahoma=tahoma,arial,helvetica,sans-serif;' + 'Terminal=terminal,monaco,monospace;' + 'Times New Roman=times new roman,times,serif;' + 'Trebuchet MS=trebuchet ms,geneva,sans-serif;' + 'Verdana=verdana,geneva,sans-serif;' + 'Webdings=webdings;' + 'Wingdings=wingdings,zapf dingbats';
    const fonts = createFormats(editor.settings.font_formats || defaultFontsFormats);
    return Tools.map(fonts, font => ({
      text: { raw: font[0] },
      value: font[1],
      textStyle: font[1].indexOf('dings') === -1 ? 'font-family:' + font[1] : ''
    }));
  };
  const registerButtons = editor => {
    editor.addButton('fontselect', () => {
      const items = getFontItems(editor);
      return {
        type: 'listbox',
        text: 'Font Family',
        tooltip: 'Font Family',
        values: items,
        fixedWidth: true,
        onPostRender: createFontNameListBoxChangeHandler(editor, items),
        onselect(e) {
          if (e.control.settings.value) {
            editor.execCommand('FontName', false, e.control.settings.value);
          }
        }
      };
    });
  };
  const register$1 = editor => {
    registerButtons(editor);
  };
  const $_6nd7uuvxjfjm4nl1 = { register: register$1 };

  const round = (number, precision) => {
    const factor = 10 ** precision;
    return Math.round(number * factor) / factor;
  };
  const toPt = (fontSize, precision) => {
    if (/[0-9.]+px$/.test(fontSize)) {
      return round(parseInt(fontSize, 10) * 72 / 96, precision || 0) + 'pt';
    }
    return fontSize;
  };
  const findMatchingValue$1 = (items, pt, px) => {
    let value;
    Tools.each(items, item => {
      if (item.value === px) {
        value = px;
      } else if (item.value === pt) {
        value = pt;
      }
    });
    return value;
  };
  const createFontSizeListBoxChangeHandler = (editor, items) => function () {
    const self = this;
    editor.on('init nodeChange', e => {
      let px, pt, precision, match;
      px = editor.queryCommandValue('FontSize');
      if (px) {
        for (precision = 3; !match && precision >= 0; precision--) {
          pt = toPt(px, precision);
          match = findMatchingValue$1(items, pt, px);
        }
      }
      self.value(match ? match : null);
      if (!match) {
        self.text(pt);
      }
    });
  };
  const getFontSizeItems = editor => {
    const defaultFontsizeFormats = '8pt 10pt 12pt 14pt 18pt 24pt 36pt';
    const fontsizeFormats = editor.settings.fontsize_formats || defaultFontsizeFormats;
    return Tools.map(fontsizeFormats.split(' '), item => {
      let text = item, value = item;
      const values = item.split('=');
      if (values.length > 1) {
        text = values[0];
        value = values[1];
      }
      return {
        text,
        value
      };
    });
  };
  const registerButtons$1 = editor => {
    editor.addButton('fontsizeselect', () => {
      const items = getFontSizeItems(editor);
      return {
        type: 'listbox',
        text: 'Font Sizes',
        tooltip: 'Font Sizes',
        values: items,
        fixedWidth: true,
        onPostRender: createFontSizeListBoxChangeHandler(editor, items),
        onclick(e) {
          if (e.control.settings.value) {
            editor.execCommand('FontSize', false, e.control.settings.value);
          }
        }
      };
    });
  };
  const register$2 = editor => {
    registerButtons$1(editor);
  };
  const $_cja9mnvyjfjm4nl4 = { register: register$2 };

  const hideMenuObjects = (editor, menu) => {
    let count = menu.length;
    Tools.each(menu, item => {
      if (item.menu) {
        item.hidden = hideMenuObjects(editor, item.menu) === 0;
      }
      const formatName = item.format;
      if (formatName) {
        item.hidden = !editor.formatter.canApply(formatName);
      }
      if (item.hidden) {
        count--;
      }
    });
    return count;
  };
  const hideFormatMenuItems = (editor, menu) => {
    let count = menu.items().length;
    menu.items().each(item => {
      if (item.menu) {
        item.visible(hideFormatMenuItems(editor, item.menu) > 0);
      }
      if (!item.menu && item.settings.menu) {
        item.visible(hideMenuObjects(editor, item.settings.menu) > 0);
      }
      const formatName = item.settings.format;
      if (formatName) {
        item.visible(editor.formatter.canApply(formatName));
      }
      if (!item.visible()) {
        count--;
      }
    });
    return count;
  };
  const createFormatMenu = editor => {
    let count = 0;
    const newFormats = [];
    const defaultStyleFormats = [
      {
        title: 'Headings',
        items: [
          {
            title: 'Heading 1',
            format: 'h1'
          },
          {
            title: 'Heading 2',
            format: 'h2'
          },
          {
            title: 'Heading 3',
            format: 'h3'
          },
          {
            title: 'Heading 4',
            format: 'h4'
          },
          {
            title: 'Heading 5',
            format: 'h5'
          },
          {
            title: 'Heading 6',
            format: 'h6'
          }
        ]
      },
      {
        title: 'Inline',
        items: [
          {
            title: 'Bold',
            icon: 'bold',
            format: 'bold'
          },
          {
            title: 'Italic',
            icon: 'italic',
            format: 'italic'
          },
          {
            title: 'Underline',
            icon: 'underline',
            format: 'underline'
          },
          {
            title: 'Strikethrough',
            icon: 'strikethrough',
            format: 'strikethrough'
          },
          {
            title: 'Superscript',
            icon: 'superscript',
            format: 'superscript'
          },
          {
            title: 'Subscript',
            icon: 'subscript',
            format: 'subscript'
          },
          {
            title: 'Code',
            icon: 'code',
            format: 'code'
          }
        ]
      },
      {
        title: 'Blocks',
        items: [
          {
            title: 'Paragraph',
            format: 'p'
          },
          {
            title: 'Blockquote',
            format: 'blockquote'
          },
          {
            title: 'Div',
            format: 'div'
          },
          {
            title: 'Pre',
            format: 'pre'
          }
        ]
      },
      {
        title: 'Alignment',
        items: [
          {
            title: 'Left',
            icon: 'alignleft',
            format: 'alignleft'
          },
          {
            title: 'Center',
            icon: 'aligncenter',
            format: 'aligncenter'
          },
          {
            title: 'Right',
            icon: 'alignright',
            format: 'alignright'
          },
          {
            title: 'Justify',
            icon: 'alignjustify',
            format: 'alignjustify'
          }
        ]
      }
    ];
    const createMenu = formats => {
      const menu = [];
      if (!formats) {
        return;
      }
      Tools.each(formats, format => {
        const menuItem = {
          text: format.title,
          icon: format.icon
        };
        if (format.items) {
          menuItem.menu = createMenu(format.items);
        } else {
          const formatName = format.format || 'custom' + count++;
          if (!format.format) {
            format.name = formatName;
            newFormats.push(format);
          }
          menuItem.format = formatName;
          menuItem.cmd = format.cmd;
        }
        menu.push(menuItem);
      });
      return menu;
    };
    const createStylesMenu = () => {
      let menu;
      if (editor.settings.style_formats_merge) {
        if (editor.settings.style_formats) {
          menu = createMenu(defaultStyleFormats.concat(editor.settings.style_formats));
        } else {
          menu = createMenu(defaultStyleFormats);
        }
      } else {
        menu = createMenu(editor.settings.style_formats || defaultStyleFormats);
      }
      return menu;
    };
    editor.on('init', () => {
      Tools.each(newFormats, format => {
        editor.formatter.register(format.name, format);
      });
    });
    return {
      type: 'menu',
      items: createStylesMenu(),
      onPostRender(e) {
        editor.fire('renderFormatsMenu', { control: e.control });
      },
      itemDefaults: {
        preview: true,
        textStyle() {
          if (this.settings.format) {
            return editor.formatter.getCssText(this.settings.format);
          }
        },
        onPostRender() {
          const self = this;
          self.parent().on('show', () => {
            let formatName, command;
            formatName = self.settings.format;
            if (formatName) {
              self.disabled(!editor.formatter.canApply(formatName));
              self.active(editor.formatter.match(formatName));
            }
            command = self.settings.cmd;
            if (command) {
              self.active(editor.queryCommandState(command));
            }
          });
        },
        onclick() {
          if (this.settings.format) {
            $_fb7c10vwjfjm4nkz.toggleFormat(editor, this.settings.format)();
          }
          if (this.settings.cmd) {
            editor.execCommand(this.settings.cmd);
          }
        }
      }
    };
  };
  const registerMenuItems = (editor, formatMenu) => {
    editor.addMenuItem('formats', {
      text: 'Formats',
      menu: formatMenu
    });
  };
  const registerButtons$2 = (editor, formatMenu) => {
    editor.addButton('styleselect', {
      type: 'menubutton',
      text: 'Formats',
      menu: formatMenu,
      onShowMenu() {
        if (editor.settings.style_formats_autohide) {
          hideFormatMenuItems(editor, this.menu);
        }
      }
    });
  };
  const register$3 = editor => {
    const formatMenu = createFormatMenu(editor);
    registerMenuItems(editor, formatMenu);
    registerButtons$2(editor, formatMenu);
  };
  const $_25wxsxvzjfjm4nl7 = { register: register$3 };

  const defaultBlocks = 'Paragraph=p;' + 'Heading 1=h1;' + 'Heading 2=h2;' + 'Heading 3=h3;' + 'Heading 4=h4;' + 'Heading 5=h5;' + 'Heading 6=h6;' + 'Preformatted=pre';
  const createFormats$1 = formats => {
    formats = formats.replace(/;$/, '').split(';');
    let i = formats.length;
    while (i--) {
      formats[i] = formats[i].split('=');
    }
    return formats;
  };
  const createListBoxChangeHandler = (editor, items, formatName) => function () {
    const self = this;
    editor.on('nodeChange', e => {
      const formatter = editor.formatter;
      let value = null;
      Tools.each(e.parents, node => {
        Tools.each(items, item => {
          if (formatName) {
            if (formatter.matchNode(node, formatName, { value: item.value })) {
              value = item.value;
            }
          } else {
            if (formatter.matchNode(node, item.value)) {
              value = item.value;
            }
          }
          if (value) {
            return false;
          }
        });
        if (value) {
          return false;
        }
      });
      self.value(value);
    });
  };
  const lazyFormatSelectBoxItems = (editor, blocks) => () => {
    const items = [];
    Tools.each(blocks, block => {
      items.push({
        text: block[0],
        value: block[1],
        textStyle() {
          return editor.formatter.getCssText(block[1]);
        }
      });
    });
    return {
      type: 'listbox',
      text: blocks[0][0],
      values: items,
      fixedWidth: true,
      onselect(e) {
        if (e.control) {
          const fmt = e.control.value();
          $_fb7c10vwjfjm4nkz.toggleFormat(editor, fmt)();
        }
      },
      onPostRender: createListBoxChangeHandler(editor, items)
    };
  };
  const buildMenuItems = (editor, blocks) => Tools.map(blocks, block => ({
    text: block[0],
    onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, block[1]),

    textStyle() {
      return editor.formatter.getCssText(block[1]);
    }
  }));
  const register$4 = editor => {
    const blocks = createFormats$1(editor.settings.block_formats || defaultBlocks);
    editor.addMenuItem('blockformats', {
      text: 'Blocks',
      menu: buildMenuItems(editor, blocks)
    });
    editor.addButton('formatselect', lazyFormatSelectBoxItems(editor, blocks));
  };
  const $_9upbynw0jfjm4nlc = { register: register$4 };

  const createCustomMenuItems = (editor, names) => {
    let items, nameList;
    if (typeof names === 'string') {
      nameList = names.split(' ');
    } else if (Tools.isArray(names)) {
      return Arr.flatten(Tools.map(names, names => createCustomMenuItems(editor, names)));
    }
    items = Tools.grep(nameList, name => name === '|' || name in editor.menuItems);
    return Tools.map(items, name => name === '|' ? { text: '-' } : editor.menuItems[name]);
  };
  const isSeparator$1 = menuItem => menuItem && menuItem.text === '-';
  const trimMenuItems = menuItems => {
    const menuItems2 = Arr.filter(menuItems, (menuItem, i, menuItems) => !isSeparator$1(menuItem) || !isSeparator$1(menuItems[i - 1]));
    return Arr.filter(menuItems2, (menuItem, i, menuItems) => !isSeparator$1(menuItem) || i > 0 && i < menuItems.length - 1);
  };
  const createContextMenuItems = (editor, context) => {
    const outputMenuItems = [{ text: '-' }];
    const menuItems = Tools.grep(editor.menuItems, menuItem => menuItem.context === context);
    Tools.each(menuItems, menuItem => {
      if (menuItem.separator === 'before') {
        outputMenuItems.push({ text: '|' });
      }
      if (menuItem.prependToContext) {
        outputMenuItems.unshift(menuItem);
      } else {
        outputMenuItems.push(menuItem);
      }
      if (menuItem.separator === 'after') {
        outputMenuItems.push({ text: '|' });
      }
    });
    return outputMenuItems;
  };
  const createInsertMenu = editor => {
    const insertButtonItems = editor.settings.insert_button_items;
    if (insertButtonItems) {
      return trimMenuItems(createCustomMenuItems(editor, insertButtonItems));
    } else {
      return trimMenuItems(createContextMenuItems(editor, 'insert'));
    }
  };
  const registerButtons$3 = editor => {
    editor.addButton('insert', {
      type: 'menubutton',
      icon: 'insert',
      menu: [],
      oncreatemenu() {
        this.menu.add(createInsertMenu(editor));
        this.menu.renderNew();
      }
    });
  };
  const register$5 = editor => {
    registerButtons$3(editor);
  };
  const $_2txcu6w1jfjm4nle = { register: register$5 };

  const registerFormatButtons = editor => {
    Tools.each({
      bold: 'Bold',
      italic: 'Italic',
      underline: 'Underline',
      strikethrough: 'Strikethrough',
      subscript: 'Subscript',
      superscript: 'Superscript'
    }, (text, name) => {
      editor.addButton(name, {
        active: false,
        tooltip: text,
        onPostRender: $_fb7c10vwjfjm4nkz.postRenderFormat(editor, name),
        onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, name)
      });
    });
  };
  const registerCommandButtons = editor => {
    Tools.each({
      outdent: [
        'Decrease indent',
        'Outdent'
      ],
      indent: [
        'Increase indent',
        'Indent'
      ],
      cut: [
        'Cut',
        'Cut'
      ],
      copy: [
        'Copy',
        'Copy'
      ],
      paste: [
        'Paste',
        'Paste'
      ],
      help: [
        'Help',
        'mceHelp'
      ],
      selectall: [
        'Select all',
        'SelectAll'
      ],
      visualaid: [
        'Visual aids',
        'mceToggleVisualAid'
      ],
      newdocument: [
        'New document',
        'mceNewDocument'
      ],
      removeformat: [
        'Clear formatting',
        'RemoveFormat'
      ],
      remove: [
        'Remove',
        'Delete'
      ]
    }, (item, name) => {
      editor.addButton(name, {
        tooltip: item[0],
        cmd: item[1]
      });
    });
  };
  const registerCommandToggleButtons = editor => {
    Tools.each({
      blockquote: [
        'Blockquote',
        'mceBlockQuote'
      ],
      subscript: [
        'Subscript',
        'Subscript'
      ],
      superscript: [
        'Superscript',
        'Superscript'
      ]
    }, (item, name) => {
      editor.addButton(name, {
        active: false,
        tooltip: item[0],
        cmd: item[1],
        onPostRender: $_fb7c10vwjfjm4nkz.postRenderFormat(editor, name)
      });
    });
  };
  const registerButtons$4 = editor => {
    registerFormatButtons(editor);
    registerCommandButtons(editor);
    registerCommandToggleButtons(editor);
  };
  const registerMenuItems$1 = editor => {
    Tools.each({
      bold: [
        'Bold',
        'Bold',
        'Meta+B'
      ],
      italic: [
        'Italic',
        'Italic',
        'Meta+I'
      ],
      underline: [
        'Underline',
        'Underline',
        'Meta+U'
      ],
      strikethrough: [
        'Strikethrough',
        'Strikethrough'
      ],
      subscript: [
        'Subscript',
        'Subscript'
      ],
      superscript: [
        'Superscript',
        'Superscript'
      ],
      removeformat: [
        'Clear formatting',
        'RemoveFormat'
      ],
      newdocument: [
        'New document',
        'mceNewDocument'
      ],
      cut: [
        'Cut',
        'Cut',
        'Meta+X'
      ],
      copy: [
        'Copy',
        'Copy',
        'Meta+C'
      ],
      paste: [
        'Paste',
        'Paste',
        'Meta+V'
      ],
      selectall: [
        'Select all',
        'SelectAll',
        'Meta+A'
      ]
    }, (item, name) => {
      editor.addMenuItem(name, {
        text: item[0],
        icon: name,
        shortcut: item[2],
        cmd: item[1]
      });
    });
    editor.addMenuItem('codeformat', {
      text: 'Code',
      icon: 'code',
      onclick: $_fb7c10vwjfjm4nkz.toggleFormat(editor, 'code')
    });
  };
  const register$6 = editor => {
    registerButtons$4(editor);
    registerMenuItems$1(editor);
  };
  const $_cnxrkrw2jfjm4nlh = { register: register$6 };

  const toggleUndoRedoState = (editor, type) => function () {
    const self = this;
    const checkState = () => {
      const typeFn = type === 'redo' ? 'hasRedo' : 'hasUndo';
      return editor.undoManager ? editor.undoManager[typeFn]() : false;
    };
    self.disabled(!checkState());
    editor.on('Undo Redo AddUndo TypingUndo ClearUndos SwitchMode', () => {
      self.disabled(editor.readonly || !checkState());
    });
  };
  const registerMenuItems$2 = editor => {
    editor.addMenuItem('undo', {
      text: 'Undo',
      icon: 'undo',
      shortcut: 'Meta+Z',
      onPostRender: toggleUndoRedoState(editor, 'undo'),
      cmd: 'undo'
    });
    editor.addMenuItem('redo', {
      text: 'Redo',
      icon: 'redo',
      shortcut: 'Meta+Y',
      onPostRender: toggleUndoRedoState(editor, 'redo'),
      cmd: 'redo'
    });
  };
  const registerButtons$5 = editor => {
    editor.addButton('undo', {
      tooltip: 'Undo',
      onPostRender: toggleUndoRedoState(editor, 'undo'),
      cmd: 'undo'
    });
    editor.addButton('redo', {
      tooltip: 'Redo',
      onPostRender: toggleUndoRedoState(editor, 'redo'),
      cmd: 'redo'
    });
  };
  const register$7 = editor => {
    registerMenuItems$2(editor);
    registerButtons$5(editor);
  };
  const $_fltznyw3jfjm4nlk = { register: register$7 };

  const toggleVisualAidState = editor => function () {
    const self = this;
    editor.on('VisualAid', e => {
      self.active(e.hasVisual);
    });
    self.active(editor.hasVisual);
  };
  const registerMenuItems$3 = editor => {
    editor.addMenuItem('visualaid', {
      text: 'Visual aids',
      selectable: true,
      onPostRender: toggleVisualAidState(editor),
      cmd: 'mceToggleVisualAid'
    });
  };
  const register$8 = editor => {
    registerMenuItems$3(editor);
  };
  const $_8hpkppw4jfjm4nll = { register: register$8 };

  const setupEnvironment = () => {
    Widget.tooltips = !global$8.iOS;
    Control$1.translate = text => EditorManager.translate(text);
  };
  const setupUiContainer = editor => {
    if (editor.settings.ui_container) {
      global$8.container = $_29in5lvsjfjm4nkm.descendant($_1zx2m1usjfjm4nhs.fromDom(document.body), editor.settings.ui_container).fold(Fun.constant(null), elm => elm.dom());
    }
  };
  const setupRtlMode = editor => {
    if (editor.rtl) {
      Control$1.rtl = true;
    }
  };
  const setupHideFloatPanels = editor => {
    editor.on('mousedown', () => {
      FloatPanel.hideAll();
    });
  };
  const setup$1 = editor => {
    setupRtlMode(editor);
    setupHideFloatPanels(editor);
    setupUiContainer(editor);
    setupEnvironment();
    $_9upbynw0jfjm4nlc.register(editor);
    $_2fcn33vvjfjm4nkx.register(editor);
    $_cnxrkrw2jfjm4nlh.register(editor);
    $_fltznyw3jfjm4nlk.register(editor);
    $_cja9mnvyjfjm4nl4.register(editor);
    $_6nd7uuvxjfjm4nl1.register(editor);
    $_25wxsxvzjfjm4nl7.register(editor);
    $_8hpkppw4jfjm4nll.register(editor);
    $_2txcu6w1jfjm4nle.register(editor);
  };
  const $_bce66dvrjfjm4nki = { setup: setup$1 };

  const GridLayout = AbsoluteLayout.extend({
    recalc(container) {
      let settings, rows, cols, items, contLayoutRect, width, height, rect, ctrlLayoutRect, ctrl, x, y, posX, posY, ctrlSettings, contPaddingBox, align, spacingH, spacingV, alignH, alignV, maxX, maxY;
      const colWidths = [];
      const rowHeights = [];
      let ctrlMinWidth, ctrlMinHeight, availableWidth, availableHeight, reverseRows, idx;
      settings = container.settings;
      items = container.items().filter(':visible');
      contLayoutRect = container.layoutRect();
      cols = settings.columns || Math.ceil(Math.sqrt(items.length));
      rows = Math.ceil(items.length / cols);
      spacingH = settings.spacingH || settings.spacing || 0;
      spacingV = settings.spacingV || settings.spacing || 0;
      alignH = settings.alignH || settings.align;
      alignV = settings.alignV || settings.align;
      contPaddingBox = container.paddingBox;
      reverseRows = 'reverseRows' in settings ? settings.reverseRows : container.isRtl();
      if (alignH && typeof alignH === 'string') {
        alignH = [alignH];
      }
      if (alignV && typeof alignV === 'string') {
        alignV = [alignV];
      }
      for (x = 0; x < cols; x++) {
        colWidths.push(0);
      }
      for (y = 0; y < rows; y++) {
        rowHeights.push(0);
      }
      for (y = 0; y < rows; y++) {
        for (x = 0; x < cols; x++) {
          ctrl = items[y * cols + x];
          if (!ctrl) {
            break;
          }
          ctrlLayoutRect = ctrl.layoutRect();
          ctrlMinWidth = ctrlLayoutRect.minW;
          ctrlMinHeight = ctrlLayoutRect.minH;
          colWidths[x] = ctrlMinWidth > colWidths[x] ? ctrlMinWidth : colWidths[x];
          rowHeights[y] = ctrlMinHeight > rowHeights[y] ? ctrlMinHeight : rowHeights[y];
        }
      }
      availableWidth = contLayoutRect.innerW - contPaddingBox.left - contPaddingBox.right;
      for (maxX = 0, x = 0; x < cols; x++) {
        maxX += colWidths[x] + (x > 0 ? spacingH : 0);
        availableWidth -= (x > 0 ? spacingH : 0) + colWidths[x];
      }
      availableHeight = contLayoutRect.innerH - contPaddingBox.top - contPaddingBox.bottom;
      for (maxY = 0, y = 0; y < rows; y++) {
        maxY += rowHeights[y] + (y > 0 ? spacingV : 0);
        availableHeight -= (y > 0 ? spacingV : 0) + rowHeights[y];
      }
      maxX += contPaddingBox.left + contPaddingBox.right;
      maxY += contPaddingBox.top + contPaddingBox.bottom;
      rect = {};
      rect.minW = maxX + (contLayoutRect.w - contLayoutRect.innerW);
      rect.minH = maxY + (contLayoutRect.h - contLayoutRect.innerH);
      rect.contentW = rect.minW - contLayoutRect.deltaW;
      rect.contentH = rect.minH - contLayoutRect.deltaH;
      rect.minW = Math.min(rect.minW, contLayoutRect.maxW);
      rect.minH = Math.min(rect.minH, contLayoutRect.maxH);
      rect.minW = Math.max(rect.minW, contLayoutRect.startMinWidth);
      rect.minH = Math.max(rect.minH, contLayoutRect.startMinHeight);
      if (contLayoutRect.autoResize && (rect.minW !== contLayoutRect.minW || rect.minH !== contLayoutRect.minH)) {
        rect.w = rect.minW;
        rect.h = rect.minH;
        container.layoutRect(rect);
        this.recalc(container);
        if (container._lastRect === null) {
          const parentCtrl = container.parent();
          if (parentCtrl) {
            parentCtrl._lastRect = null;
            parentCtrl.recalc();
          }
        }
        return;
      }
      if (contLayoutRect.autoResize) {
        rect = container.layoutRect(rect);
        rect.contentW = rect.minW - contLayoutRect.deltaW;
        rect.contentH = rect.minH - contLayoutRect.deltaH;
      }
      let flexV;
      if (settings.packV === 'start') {
        flexV = 0;
      } else {
        flexV = availableHeight > 0 ? Math.floor(availableHeight / rows) : 0;
      }
      let totalFlex = 0;
      const flexWidths = settings.flexWidths;
      if (flexWidths) {
        for (x = 0; x < flexWidths.length; x++) {
          totalFlex += flexWidths[x];
        }
      } else {
        totalFlex = cols;
      }
      const ratio = availableWidth / totalFlex;
      for (x = 0; x < cols; x++) {
        colWidths[x] += flexWidths ? flexWidths[x] * ratio : ratio;
      }
      posY = contPaddingBox.top;
      for (y = 0; y < rows; y++) {
        posX = contPaddingBox.left;
        height = rowHeights[y] + flexV;
        for (x = 0; x < cols; x++) {
          if (reverseRows) {
            idx = y * cols + cols - 1 - x;
          } else {
            idx = y * cols + x;
          }
          ctrl = items[idx];
          if (!ctrl) {
            break;
          }
          ctrlSettings = ctrl.settings;
          ctrlLayoutRect = ctrl.layoutRect();
          width = Math.max(colWidths[x], ctrlLayoutRect.startMinWidth);
          ctrlLayoutRect.x = posX;
          ctrlLayoutRect.y = posY;
          align = ctrlSettings.alignH || (alignH ? alignH[x] || alignH[0] : null);
          if (align === 'center') {
            ctrlLayoutRect.x = posX + width / 2 - ctrlLayoutRect.w / 2;
          } else if (align === 'right') {
            ctrlLayoutRect.x = posX + width - ctrlLayoutRect.w;
          } else if (align === 'stretch') {
            ctrlLayoutRect.w = width;
          }
          align = ctrlSettings.alignV || (alignV ? alignV[x] || alignV[0] : null);
          if (align === 'center') {
            ctrlLayoutRect.y = posY + height / 2 - ctrlLayoutRect.h / 2;
          } else if (align === 'bottom') {
            ctrlLayoutRect.y = posY + height - ctrlLayoutRect.h;
          } else if (align === 'stretch') {
            ctrlLayoutRect.h = height;
          }
          ctrl.layoutRect(ctrlLayoutRect);
          posX += width + spacingH;
          if (ctrl.recalc) {
            ctrl.recalc();
          }
        }
        posY += height + spacingV;
      }
    }
  });

  const Iframe$1 = Widget.extend({
    renderHtml() {
      const self = this;
      self.classes.add('iframe');
      self.canFocus = false;
      return '<iframe id="' + self._id + '" class="' + self.classes + '" tabindex="-1" src="' + (self.settings.url || 'javascript:\'\'') + '" frameborder="0"></iframe>';
    },
    src(src) {
      this.getEl().src = src;
    },
    html(html, callback) {
      const self = this, body = this.getEl().contentWindow.document.body;
      if (!body) {
        Delay.setTimeout(() => {
          self.html(html);
        });
      } else {
        body.innerHTML = html;
        if (callback) {
          callback();
        }
      }
      return this;
    }
  });

  const InfoBox = Widget.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      self.classes.add('widget').add('infobox');
      self.canFocus = false;
    },
    severity(level) {
      this.classes.remove('error');
      this.classes.remove('warning');
      this.classes.remove('success');
      this.classes.add(level);
    },
    help(state) {
      this.state.set('help', state);
    },
    renderHtml() {
      const self = this, prefix = self.classPrefix;
      return '<div id="' + self._id + '" class="' + self.classes + '">' + '<div id="' + self._id + '-body">' + self.encode(self.state.get('text')) + '<button role="button" tabindex="-1">' + '<i class="' + prefix + 'ico ' + prefix + 'i-help"></i>' + '</button>' + '</div>' + '</div>';
    },
    bindStates() {
      const self = this;
      self.state.on('change:text', e => {
        self.getEl('body').firstChild.data = self.encode(e.value);
        if (self.state.get('rendered')) {
          self.updateLayoutRect();
        }
      });
      self.state.on('change:help', e => {
        self.classes.toggle('has-help', e.value);
        if (self.state.get('rendered')) {
          self.updateLayoutRect();
        }
      });
      return self._super();
    }
  });

  const Label = Widget.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      self.classes.add('widget').add('label');
      self.canFocus = false;
      if (settings.multiline) {
        self.classes.add('autoscroll');
      }
      if (settings.strong) {
        self.classes.add('strong');
      }
    },
    initLayoutRect() {
      const self = this, layoutRect = self._super();
      if (self.settings.multiline) {
        const size = funcs.getSize(self.getEl());
        if (size.width > layoutRect.maxW) {
          layoutRect.minW = layoutRect.maxW;
          self.classes.add('multiline');
        }
        self.getEl().style.width = layoutRect.minW + 'px';
        layoutRect.startMinH = layoutRect.h = layoutRect.minH = Math.min(layoutRect.maxH, funcs.getSize(self.getEl()).height);
      }
      return layoutRect;
    },
    repaint() {
      const self = this;
      if (!self.settings.multiline) {
        self.getEl().style.lineHeight = self.layoutRect().h + 'px';
      }
      return self._super();
    },
    severity(level) {
      this.classes.remove('error');
      this.classes.remove('warning');
      this.classes.remove('success');
      this.classes.add(level);
    },
    renderHtml() {
      const self = this;
      let targetCtrl, forName, forId = self.settings.forId;
      const text = self.settings.html ? self.settings.html : self.encode(self.state.get('text'));
      if (!forId && (forName = self.settings.forName)) {
        targetCtrl = self.getRoot().find('#' + forName)[0];
        if (targetCtrl) {
          forId = targetCtrl._id;
        }
      }
      if (forId) {
        return '<label id="' + self._id + '" class="' + self.classes + '"' + (forId ? ' for="' + forId + '"' : '') + '>' + text + '</label>';
      }
      return '<span id="' + self._id + '" class="' + self.classes + '">' + text + '</span>';
    },
    bindStates() {
      const self = this;
      self.state.on('change:text', e => {
        self.innerHtml(self.encode(e.value));
        if (self.state.get('rendered')) {
          self.updateLayoutRect();
        }
      });
      return self._super();
    }
  });

  const Toolbar$1 = Container.extend({
    Defaults: {
      role: 'toolbar',
      layout: 'flow'
    },
    init(settings) {
      const self = this;
      self._super(settings);
      self.classes.add('toolbar');
    },
    postRender() {
      const self = this;
      self.items().each(ctrl => {
        ctrl.classes.add('toolbar-item');
      });
      return self._super();
    }
  });

  const MenuBar = Toolbar$1.extend({
    Defaults: {
      role: 'menubar',
      containerCls: 'menubar',
      ariaRoot: true,
      defaults: { type: 'menubutton' }
    }
  });

  function isChildOf$1(node, parent) {
    while (node) {
      if (parent === node) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  }
  const MenuButton = Button.extend({
    init(settings) {
      const self = this;
      self._renderOpen = true;
      self._super(settings);
      settings = self.settings;
      self.classes.add('menubtn');
      if (settings.fixedWidth) {
        self.classes.add('fixed-width');
      }
      self.aria('haspopup', true);
      self.state.set('menu', settings.menu || self.render());
    },
    showMenu(toggle) {
      const self = this;
      let menu;
      if (self.menu && self.menu.visible() && toggle !== false) {
        return self.hideMenu();
      }
      if (!self.menu) {
        menu = self.state.get('menu') || [];
        self.classes.add('opened');
        if (menu.length) {
          menu = {
            type: 'menu',
            animate: true,
            items: menu
          };
        } else {
          menu.type = menu.type || 'menu';
          menu.animate = true;
        }
        if (!menu.renderTo) {
          self.menu = Factory.create(menu).parent(self).renderTo();
        } else {
          self.menu = menu.parent(self).show().renderTo();
        }
        self.fire('createmenu');
        self.menu.reflow();
        self.menu.on('cancel', e => {
          if (e.control.parent() === self.menu) {
            e.stopPropagation();
            self.focus();
            self.hideMenu();
          }
        });
        self.menu.on('select', () => {
          self.focus();
        });
        self.menu.on('show hide', e => {
          if (e.control === self.menu) {
            self.activeMenu(e.type === 'show');
            self.classes.toggle('opened', e.type === 'show');
          }
          self.aria('expanded', e.type === 'show');
        }).fire('show');
      }
      self.menu.show();
      self.menu.layoutRect({ w: self.layoutRect().w });
      self.menu.repaint();
      self.menu.moveRel(self.getEl(), self.isRtl() ? [
        'br-tr',
        'tr-br'
      ] : [
        'bl-tl',
        'tl-bl'
      ]);
      self.fire('showmenu');
    },
    hideMenu() {
      const self = this;
      if (self.menu) {
        self.menu.items().each(item => {
          if (item.hideMenu) {
            item.hideMenu();
          }
        });
        self.menu.hide();
      }
    },
    activeMenu(state) {
      this.classes.toggle('active', state);
    },
    renderHtml() {
      const self = this, id = self._id, prefix = self.classPrefix;
      let icon = self.settings.icon, image;
      const text = self.state.get('text');
      let textHtml = '';
      image = self.settings.image;
      if (image) {
        icon = 'none';
        if (typeof image !== 'string') {
          image = window.getSelection ? image[0] : image[1];
        }
        image = ' style="background-image: url(\'' + image + '\')"';
      } else {
        image = '';
      }
      if (text) {
        self.classes.add('btn-has-text');
        textHtml = '<span class="' + prefix + 'txt">' + self.encode(text) + '</span>';
      }
      icon = self.settings.icon ? prefix + 'ico ' + prefix + 'i-' + icon : '';
      self.aria('role', self.parent() instanceof MenuBar ? 'menuitem' : 'button');
      return '<div id="' + id + '" class="' + self.classes + '" tabindex="-1" aria-labelledby="' + id + '">' + '<button id="' + id + '-open" role="presentation" type="button" tabindex="-1">' + (icon ? '<i class="' + icon + '"' + image + '></i>' : '') + textHtml + ' <i class="' + prefix + 'caret"></i>' + '</button>' + '</div>';
    },
    postRender() {
      const self = this;
      self.on('click', e => {
        if (e.control === self && isChildOf$1(e.target, self.getEl())) {
          self.focus();
          self.showMenu(!e.aria);
          if (e.aria) {
            self.menu.items().filter(':visible')[0].focus();
          }
        }
      });
      self.on('mouseenter', e => {
        const overCtrl = e.control;
        const parent = self.parent();
        let hasVisibleSiblingMenu;
        if (overCtrl && parent && overCtrl instanceof MenuButton && overCtrl.parent() === parent) {
          parent.items().filter('MenuButton').each(ctrl => {
            if (ctrl.hideMenu && ctrl !== overCtrl) {
              if (ctrl.menu && ctrl.menu.visible()) {
                hasVisibleSiblingMenu = true;
              }
              ctrl.hideMenu();
            }
          });
          if (hasVisibleSiblingMenu) {
            overCtrl.focus();
            overCtrl.showMenu();
          }
        }
      });
      return self._super();
    },
    bindStates() {
      const self = this;
      self.state.on('change:menu', () => {
        if (self.menu) {
          self.menu.remove();
        }
        self.menu = null;
      });
      return self._super();
    },
    remove() {
      this._super();
      if (this.menu) {
        this.menu.remove();
      }
    }
  });

  const Menu = FloatPanel.extend({
    Defaults: {
      defaultType: 'menuitem',
      border: 1,
      layout: 'stack',
      role: 'application',
      bodyRole: 'menu',
      ariaRoot: true
    },
    init(settings) {
      const self = this;
      settings.autohide = true;
      settings.constrainToViewport = true;
      if (typeof settings.items === 'function') {
        settings.itemsFactory = settings.items;
        settings.items = [];
      }
      if (settings.itemDefaults) {
        const items = settings.items;
        let i = items.length;
        while (i--) {
          items[i] = Tools.extend({}, settings.itemDefaults, items[i]);
        }
      }
      self._super(settings);
      self.classes.add('menu');
      if (settings.animate && global$8.ie !== 11) {
        self.classes.add('animate');
      }
    },
    repaint() {
      this.classes.toggle('menu-align', true);
      this._super();
      this.getEl().style.height = '';
      this.getEl('body').style.height = '';
      return this;
    },
    cancel() {
      const self = this;
      self.hideAll();
      self.fire('select');
    },
    load() {
      const self = this;
      let time, factory;
      function hideThrobber() {
        if (self.throbber) {
          self.throbber.hide();
          self.throbber = null;
        }
      }
      factory = self.settings.itemsFactory;
      if (!factory) {
        return;
      }
      if (!self.throbber) {
        self.throbber = new Throbber(self.getEl('body'), true);
        if (self.items().length === 0) {
          self.throbber.show();
          self.fire('loading');
        } else {
          self.throbber.show(100, () => {
            self.items().remove();
            self.fire('loading');
          });
        }
        self.on('hide close', hideThrobber);
      }
      self.requestTime = time = new Date().getTime();
      self.settings.itemsFactory(items => {
        if (items.length === 0) {
          self.hide();
          return;
        }
        if (self.requestTime !== time) {
          return;
        }
        self.getEl().style.width = '';
        self.getEl('body').style.width = '';
        hideThrobber();
        self.items().remove();
        self.getEl('body').innerHTML = '';
        self.add(items);
        self.renderNew();
        self.fire('loaded');
      });
    },
    hideAll() {
      const self = this;
      this.find('menuitem').exec('hideMenu');
      return self._super();
    },
    preRender() {
      const self = this;
      self.items().each(ctrl => {
        const settings = ctrl.settings;
        if (settings.icon || settings.image || settings.selectable) {
          self._hasIcons = true;
          return false;
        }
      });
      if (self.settings.itemsFactory) {
        self.on('postrender', () => {
          if (self.settings.itemsFactory) {
            self.load();
          }
        });
      }
      self.on('show hide', e => {
        if (e.control === self) {
          if (e.type === 'show') {
            Delay.setTimeout(() => {
              self.classes.add('in');
            }, 0);
          } else {
            self.classes.remove('in');
          }
        }
      });
      return self._super();
    }
  });

  const ListBox = MenuButton.extend({
    init(settings) {
      const self = this;
      let values, selected, selectedText, lastItemCtrl;
      function setSelected(menuValues) {
        for (let i = 0; i < menuValues.length; i++) {
          selected = menuValues[i].selected || settings.value === menuValues[i].value;
          if (selected) {
            selectedText = selectedText || menuValues[i].text;
            self.state.set('value', menuValues[i].value);
            return true;
          }
          if (menuValues[i].menu) {
            if (setSelected(menuValues[i].menu)) {
              return true;
            }
          }
        }
      }
      self._super(settings);
      settings = self.settings;
      self._values = values = settings.values;
      if (values) {
        if (typeof settings.value !== 'undefined') {
          setSelected(values);
        }
        if (!selected && values.length > 0) {
          selectedText = values[0].text;
          self.state.set('value', values[0].value);
        }
        self.state.set('menu', values);
      }
      self.state.set('text', settings.text || selectedText);
      self.classes.add('listbox');
      self.on('select', e => {
        const ctrl = e.control;
        if (lastItemCtrl) {
          e.lastControl = lastItemCtrl;
        }
        if (settings.multiple) {
          ctrl.active(!ctrl.active());
        } else {
          self.value(e.control.value());
        }
        lastItemCtrl = ctrl;
      });
    },
    value(value) {
      if (arguments.length === 0) {
        return this.state.get('value');
      }
      if (typeof value === 'undefined') {
        return this;
      }
      if (this.settings.values) {
        const matchingValues = Tools.grep(this.settings.values, a => a.value === value);
        if (matchingValues.length > 0) {
          this.state.set('value', value);
        }
      } else {
        this.state.set('value', value);
      }
      return this;
    },
    bindStates() {
      const self = this;
      function activateMenuItemsByValue(menu, value) {
        if (menu instanceof Menu) {
          menu.items().each(ctrl => {
            if (!ctrl.hasMenus()) {
              ctrl.active(ctrl.value() === value);
            }
          });
        }
      }
      function getSelectedItem(menuValues, value) {
        let selectedItem;
        if (!menuValues) {
          return;
        }
        for (let i = 0; i < menuValues.length; i++) {
          if (menuValues[i].value === value) {
            return menuValues[i];
          }
          if (menuValues[i].menu) {
            selectedItem = getSelectedItem(menuValues[i].menu, value);
            if (selectedItem) {
              return selectedItem;
            }
          }
        }
      }
      self.on('show', e => {
        activateMenuItemsByValue(e.control, self.value());
      });
      self.state.on('change:value', e => {
        const selectedItem = getSelectedItem(self.state.get('menu'), e.value);
        if (selectedItem) {
          self.text(selectedItem.text);
        } else {
          self.text(self.settings.text);
        }
      });
      return self._super();
    }
  });

  const toggleTextStyle = (ctrl, state) => {
    const textStyle = ctrl._textStyle;
    if (textStyle) {
      const textElm = ctrl.getEl('text');
      textElm.setAttribute('style', textStyle);
      if (state) {
        textElm.style.color = '';
        textElm.style.backgroundColor = '';
      }
    }
  };
  const MenuItem = Widget.extend({
    Defaults: {
      border: 0,
      role: 'menuitem'
    },
    init(settings) {
      const self = this;
      let text;
      self._super(settings);
      settings = self.settings;
      self.classes.add('menu-item');
      if (settings.menu) {
        self.classes.add('menu-item-expand');
      }
      if (settings.preview) {
        self.classes.add('menu-item-preview');
      }
      text = self.state.get('text');
      if (text === '-' || text === '|') {
        self.classes.add('menu-item-sep');
        self.aria('role', 'separator');
        self.state.set('text', '-');
      }
      if (settings.selectable) {
        self.aria('role', 'menuitemcheckbox');
        self.classes.add('menu-item-checkbox');
        settings.icon = 'selected';
      }
      if (!settings.preview && !settings.selectable) {
        self.classes.add('menu-item-normal');
      }
      self.on('mousedown', e => {
        e.preventDefault();
      });
      if (settings.menu && !settings.ariaHideMenu) {
        self.aria('haspopup', true);
      }
    },
    hasMenus() {
      return !!this.settings.menu;
    },
    showMenu() {
      const self = this;
      const settings = self.settings;
      let menu;
      const parent = self.parent();
      parent.items().each(ctrl => {
        if (ctrl !== self) {
          ctrl.hideMenu();
        }
      });
      if (settings.menu) {
        menu = self.menu;
        if (!menu) {
          menu = settings.menu;
          if (menu.length) {
            menu = {
              type: 'menu',
              items: menu
            };
          } else {
            menu.type = menu.type || 'menu';
          }
          if (parent.settings.itemDefaults) {
            menu.itemDefaults = parent.settings.itemDefaults;
          }
          menu = self.menu = Factory.create(menu).parent(self).renderTo();
          menu.reflow();
          menu.on('cancel', e => {
            e.stopPropagation();
            self.focus();
            menu.hide();
          });
          menu.on('show hide', e => {
            if (e.control.items) {
              e.control.items().each(ctrl => {
                ctrl.active(ctrl.settings.selected);
              });
            }
          }).fire('show');
          menu.on('hide', e => {
            if (e.control === menu) {
              self.classes.remove('selected');
            }
          });
          menu.submenu = true;
        } else {
          menu.show();
        }
        menu._parentMenu = parent;
        menu.classes.add('menu-sub');
        let rel = menu.testMoveRel(self.getEl(), self.isRtl() ? [
          'tl-tr',
          'bl-br',
          'tr-tl',
          'br-bl'
        ] : [
          'tr-tl',
          'br-bl',
          'tl-tr',
          'bl-br'
        ]);
        menu.moveRel(self.getEl(), rel);
        menu.rel = rel;
        rel = 'menu-sub-' + rel;
        menu.classes.remove(menu._lastRel).add(rel);
        menu._lastRel = rel;
        self.classes.add('selected');
        self.aria('expanded', true);
      }
    },
    hideMenu() {
      const self = this;
      if (self.menu) {
        self.menu.items().each(item => {
          if (item.hideMenu) {
            item.hideMenu();
          }
        });
        self.menu.hide();
        self.aria('expanded', false);
      }
      return self;
    },
    renderHtml() {
      const self = this;
      const id = self._id;
      const settings = self.settings;
      const prefix = self.classPrefix;
      let text = self.state.get('text');
      let icon = self.settings.icon, image = '', shortcut = settings.shortcut;
      let url = self.encode(settings.url), iconHtml = '';
      function convertShortcut(shortcut) {
        let i, value, replace = {};
        if (global$8.mac) {
          replace = {
            alt: '&#x2325;',
            ctrl: '&#x2318;',
            shift: '&#x21E7;',
            meta: '&#x2318;'
          };
        } else {
          replace = { meta: 'Ctrl' };
        }
        shortcut = shortcut.split('+');
        for (i = 0; i < shortcut.length; i++) {
          value = replace[shortcut[i].toLowerCase()];
          if (value) {
            shortcut[i] = value;
          }
        }
        return shortcut.join('+');
      }
      function escapeRegExp(str) {
        return str.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
      }
      function markMatches(text) {
        const match = settings.match || '';
        return match ? text.replace(new RegExp(escapeRegExp(match), 'gi'), match => '!mce~match[' + match + ']mce~match!') : text;
      }
      function boldMatches(text) {
        return text.replace(new RegExp(escapeRegExp('!mce~match['), 'g'), '<b>').replace(new RegExp(escapeRegExp(']mce~match!'), 'g'), '</b>');
      }
      if (icon) {
        self.parent().classes.add('menu-has-icons');
      }
      if (settings.image) {
        image = ' style="background-image: url(\'' + settings.image + '\')"';
      }
      if (shortcut) {
        shortcut = convertShortcut(shortcut);
      }
      icon = prefix + 'ico ' + prefix + 'i-' + (self.settings.icon || 'none');
      iconHtml = text !== '-' ? '<i class="' + icon + '"' + image + '></i>\xA0' : '';
      text = boldMatches(self.encode(markMatches(text)));
      url = boldMatches(self.encode(markMatches(url)));
      return '<div id="' + id + '" class="' + self.classes + '" tabindex="-1">' + iconHtml + (text !== '-' ? '<span id="' + id + '-text" class="' + prefix + 'text">' + text + '</span>' : '') + (shortcut ? '<div id="' + id + '-shortcut" class="' + prefix + 'menu-shortcut">' + shortcut + '</div>' : '') + (settings.menu ? '<div class="' + prefix + 'caret"></div>' : '') + (url ? '<div class="' + prefix + 'menu-item-link">' + url + '</div>' : '') + '</div>';
    },
    postRender() {
      const self = this, settings = self.settings;
      let textStyle = settings.textStyle;
      if (typeof textStyle === 'function') {
        textStyle = textStyle.call(this);
      }
      if (textStyle) {
        const textElm = self.getEl('text');
        if (textElm) {
          textElm.setAttribute('style', textStyle);
          self._textStyle = textStyle;
        }
      }
      self.on('mouseenter click', e => {
        if (e.control === self) {
          if (!settings.menu && e.type === 'click') {
            self.fire('select');
            Delay.requestAnimationFrame(() => {
              self.parent().hideAll();
            });
          } else {
            self.showMenu();
            if (e.aria) {
              self.menu.focus(true);
            }
          }
        }
      });
      self._super();
      return self;
    },
    hover() {
      const self = this;
      self.parent().items().each(ctrl => {
        ctrl.classes.remove('selected');
      });
      self.classes.toggle('selected', true);
      return self;
    },
    active(state) {
      toggleTextStyle(this, state);
      if (typeof state !== 'undefined') {
        this.aria('checked', state);
      }
      return this._super(state);
    },
    remove() {
      this._super();
      if (this.menu) {
        this.menu.remove();
      }
    }
  });

  const Radio = Checkbox.extend({
    Defaults: {
      classes: 'radio',
      role: 'radio'
    }
  });

  const ResizeHandle = Widget.extend({
    renderHtml() {
      const self = this, prefix = self.classPrefix;
      self.classes.add('resizehandle');
      if (self.settings.direction === 'both') {
        self.classes.add('resizehandle-both');
      }
      self.canFocus = false;
      return '<div id="' + self._id + '" class="' + self.classes + '">' + '<i class="' + prefix + 'ico ' + prefix + 'i-resize"></i>' + '</div>';
    },
    postRender() {
      const self = this;
      self._super();
      self.resizeDragHelper = new DragHelper(this._id, {
        start() {
          self.fire('ResizeStart');
        },
        drag(e) {
          if (self.settings.direction !== 'both') {
            e.deltaX = 0;
          }
          self.fire('Resize', e);
        },
        stop() {
          self.fire('ResizeEnd');
        }
      });
    },
    remove() {
      if (this.resizeDragHelper) {
        this.resizeDragHelper.destroy();
      }
      return this._super();
    }
  });

  function createOptions(options) {
    let strOptions = '';
    if (options) {
      for (let i = 0; i < options.length; i++) {
        strOptions += '<option value="' + options[i] + '">' + options[i] + '</option>';
      }
    }
    return strOptions;
  }
  const SelectBox = Widget.extend({
    Defaults: {
      classes: 'selectbox',
      role: 'selectbox',
      options: []
    },
    init(settings) {
      const self = this;
      self._super(settings);
      if (self.settings.size) {
        self.size = self.settings.size;
      }
      if (self.settings.options) {
        self._options = self.settings.options;
      }
      self.on('keydown', e => {
        let rootControl;
        if (e.keyCode === 13) {
          e.preventDefault();
          self.parents().reverse().each(ctrl => {
            if (ctrl.toJSON) {
              rootControl = ctrl;
              return false;
            }
          });
          self.fire('submit', { data: rootControl.toJSON() });
        }
      });
    },
    options(state) {
      if (!arguments.length) {
        return this.state.get('options');
      }
      this.state.set('options', state);
      return this;
    },
    renderHtml() {
      const self = this;
      let options, size = '';
      options = createOptions(self._options);
      if (self.size) {
        size = ' size = "' + self.size + '"';
      }
      return '<select id="' + self._id + '" class="' + self.classes + '"' + size + '>' + options + '</select>';
    },
    bindStates() {
      const self = this;
      self.state.on('change:options', e => {
        self.getEl().innerHTML = createOptions(e.value);
      });
      return self._super();
    }
  });

  function constrain(value, minVal, maxVal) {
    if (value < minVal) {
      value = minVal;
    }
    if (value > maxVal) {
      value = maxVal;
    }
    return value;
  }
  function setAriaProp(el, name, value) {
    el.setAttribute('aria-' + name, value);
  }
  function updateSliderHandle(ctrl, value) {
    let maxHandlePos, shortSizeName, sizeName, stylePosName, styleValue, handleEl;
    if (ctrl.settings.orientation === 'v') {
      stylePosName = 'top';
      sizeName = 'height';
      shortSizeName = 'h';
    } else {
      stylePosName = 'left';
      sizeName = 'width';
      shortSizeName = 'w';
    }
    handleEl = ctrl.getEl('handle');
    maxHandlePos = (ctrl.layoutRect()[shortSizeName] || 100) - funcs.getSize(handleEl)[sizeName];
    styleValue = maxHandlePos * ((value - ctrl._minValue) / (ctrl._maxValue - ctrl._minValue)) + 'px';
    handleEl.style[stylePosName] = styleValue;
    handleEl.style.height = ctrl.layoutRect().h + 'px';
    setAriaProp(handleEl, 'valuenow', value);
    setAriaProp(handleEl, 'valuetext', '' + ctrl.settings.previewFilter(value));
    setAriaProp(handleEl, 'valuemin', ctrl._minValue);
    setAriaProp(handleEl, 'valuemax', ctrl._maxValue);
  }
  const Slider = Widget.extend({
    init(settings) {
      const self = this;
      if (!settings.previewFilter) {
        settings.previewFilter = value => Math.round(value * 100) / 100;
      }
      self._super(settings);
      self.classes.add('slider');
      if (settings.orientation === 'v') {
        self.classes.add('vertical');
      }
      self._minValue = $_dt75eqv0jfjm4nio.isNumber(settings.minValue) ? settings.minValue : 0;
      self._maxValue = $_dt75eqv0jfjm4nio.isNumber(settings.maxValue) ? settings.maxValue : 100;
      self._initValue = self.state.get('value');
    },
    renderHtml() {
      const self = this, id = self._id, prefix = self.classPrefix;
      return '<div id="' + id + '" class="' + self.classes + '">' + '<div id="' + id + '-handle" class="' + prefix + 'slider-handle" role="slider" tabindex="-1"></div>' + '</div>';
    },
    reset() {
      this.value(this._initValue).repaint();
    },
    postRender() {
      const self = this;
      let minValue, maxValue, screenCordName, stylePosName, sizeName, shortSizeName;
      function toFraction(min, max, val) {
        return (val + min) / (max - min);
      }
      function fromFraction(min, max, val) {
        return val * (max - min) - min;
      }
      function handleKeyboard(minValue, maxValue) {
        function alter(delta) {
          let value;
          value = self.value();
          value = fromFraction(minValue, maxValue, toFraction(minValue, maxValue, value) + delta * 0.05);
          value = constrain(value, minValue, maxValue);
          self.value(value);
          self.fire('dragstart', { value });
          self.fire('drag', { value });
          self.fire('dragend', { value });
        }
        self.on('keydown', e => {
          switch (e.keyCode) {
          case 37:
          case 38:
            alter(-1);
            break;
          case 39:
          case 40:
            alter(1);
            break;
          }
        });
      }
      function handleDrag(minValue, maxValue, handleEl) {
        let startPos, startHandlePos, maxHandlePos, handlePos, value;
        self._dragHelper = new DragHelper(self._id, {
          handle: self._id + '-handle',
          start(e) {
            startPos = e[screenCordName];
            startHandlePos = parseInt(self.getEl('handle').style[stylePosName], 10);
            maxHandlePos = (self.layoutRect()[shortSizeName] || 100) - funcs.getSize(handleEl)[sizeName];
            self.fire('dragstart', { value });
          },
          drag(e) {
            const delta = e[screenCordName] - startPos;
            handlePos = constrain(startHandlePos + delta, 0, maxHandlePos);
            handleEl.style[stylePosName] = handlePos + 'px';
            value = minValue + handlePos / maxHandlePos * (maxValue - minValue);
            self.value(value);
            self.tooltip().text('' + self.settings.previewFilter(value)).show().moveRel(handleEl, 'bc tc');
            self.fire('drag', { value });
          },
          stop() {
            self.tooltip().hide();
            self.fire('dragend', { value });
          }
        });
      }
      minValue = self._minValue;
      maxValue = self._maxValue;
      if (self.settings.orientation === 'v') {
        screenCordName = 'screenY';
        stylePosName = 'top';
        sizeName = 'height';
        shortSizeName = 'h';
      } else {
        screenCordName = 'screenX';
        stylePosName = 'left';
        sizeName = 'width';
        shortSizeName = 'w';
      }
      self._super();
      handleKeyboard(minValue, maxValue);
      handleDrag(minValue, maxValue, self.getEl('handle'));
    },
    repaint() {
      this._super();
      updateSliderHandle(this, this.value());
    },
    bindStates() {
      const self = this;
      self.state.on('change:value', e => {
        updateSliderHandle(self, e.value);
      });
      return self._super();
    }
  });

  const Spacer = Widget.extend({
    renderHtml() {
      const self = this;
      self.classes.add('spacer');
      self.canFocus = false;
      return '<div id="' + self._id + '" class="' + self.classes + '"></div>';
    }
  });

  const SplitButton = MenuButton.extend({
    Defaults: {
      classes: 'widget btn splitbtn',
      role: 'button'
    },
    repaint() {
      const self = this;
      const elm = self.getEl();
      const rect = self.layoutRect();
      let mainButtonElm, menuButtonElm;
      self._super();
      mainButtonElm = elm.firstChild;
      menuButtonElm = elm.lastChild;
      global$9(mainButtonElm).css({
        width: rect.w - funcs.getSize(menuButtonElm).width,
        height: rect.h - 2
      });
      global$9(menuButtonElm).css({ height: rect.h - 2 });
      return self;
    },
    activeMenu(state) {
      const self = this;
      global$9(self.getEl().lastChild).toggleClass(self.classPrefix + 'active', state);
    },
    renderHtml() {
      const self = this;
      const id = self._id;
      const prefix = self.classPrefix;
      let image;
      let icon = self.state.get('icon');
      const text = self.state.get('text');
      const settings = self.settings;
      let textHtml = '', ariaPressed;
      image = settings.image;
      if (image) {
        icon = 'none';
        if (typeof image !== 'string') {
          image = window.getSelection ? image[0] : image[1];
        }
        image = ' style="background-image: url(\'' + image + '\')"';
      } else {
        image = '';
      }
      icon = settings.icon ? prefix + 'ico ' + prefix + 'i-' + icon : '';
      if (text) {
        self.classes.add('btn-has-text');
        textHtml = '<span class="' + prefix + 'txt">' + self.encode(text) + '</span>';
      }
      ariaPressed = typeof settings.active === 'boolean' ? ' aria-pressed="' + settings.active + '"' : '';
      return '<div id="' + id + '" class="' + self.classes + '" role="button"' + ariaPressed + ' tabindex="-1">' + '<button type="button" hidefocus="1" tabindex="-1">' + (icon ? '<i class="' + icon + '"' + image + '></i>' : '') + textHtml + '</button>' + '<button type="button" class="' + prefix + 'open" hidefocus="1" tabindex="-1">' + (self._menuBtnText ? (icon ? '\xA0' : '') + self._menuBtnText : '') + ' <i class="' + prefix + 'caret"></i>' + '</button>' + '</div>';
    },
    postRender() {
      const self = this, onClickHandler = self.settings.onclick;
      self.on('click', function (e) {
        let node = e.target;
        if (e.control === this) {
          while (node) {
            if (e.aria && e.aria.key !== 'down' || node.nodeName === 'BUTTON' && node.className.indexOf('open') === -1) {
              e.stopImmediatePropagation();
              if (onClickHandler) {
                onClickHandler.call(this, e);
              }
              return;
            }
            node = node.parentNode;
          }
        }
      });
      delete self.settings.onclick;
      return self._super();
    }
  });

  const StackLayout = FlowLayout.extend({
    Defaults: {
      containerClass: 'stack-layout',
      controlClass: 'stack-layout-item',
      endClass: 'break'
    },
    isNative() {
      return true;
    }
  });

  const TabPanel = Panel.extend({
    Defaults: {
      layout: 'absolute',
      defaults: { type: 'panel' }
    },
    activateTab(idx) {
      let activeTabElm;
      if (this.activeTabId) {
        activeTabElm = this.getEl(this.activeTabId);
        global$9(activeTabElm).removeClass(this.classPrefix + 'active');
        activeTabElm.setAttribute('aria-selected', 'false');
      }
      this.activeTabId = 't' + idx;
      activeTabElm = this.getEl('t' + idx);
      activeTabElm.setAttribute('aria-selected', 'true');
      global$9(activeTabElm).addClass(this.classPrefix + 'active');
      this.items()[idx].show().fire('showtab');
      this.reflow();
      this.items().each((item, i) => {
        if (idx !== i) {
          item.hide();
        }
      });
    },
    renderHtml() {
      const self = this;
      const layout = self._layout;
      let tabsHtml = '';
      const prefix = self.classPrefix;
      self.preRender();
      layout.preRender(self);
      self.items().each((ctrl, i) => {
        const id = self._id + '-t' + i;
        ctrl.aria('role', 'tabpanel');
        ctrl.aria('labelledby', id);
        tabsHtml += '<div id="' + id + '" class="' + prefix + 'tab" ' + 'unselectable="on" role="tab" aria-controls="' + ctrl._id + '" aria-selected="false" tabIndex="-1">' + self.encode(ctrl.settings.title) + '</div>';
      });
      return '<div id="' + self._id + '" class="' + self.classes + '" hidefocus="1" tabindex="-1">' + '<div id="' + self._id + '-head" class="' + prefix + 'tabs" role="tablist">' + tabsHtml + '</div>' + '<div id="' + self._id + '-body" class="' + self.bodyClasses + '">' + layout.renderHtml(self) + '</div>' + '</div>';
    },
    postRender() {
      const self = this;
      self._super();
      self.settings.activeTab = self.settings.activeTab || 0;
      self.activateTab(self.settings.activeTab);
      this.on('click', e => {
        const targetParent = e.target.parentNode;
        if (targetParent && targetParent.id === self._id + '-head') {
          let i = targetParent.childNodes.length;
          while (i--) {
            if (targetParent.childNodes[i] === e.target) {
              self.activateTab(i);
            }
          }
        }
      });
    },
    initLayoutRect() {
      const self = this;
      let rect, minW, minH;
      minW = funcs.getSize(self.getEl('head')).width;
      minW = minW < 0 ? 0 : minW;
      minH = 0;
      self.items().each(item => {
        minW = Math.max(minW, item.layoutRect().minW);
        minH = Math.max(minH, item.layoutRect().minH);
      });
      self.items().each(ctrl => {
        ctrl.settings.x = 0;
        ctrl.settings.y = 0;
        ctrl.settings.w = minW;
        ctrl.settings.h = minH;
        ctrl.layoutRect({
          x: 0,
          y: 0,
          w: minW,
          h: minH
        });
      });
      const headH = funcs.getSize(self.getEl('head')).height;
      self.settings.minWidth = minW;
      self.settings.minHeight = minH + headH;
      rect = self._super();
      rect.deltaH += headH;
      rect.innerH = rect.h - rect.deltaH;
      return rect;
    }
  });

  const TextBox = Widget.extend({
    init(settings) {
      const self = this;
      self._super(settings);
      self.classes.add('textbox');
      if (settings.multiline) {
        self.classes.add('multiline');
      } else {
        self.on('keydown', e => {
          let rootControl;
          if (e.keyCode === 13) {
            e.preventDefault();
            self.parents().reverse().each(ctrl => {
              if (ctrl.toJSON) {
                rootControl = ctrl;
                return false;
              }
            });
            self.fire('submit', { data: rootControl.toJSON() });
          }
        });
        self.on('keyup', e => {
          self.state.set('value', e.target.value);
        });
      }
    },
    repaint() {
      const self = this;
      let style, rect, borderBox, borderW, borderH = 0, lastRepaintRect;
      style = self.getEl().style;
      rect = self._layoutRect;
      lastRepaintRect = self._lastRepaintRect || {};
      const doc = document;
      if (!self.settings.multiline && doc.all && (!doc.documentMode || doc.documentMode <= 8)) {
        style.lineHeight = rect.h - borderH + 'px';
      }
      borderBox = self.borderBox;
      borderW = borderBox.left + borderBox.right + 8;
      borderH = borderBox.top + borderBox.bottom + (self.settings.multiline ? 8 : 0);
      if (rect.x !== lastRepaintRect.x) {
        style.left = rect.x + 'px';
        lastRepaintRect.x = rect.x;
      }
      if (rect.y !== lastRepaintRect.y) {
        style.top = rect.y + 'px';
        lastRepaintRect.y = rect.y;
      }
      if (rect.w !== lastRepaintRect.w) {
        style.width = rect.w - borderW + 'px';
        lastRepaintRect.w = rect.w;
      }
      if (rect.h !== lastRepaintRect.h) {
        style.height = rect.h - borderH + 'px';
        lastRepaintRect.h = rect.h;
      }
      self._lastRepaintRect = lastRepaintRect;
      self.fire('repaint', {}, false);
      return self;
    },
    renderHtml() {
      const self = this;
      const settings = self.settings;
      let attrs, elm;
      attrs = {
        id: self._id,
        hidefocus: '1'
      };
      Tools.each([
        'rows',
        'spellcheck',
        'maxLength',
        'size',
        'readonly',
        'min',
        'max',
        'step',
        'list',
        'pattern',
        'placeholder',
        'required',
        'multiple'
      ], name => {
        attrs[name] = settings[name];
      });
      if (self.disabled()) {
        attrs.disabled = 'disabled';
      }
      if (settings.subtype) {
        attrs.type = settings.subtype;
      }
      elm = funcs.create(settings.multiline ? 'textarea' : 'input', attrs);
      elm.value = self.state.get('value');
      elm.className = self.classes;
      return elm.outerHTML;
    },
    value(value) {
      if (arguments.length) {
        this.state.set('value', value);
        return this;
      }
      if (this.state.get('rendered')) {
        this.state.set('value', this.getEl().value);
      }
      return this.state.get('value');
    },
    postRender() {
      const self = this;
      self.getEl().value = self.state.get('value');
      self._super();
      self.$el.on('change', e => {
        self.state.set('value', e.target.value);
        self.fire('change', e);
      });
    },
    bindStates() {
      const self = this;
      self.state.on('change:value', e => {
        if (self.getEl().value !== e.value) {
          self.getEl().value = e.value;
        }
      });
      self.state.on('change:disabled', e => {
        self.getEl().disabled = e.value;
      });
      return self._super();
    },
    remove() {
      this.$el.off();
      this._super();
    }
  });

  const getApi = () => ({
    Selector,
    Collection: Collection$2,
    ReflowQueue: $_8awxqftqjfjm4ne5,
    Control: Control$1,
    Factory: Factory,
    KeyboardNavigation,
    Container,
    DragHelper,
    Scrollable: $_rcleftsjfjm4nec,
    Panel,
    Movable: $_1ttsxttdjfjm4ncj,
    Resizable: $_c9f860tujfjm4nev,
    FloatPanel,
    Window,
    MessageBox,
    Tooltip,
    Widget,
    Progress,
    Notification,
    Layout,
    AbsoluteLayout,
    Button,
    ButtonGroup,
    Checkbox,
    ComboBox,
    ColorBox,
    PanelButton,
    ColorButton,
    ColorPicker,
    Path,
    ElementPath,
    FormItem,
    Form,
    FieldSet,
    FilePicker,
    FitLayout,
    FlexLayout,
    FlowLayout,
    FormatControls: $_bce66dvrjfjm4nki,
    GridLayout,
    Iframe: Iframe$1,
    InfoBox,
    Label,
    Toolbar: Toolbar$1,
    MenuBar,
    MenuButton,
    MenuItem,
    Throbber,
    Menu,
    ListBox,
    Radio,
    ResizeHandle,
    SelectBox,
    Slider,
    Spacer,
    SplitButton,
    StackLayout,
    TabPanel,
    TextBox,
    DropZone,
    BrowseButton
  });
  const appendTo = target => {
    if (target.ui) {
      Tools.each(getApi(), (ref, key) => {
        target.ui[key] = ref;
      });
    } else {
      target.ui = getApi();
    }
  };
  const registerToFactory = () => {
    Tools.each(getApi(), (ref, key) => {
      Factory.add(key, ref);
    });
  };
  const Api = {
    appendTo,
    registerToFactory
  };

  Api.registerToFactory();
  Api.appendTo(window.tinymce ? window.tinymce : {});
  $_bce66dvrjfjm4nki.setup(editor);
  new NodeChangedSelector(editor)
  InsertEmptyParagrash.register(editor)
  KeyEvent.register(editor)
  TiaraClick.register(editor)
  return $_dtr9igsljfjm4nah.get(editor);
}

export default classic;
